/**
 * Timestamp.java
 *
 * Created on 7. Februar 2007, 11:21
 *
 */
package de.retrobib.JSPTags;

import java.util.GregorianCalendar;
import java.util.Locale;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.SimpleTagSupport;
import org.apache.log4j.Logger;
import org.ca.utils.misc.DateUtil;

/**
 * Das Tag Timestamp liefert einen <i>friendlyString</i> eines Zeitstempels 
 * (Zeitstempel = Millisekunden seit 1.1.1970) zurück.
 * 
 * @author caschoff
 * @version 1.0
 *
 */
public class Timestamp extends SimpleTagSupport {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Timestamp.class);
    /**
     * Der Zeitstempel.
     */
    private long time;

    /**
     * Called by the container to invoke this tag.
     * @throws JspException Ein JSP-Fehler ist aufgetreten.
     */
    @Override
    public void doTag()
            throws JspException {
        JspWriter out = getJspContext().getOut();

        try {
            GregorianCalendar gc = new GregorianCalendar();
            gc.setTimeInMillis(time);
            out.print(DateUtil.getFormattedDateString(gc, Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT));
        } catch (java.io.IOException ex) {
            logger.error("Exception in timestamp", ex);
            throw new JspException(ex.getMessage());
        }
    }

    /*
     * Getter/Setter - Tags brauchen nur Setter
     */
    /**
     * Den Zeitstempel setzten.
     * @param time Der Zeitstempel.
     */
    public void setTime(long time) {
        this.time = time;
    }
}
