/**
 * GehezuAction.java
 *
 * Created on 15. August 2007, 09:19
 *
 */
package de.retrobib.actions;

import de.retrobib.Constants.ViewNames;
import de.retrobib.db.model.Seite;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.hibernate.Query;
import org.hibernate.Session;

/**
 * Action für die Seite "gehezu.html".
 *
 * 24.11.2007: 1.1 - Querys auf HQL mit .set-Methoden umgeschrieben.
 *
 * @author caschoff
 * @version 1.1
 *
 */
public class GehezuAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(GehezuAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {

        // aus gehezu.html
        if (request.getParameter("goto") != null) {
            String werk = request.getParameter("werk");
            request.setAttribute("werk", werk);
            String band = request.getParameter("band");
            request.setAttribute("band", band);
            String seite = request.getParameter("seite");
            request.setAttribute("seite", seite);

            if ((seite != null) && (seite.trim().length() != 0)) {
                //
                // QueryString aufbauen
                //
                String queryString = "from Seite s where s.pagenumber like :pagenumber and s.deleted=false";
                if ((band != null) && (band.trim().length() != 0)) {
                    queryString += " and s.teilid.bandid.name like :bandname";
                    request.setAttribute("band", band);
                }
                if ((werk != null) && (werk.trim().length() != 0)) {
                    queryString += " and s.teilid.bandid.werkid.name like :werkname";
                }
                queryString += " order by s.sort";
                logger.debug(queryString);

                Query query = hiberSession.createQuery(queryString).setString("pagenumber", "%" + seite + "%");

                //
                // Parameter setzen
                //
                if ((band != null) && (band.trim().length() != 0)) {
                    query = query.setString("bandname", "%" + band + "%");
                }
                if ((werk != null) && (werk.trim().length() != 0)) {
                    query = query.setString("werkname", "%" + werk + "%");
                }
                @SuppressWarnings("unchecked")
                List<Seite> l = (List<Seite>) query.list();

                if (l.size() == 0) {
                    request.setAttribute("hint", "Es wurde keine Seite gefunden, die den Kriterien entsprechen.");
                } else if (l.size() == 1) {
                    Seite s = l.get(0);
                    response.sendRedirect("/retrobib/seite.html?id=" + s.getId());
                    return null;
                } else if (l.size() > 100) {
                    request.setAttribute("hint", "Es wurden " + l.size() + " Seiten gefunden, die den Kriterien entsprechen. Bitte schränken Sie die Angaben weiter ein.");
                } else {
                    request.setAttribute("hint", l.size() + " Seiten entsprechen Ihren Kriterien. Bitte wählen Sie die passende aus.");
                    request.setAttribute("seiten", l);
                    return mapping.findForward(ViewNames.seitenauswahlView);
                }
            } else {
                request.setAttribute("hint", "Sie müssen die Seite angeben.");
            }
        }

        // aus suche.html (erweiterte Suche für Korrektoren)
        if (request.getParameter("extsearch") != null) {
            String werk = request.getParameter("werk");
            String band = request.getParameter("band");
            String seite = request.getParameter("seite");
            String status = request.getParameter("status");
            String email = request.getParameter("email");
            String nokeywords = request.getParameter("nokeywords");
            String noseewords = request.getParameter("noseewords");

            //
            // QueryString aufbauen
            //
            String queryString = "from Seite s where s.deleted=false";
            if (werk.trim().length() != 0) {
                queryString += " and s.teilid.bandid.werkid.name like :werkname";
            }
            if (band.trim().length() != 0) {
                queryString += " and s.teilid.bandid.name like :bandname";
            }
            if (seite.trim().length() != 0) {
                queryString += " and s.pagenumber like :pagenumber";
            }
            if ((nokeywords == null) && (noseewords == null) && (!status.equals("-1"))) {
                queryString += " and s.status=:status";
            }
            if (email.trim().length() != 0) {
                queryString += " and s.accountid.email like :email";
            }
            if ((nokeywords != null) && (nokeywords.trim().length() != 0)) {
                queryString += " and s.status>0 and s.nokeywords<>true and not exists (select id from Schlagwort sc where sc.deleted<>true and sc.seiteid=s.id)";
            }
            if ((noseewords != null) && (noseewords.trim().length() != 0)) {
                queryString += " and s.status>0 and s.noseewords<>true and not exists (select id from Siehewort sw where sw.deleted<>true and sw.seiteid=s.id)";
            }
            queryString += " order by s.sort";
            logger.debug(queryString);

            Query query = hiberSession.createQuery(queryString);

            //
            // Parameter setzen
            //
            if (werk.trim().length() != 0) {
                query = query.setString("werkname", "%" + werk + "%");
            }
            if (band.trim().length() != 0) {
                query = query.setString("bandname", "%" + band + "%");
            }
            if (seite.trim().length() != 0) {
                query = query.setString("pagenumber", "%" + seite + "%");
            }
            if ((nokeywords == null) && (noseewords == null) && (!status.equals("-1"))) {
                query = query.setInteger("status", Integer.parseInt(status));
            }
            if (email.trim().length() != 0) {
                query = query.setString("email", "%" + email + "%");
            }
            @SuppressWarnings("unchecked")
            List<Seite> l = (List<Seite>) query.list();

            if (l.size() == 0) {
                request.setAttribute("hint", "Es wurde keine Seite gefunden, die den Kriterien entsprechen.");
            } else if (l.size() > 1000) {
                request.setAttribute("hint", "Es wurden mehr als 1000 (" + l.size() + ") Seiten gefunden, die den Kriterien entsprechen. Bitte schränken Sie die Angaben weiter ein.");
            } else {
                request.setAttribute("hint", l.size() + " Seiten entsprechen Ihren Kriterien. Bitte wählen Sie die passende aus.");
                request.setAttribute("seiten", l);
            }

            return mapping.findForward(ViewNames.seitenauswahlView);
        }

        return mapping.findForward(ViewNames.gehezuView);
    }
}
