/**
 * LoginAction.java
 *
 * Created on 15. August 2007, 11:18
 *
 */
package de.retrobib.actions;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.Constants.ViewNames;
import de.retrobib.db.logic.AccountLogic;
import de.retrobib.db.logic.AccountsettingLogic;
import de.retrobib.db.logic.GlobalsettingLogic;
import de.retrobib.db.model.Account;
import de.retrobib.db.model.Accountsetting;
import de.retrobib.db.model.Seite;
import de.retrobib.utils.Mailer;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.DateUtil;
import org.ca.utils.misc.MD5Util;
import org.ca.utils.misc.PasswordUtil;
import org.ca.utils.misc.RequestUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "login.html".
 *
 * 21.11.2007: 1.1 - AccountSetting (LastLogin) hinzugefügt.
 *
 * @author caschoff
 * @version 1.1
 *
 */
public class LoginAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(LoginAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {

        // default-View
        String forwardName = ViewNames.loginView;

        // Focus auf E-Mail setzen
        request.setAttribute("focus", "email");

        // Testen, ob irgendein Account in der Datenbank ist,
        // bei 0 Accounts aus alter Datenbank migrieren...
        if ((Long) hiberSession.getNamedQuery("Account.count").uniqueResult() == 0) {
            logger.info("Kein Account da, starte Migration...");
            logger.info("... Migration der Accounts beendet.");
        }

        // Login-Daten aus Formular bekommen?
        if (RequestUtil.commandIs("login", request)) {
            String eMail = request.getParameter("email").toLowerCase();
            String password = request.getParameter("password");

            if ((eMail != null) && (eMail.trim().length() != 0) && (password != null) && (password.trim().length() != 0)) {
                Long count = (Long) hiberSession.getNamedQuery("Account.validation").setString("email", eMail).setString("password", MD5Util.plainStringToMD5(password)).uniqueResult();
                logger.debug("Login-Anfrage (email=" + eMail + "/pass=" + MD5Util.plainStringToMD5(password) + ")");
                logger.debug("Treffer für dieses Tupel: " + count);

                if (count == 1) {
                    logger.debug("Login o.k....");
                    // Bingo
                    Account account = (Account) hiberSession.getNamedQuery("Account by data").setString("email", eMail).setString("password", MD5Util.plainStringToMD5(password)).list().get(0);
                    AccountsettingLogic.saveAccountsetting(account, hiberSession, MiscConstants.accountSettingLastLogin, "" + System.currentTimeMillis());
                    Iterator<Accountsetting> it = account.getAccountsettingCollection().iterator();
                    // Lazy-Loading auflösen...
                    while (it.hasNext()) {
                        it.next();
                    }
                    logger.debug(account);
                    // Eventuell vorhandene One-Time-Pads verwerfen.
                    AccountsettingLogic.deleteAccountsetting(account, hiberSession, MiscConstants.accountSettingOnetimepad);
                    // Alte Session verwerfen, verhindert Session-Übernahmen.
                    httpSession = request.getSession(true);
                    httpSession.invalidate();
                    // Neue Session erstellen.
                    httpSession = request.getSession(true);
                    // Session ist unbegrenzt gültig.
                    httpSession.setMaxInactiveInterval(-1);
                    httpSession.setAttribute(MiscConstants.account, account);
                    // Ausgeklinkte Seiten anhängen...
                    @SuppressWarnings("unchecked")
                    List<Seite> outPages = (List<Seite>) hiberSession.getNamedQuery("ausgeklinkte Seiten").setLong("accountid", getAccountFromSession(httpSession).getId()).list();
                    request.setAttribute("outPages", outPages);
                    // andere Daten anhängen...
                    if (GlobalsettingLogic.getGlobalsetting(hiberSession, MiscConstants.globalSettingLastLuceneUpdate) != null) {
                        request.setAttribute("lastLuceneUpdate", DateUtil.getFormattedDateString(Long.parseLong(GlobalsettingLogic.getGlobalsetting(hiberSession, MiscConstants.globalSettingLastLuceneUpdate).getValue()), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT));
                    }
                    // zur Verwaltung schicken...
                    forwardName = ViewNames.verwaltungView;
                } else {
                    logger.debug("Login fehlgeschlagen...");
                }
            }
        }

        // Logout bekommen?
        if (RequestUtil.commandIs("logout", request)) {
            // Alte Session verwerfen
            httpSession = request.getSession(true);
            httpSession.invalidate();
            forwardName = ViewNames.indexView;
        }

        // Passwort zurücksetzen?
        if (RequestUtil.commandIs("resetpass", request)) {
            String eMail = request.getParameter("email");
            @SuppressWarnings("unchecked")
            List<Account> accounts = (List<Account>) hiberSession.getNamedQuery("Account by email").setString("email", eMail.toLowerCase()).list();
            if (accounts.size() == 1) {
                Account account = accounts.get(0);
                // Eventuell vorhandene One-Time-Pads verwerfen.
                AccountsettingLogic.deleteAccountsetting(account, hiberSession, MiscConstants.accountSettingOnetimepad);
                // One-Time-Pad anlegen...
                String oneTimePad = "" + System.currentTimeMillis();
                AccountsettingLogic.saveAccountsetting(account, hiberSession, MiscConstants.accountSettingOnetimepad, oneTimePad);

                // eMail verschicken...
                StringBuilder mailText = new StringBuilder();
                mailText.append("Zurücksetzen des Passwortes:\n\n");
                mailText.append("Wenn Sie auf den folgenden Link klicken, wird Ihr Passwort in der Retro-Bibliothek zurückgesetzt. ");
                mailText.append("Sie erhalten das neue Passwort dann wiederum per eMail zugeschickt.\n");
                mailText.append("Wenn Ihnen Ihr Passwort zwischenzeitlich eingefallen ist, können Sie es wie gewohnt verwenden.\n");
                mailText.append("Wenn Sie nicht wissen, warum Sie diese eMail erhalten haben, können Sie sie löschen.\n\n");
                mailText.append(MiscConstants.baseURL + "/retrobib/login.html?otp=" + oneTimePad + "\n\n");
                mailText.append("Diese eMail wurde automatisch generiert.\n");
                mailText.append("Bei Fragen wenden Sie sich an Christian Aschoff (caschoff@mac.com)\n");

                new Mailer().send("Passwortrücksetzung in der retro|bib", mailText.toString(), account.getEmail(), null);

                // Nachricht
                request.setAttribute("resetmessage", "Falls die eMail-Adresse dem System bekannt war, wurde eine eMail mit weiteren Anweisungen an die eMail-Adresse verschickt.");
            }
        }

        // One-Time-Pad bekommen?
        if (request.getParameter("otp") != null) {
            @SuppressWarnings("unchecked")
            List<Accountsetting> otps = (List<Accountsetting>) hiberSession.getNamedQuery("Accountsetting by otp").setString("otp", request.getParameter("otp")).list();
            if (otps.size() == 1) {
                Accountsetting accountsetting = otps.get(0);
                String newPass = PasswordUtil.getTempPassword();
                Account account = accountsetting.getAccountid();
                account.setPassword(MD5Util.plainStringToMD5(newPass));
                AccountLogic.updateAccount(account, hiberSession, true, true);
                AccountsettingLogic.deleteAccountsetting(account, hiberSession, MiscConstants.accountSettingOnetimepad);

                // eMail verschicken...
                StringBuilder mailText = new StringBuilder();
                mailText.append("Zurücksetzen des Passwortes:\n\n");
                mailText.append("Ihre Zugansgdaten lauten:\n\n");
                mailText.append("eMail = " + account.getEmail() + "\n");
                mailText.append("Passwort = " + newPass + "\n\n");
                mailText.append("Sie können Ihr Passwort wie gewohnt in der Veraltung ändern.\n\n");
                mailText.append("\n\n");
                mailText.append("Diese eMail wurde automatisch generiert.\n");
                mailText.append("Bei Fragen wenden Sie sich an Christian Aschoff (caschoff@mac.com)\n");

                new Mailer().send("Neues Passwort in der retro|bib", mailText.toString(), account.getEmail(), null);

                // Nachricht
                request.setAttribute("resetmessage", "Ihr neues Passwort an Ihre eMail-Adresse verschickt.");
            }
        }

        return mapping.findForward(forwardName);
    }
}
