/**
 * RetrobibAction.java
 *
 * Created on 21. August 2007, 10:52
 *
 */
package de.retrobib.actions;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.Constants.RunningTasks;
import de.retrobib.Constants.ViewNames;
import de.retrobib.db.model.Account;
import de.retrobib.db.model.Seite;
import de.retrobib.listeners.SessionManager;
import de.retrobib.lucene.Sherlock;
import de.retrobib.utils.Mailer;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.DateUtil;
import org.ca.utils.misc.DebugUtil;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

/**
 * Action als Grundlage für andere Actions mit ein paar häufig benutzten
 * Funktionalitäten.
 *
 * 27.11.2007: 1.1 - Methode filterHttpSpam, isRequestParameterEmpty hinzugefügt.
 *
 * @author caschoff
 * @version 1.1
 */
public abstract class RetrobibAction extends Action {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(RetrobibAction.class);
    /**
     * Der Author der Seite für die Meta-Tags. Default=Christian Aschoff, kann
     * in den abgeleiteten Actions überschrieben werden.
     */
    protected String author = "Christian Aschoff";

    /**
     * This is the action called from the Struts framework.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @return The ActionForward for the view.
     * @throws java.lang.Exception In case of an unprocessed error.
     */
    @Override
    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        // Der Forwarder zur View.
        ActionForward forward = null;

        // Zeitmessung, und...
        long timestampStart = System.currentTimeMillis();
        // Nachrichten sollten alle Actions haben.
        ArrayList<Message> messages = new ArrayList<Message>();

        // Hibernate-Sessions holen...
        Session hiberSession = ((SessionFactory) servlet.getServletContext().getAttribute(MiscConstants.retrobibFactory)).openSession();
        // Request einrichten und Http-Session holen...
        request.setCharacterEncoding("UTF-8");
        HttpSession httpSession = request.getSession();

        // per Default das Suchfeld fokussieren...
        request.setAttribute("focus", "suchtext");

        try {
            // Aufruf für abgeleitete Klassen.
            forward = executeAction(mapping, form, request, response, messages, hiberSession, httpSession);

            // Anzahl der Korrektoren anhängen.
            long korrektorcount = (Long) hiberSession.getNamedQuery("Account.count").uniqueResult();
            request.setAttribute("korrektorCount", (korrektorcount - 2));
        } catch (Throwable ex) {
            logger.error("Exception in Action", ex);
            // Mail Error & Request...
            StringBuilder mailContent = new StringBuilder();
            mailContent.append("StackTrace:\n");
            mailContent.append(DebugUtil.getStackTrace(ex));
            mailContent.append("\nRequest:\n");
            mailContent.append(DebugUtil.getRequestAsString(request));
            new Mailer().send("Fehler in der retro-bib: " + ex.getMessage(), mailContent.toString(), MiscConstants.adminMail, null);
            forward = mapping.findForward(ViewNames.errorView);
        } finally {
            hiberSession.close();
        }

        // Jede Action sollte Nachrichten anfuegen...
        request.setAttribute("messages", messages);

        // die letzten Suchen
        request.setAttribute("lastQuerys", Sherlock.getLastQuerys());

        // Benutzer
        request.setAttribute("usercount", SessionManager.getSessioncount());
        request.setAttribute("registeredusercount", SessionManager.getRegistereduserscount());

        // Globale Systemnachrichten für Alle
        request.setAttribute("globalMessages", getGlobalMessages());
        // Globale Systemnachrichten für Korrektoren
        request.setAttribute("globalKorrektorMessages", getGlobalKorrektorMessages());

        // meta-Tags setzten
        request.setAttribute("author", author);

        StringBuilder loggerInfo = new StringBuilder();
        loggerInfo.append("Executed Action for ");
        loggerInfo.append(request.getRemoteAddr());
        loggerInfo.append("<->");
        loggerInfo.append(request.getRequestURL());
        loggerInfo.append(" in ");
        loggerInfo.append((System.currentTimeMillis() - timestampStart));
        loggerInfo.append("ms, referer was '");
        loggerInfo.append(request.getHeader("Referer"));
        loggerInfo.append("'.");
        logger.info(loggerInfo.toString());

        return forward;
    }

    // Globale Systemnachrichten für Alle
    /**
     * Die globalen Systemnachrichten bekommen.
     *
     * @return Die globalen Systemnachrichten.
     */
    public ArrayList<Message> getGlobalMessages() {
        ArrayList<Message> globalMessages = new ArrayList<Message>();

        if (RunningTasks.luceneRunning) {
            globalMessages.add(new Message("Zur Zeit wird der Volltextindex aktualisiert. Sie erhalten daher bei Suchen nicht die volle Anzahl an Treffern. Die Aktualisierung dauert typischerweise wenige Minuten.", Message.INFO));
        }

        return globalMessages;
    }

    // Globale Systemnachrichten für Korrektoren.
    /**
     * Die globalen Systemnachrichten für Korrektoren bekommen.
     * @return Die globalen Systemnachrichten für Korrektoren.
     */
    public ArrayList<Message> getGlobalKorrektorMessages() {
        ArrayList<Message> globalKorrektorMessages = new ArrayList<Message>();

        if (RunningTasks.renumberRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft eine Reorganisation der Sortierungen aller Objekte. Bitte ändern Sie solange keine Meta-Daten. Texte aus-/einklinken ist kein Problem.", Message.INFO));
        }

        if (RunningTasks.importAccountsRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Accounts.", Message.INFO));
        }

        if (RunningTasks.importWerkeRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Werke.", Message.INFO));
        }

        if (RunningTasks.importBaendeRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Bände.", Message.INFO));
        }

        if (RunningTasks.importSeitenRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Seiten.", Message.INFO));
        }

        if (RunningTasks.importTexteRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Texte.", Message.INFO));
        }

        if (RunningTasks.importFaksimilesRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Faksimiles.", Message.INFO));
        }

        if (RunningTasks.importHiresfaksimilesRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Hires-Faksimiles.", Message.INFO));
        }

        if (RunningTasks.importSchlagworteRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Schlagwörter.", Message.INFO));
        }

        if (RunningTasks.importVerweiseRunning) {
            globalKorrektorMessages.add(new Message("Zur Zeit läuft ein Import der Verweise.", Message.INFO));
        }

        return globalKorrektorMessages;
    }

    /**
     * Holt den aktiven Account aus der Session.
     *
     * @param session Die Session.
     * @return Der Account aus der Session oder null, falls kein Account mit der Session assocciert ist.
     */
    protected Account getAccountFromSession(HttpSession session) {
        Account account = null;

        try {
            account = (Account) session.getAttribute(MiscConstants.account);
        } catch (Exception ex) {
            logger.debug("Absichtliche Exception. Kein Problem.");
        }

        return account;
    }

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected abstract ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception;

    /**
     * Http-Spam aus Parametern neutralisieren.<br />
     * Dazu werden einfach die Zeichen ;, > und < durch Blanks ersetzt.
     *
     * @param what Der zu filternde String.
     * @return Der gefilterte String.
     */
    protected String filterHttpSpam(String what) {
        StringBuilder ret = new StringBuilder();

        for (int i = 0; i < what.length(); i++) {
            switch (what.charAt(i)) {
                case ';':
                case '>':
                case '<':
                    ret.append(' ');
                    break;
                default:
                    ret.append(what.charAt(i));
            }
        }

        return ret.toString();
    }

    /**
     * Schreibt den <code>HttpServletRequest</code> in das Logfile.<br />
     * Für Fehlersuchen und Analysen des Requests.
     *
     * @param request Der Request.
     */
    protected void writeRequestToLog(HttpServletRequest request) {
        logger.debug(DebugUtil.getRequestAsString(request));
    }

    /**
     * Ein Backup der Seite per Mail senden.
     *
     * @param account Der initiierende Account.
     * @param seite Die Seite.
     * @param oldText Der alte Text.
     * @param newText Der neue Text.
     * @throws java.lang.Exception In einem Fehlerfall.
     */
    protected void sendBackupMail(Account account, Seite seite, String oldText, String newText)
            throws Exception {
        String subject = "r|b Backupmail: " + seite.getTeilid().getBandid().getWerkid().getShortname() + " Seite Nr. " + seite.getPagenumber();

        StringBuilder mailText = new StringBuilder();
        mailText.append("Werk: ");
        mailText.append(seite.getTeilid().getBandid().getWerkid().getName());
        mailText.append("\n");

        mailText.append("Band: ");
        mailText.append(seite.getTeilid().getBandid().getName());
        mailText.append("\n");

        mailText.append("Teil: ");
        mailText.append(seite.getTeilid().getName());
        mailText.append("\n");

        mailText.append("Seite: Nr. '");
        mailText.append(seite.getPagenumber());
        mailText.append("' von '");
        mailText.append(seite.getStartword());
        mailText.append("' bis '");
        mailText.append(seite.getEndword());
        mailText.append("'\n");

        mailText.append("Bearbeiter: ");
        mailText.append(account.getEmail());
        mailText.append("\n");

        mailText.append("Timestamp: ");
        mailText.append(DateUtil.getFormattedDateString(seite.getTimestamp(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT));
        mailText.append("\n");

        mailText.append("\n");

        mailText.append("NEU ---------------------------------------------------------\n");
        mailText.append(newText);
        mailText.append("\nENDE NEU ----------------------------------------------------\n");

        mailText.append("ALT ---------------------------------------------------------\n");
        mailText.append(oldText);
        mailText.append("\nENDE ALT ----------------------------------------------------\n");

        new Mailer().send(subject, mailText.toString(), MiscConstants.adminMail, null);
        if (account.getBackupmail()) {
            new Mailer().send(subject, mailText.toString(), account.getEmail(), null);
        }
    }
}
