/*
 * SchlagwortAction.java
 *
 * Created on 30. August 2007, 10:31
 *
 */
package de.retrobib.actions;

import de.retrobib.Constants.ViewNames;
import de.retrobib.db.model.Schlagwort;
import de.retrobib.db.model.Seite;
import de.retrobib.webelements.Message;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.RequestUtil;
import org.ca.utils.misc.TextUtil;
import org.hibernate.Query;
import org.hibernate.Session;

/**
 * Action für die Seite "schlagwort.html".
 *
 * 21.11.2007: 1.1 - Weitere URL-Variablen hinzugefügt.
 * 24.11.2007: 1.2 - Query auf HQL mit .set-Methoden umgeschrieben.
 * 27.11.2007: 1.3 - Erweiterung auf Angabe via Werk/Band/Seite, httpSpamFilter eingebaut.
 *
 * @author caschoff
 * @version 1.3
 *
 */
public class SchlagwortAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SchlagwortAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @return Der Action-Forward.
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {

        //
        // Wurde kein Wort aber eine Seite spezifiziert?
        //
        if (!RequestUtil.isEmpty("werk", request)
                && !RequestUtil.isEmpty("bandnr", request)
                && !RequestUtil.isEmpty("seitenr", request)
                && RequestUtil.isEmpty("wort", request)) {

            String werkKurzname = request.getParameter("werk");
            String seiteNummer = request.getParameter("seitenr");
            String bandNummer = request.getParameter("bandnr");
            logger.debug(werkKurzname + "/" + bandNummer + "/" + seiteNummer);
            @SuppressWarnings("unchecked")
            List<Seite> l = (List<Seite>) hiberSession.getNamedQuery("Seite by werk, band and nummer").setString("werk", werkKurzname).setString("nummer", "%" + seiteNummer).setString("band", bandNummer).list();
            if (l.size() == 0) {
                request.setAttribute("hint", "Es wurde keine passende Seite gefunden.");
                return mapping.findForward(ViewNames.schlagwortView);
            } else if (l.size() == 1) {
                Seite s = (Seite) l.get(0);
                response.sendRedirect("/retrobib/seite.html?id=" + s.getId());
                return null;
            } else if (l.size() < 100) {
                request.setAttribute("seiten", l);
                return mapping.findForward(ViewNames.seitenauswahlView);
            } else {
                request.setAttribute("hint", "Es wurde mehr als 100 Seiten gefunden. Bitte schränken Sie die Anfrage ein.");
                return mapping.findForward(ViewNames.schlagwortView);
            }
        }

        //
        // Wurde ein Wort spezifiziert?
        //
        if (RequestUtil.isEmpty("wort", request)) {
            request.setAttribute("hint", "Sie haben kein Schlagwort angegeben.");
            return mapping.findForward(ViewNames.schlagwortView);
        }

        //
        // Schlagwort fischen...
        //
        String encoding = request.getParameter("encoding");
        if (encoding == null) {
            encoding = "UTF-8";
        }
        String wort = new String(request.getParameter("wort").getBytes(), encoding);
        if (!encoding.equals("UTF-8")) {
            wort = TextUtil.transcode(wort, encoding, "UTF-8");
        }
        wort = filterHttpSpam(wort.trim());
        request.setAttribute("schlagwort", wort);

        //
        // Weitere URL-Variablen zum Einschränken?
        //
        String werkKurzname = request.getParameter("werk");
        String seiteNummer = request.getParameter("seitenr");
        String bandNummer = request.getParameter("bandnr");

        String queryString = "from Schlagwort s where s.deleted=false and s.keyword=:keyword";
        if ((werkKurzname != null) && (werkKurzname.trim().length() != 0)) {
            queryString += " and s.seiteid.teilid.bandid.werkid.shortname=:shortname";
        }
        if ((seiteNummer != null) && (seiteNummer.trim().length() != 0)) {
            queryString += " and s.seiteid.pagenumber like :pagenumber";
        }
        if ((bandNummer != null) && (bandNummer.trim().length() != 0)) {
            queryString += " and s.seiteid.teilid.bandid.number=:number";
        }
        logger.debug("Query: '" + queryString + "'");

        Query query = hiberSession.createQuery(queryString).setString("keyword", wort);
        if ((werkKurzname != null) && (werkKurzname.trim().length() != 0)) {
            query = query.setString("shortname", werkKurzname);
        }
        if ((seiteNummer != null) && (seiteNummer.trim().length() != 0)) {
            query = query.setString("pagenumber", "%" + seiteNummer);
        }
        if ((bandNummer != null) && (bandNummer.trim().length() != 0)) {
            query = query.setString("number", bandNummer);
        }
        @SuppressWarnings("unchecked")
        List<Schlagwort> schlagwoerter = (List<Schlagwort>) query.list();

        if (schlagwoerter.size() == 0) {
            // kein Schlagwort in Datenbank, auf Suche gehen...
            String werkRestriction = "";
            // Werkseinschränkung?
            if (werkKurzname != null) {
                werkRestriction = "&sn_" + werkKurzname + "=on";
            }
            response.sendRedirect("/retrobib/suchtreffer.html?suchtext=" + URLEncoder.encode(wort, "UTF-8") + werkRestriction);
            return null;
        }

        if (schlagwoerter.size() == 1) {
            // nur ein Schlagwort in Datenbank, auf Seite gehen...
            Schlagwort swort = schlagwoerter.get(0);
            response.sendRedirect("/retrobib/seite.html?id=" + swort.getSeiteid().getId() + "#" + URLEncoder.encode(swort.getKeyword(), "UTF-8").replaceAll("\\+", "%20"));
            return null;
        }

        // mehrere Schlagwörter, an Request anhängen und auf Auswahl gehen...
        request.setAttribute("schlagwoerter", schlagwoerter);
        request.setAttribute("count", schlagwoerter.size());

        return mapping.findForward(ViewNames.schlagwortView);
    }
}
