/*
 * SeiteAction.java
 *
 * Created on 13. August 2007, 10:31
 *
 */
package de.retrobib.actions;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.Constants.ViewNames;
import de.retrobib.db.logic.SchlagwortLogic;
import de.retrobib.db.logic.SeiteLogic;
import de.retrobib.db.logic.SeitentextLogic;
import de.retrobib.db.logic.SiehewortLogic;
import de.retrobib.db.model.Account;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Faksimile;
import de.retrobib.db.model.Hiresfaksimile;
import de.retrobib.db.model.Schlagwort;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.db.model.Siehewort;
import de.retrobib.db.model.Teil;
import de.retrobib.db.model.Werk;
import de.retrobib.webelements.MainMenueItem;
import de.retrobib.webelements.Message;
import de.retrobib.xml.XhtmlTransformer;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.RequestUtil;
import org.ca.utils.misc.TextUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "seite.html".
 *
 * 17.11.2007: Fix 1.1: Doppelte Verwendung von encodeXML in tagsieheWorte und tagHinweisWorte
 * führte zu falscher Darstellung der Seite.
 * 20.11.2007: Erweiterung 1.2: Nachsehen der Schlagworte einer Seite via Bifab-API.
 *
 * @author caschoff
 * @version 1.2
 *
 */
public class SeiteAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SeiteAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {

        //
        // Redirect vom alten Meyers bekommen?
        //
        if (request.getParameter("oldid") != null) {
            long oldid = Long.parseLong(request.getParameter("oldid"));
            logger.debug("Redirect vom alten Meyers (oldid=" + oldid + "), Referer='" + request.getHeader("Referer") + "'...");

            long count = (Long) hiberSession.getNamedQuery("Seite.count by oldid").setLong("oldid", oldid).uniqueResult();
            if (count == 1) {
                Seite seite = (Seite) hiberSession.getNamedQuery("Seite by oldid").setLong("oldid", oldid).list().get(0);
                logger.debug("Oldid ist '" + oldid + "', sende 'permanently moved' zu id '" + seite.getId() + "'...");

                response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                response.setHeader("Location", "/retrobib/seite.html?id=" + seite.getId());
                return null;
            } else {
                response.sendError(404);
                return null;
            }
        }

        //
        // Statt id eine tid (Text-ID), fid (Faximile-ID) oder hfid (Hires-Faksimile-ID) bekommen?
        //
        // tid...
        if (request.getParameter("tid") != null) {
            long tid = Long.parseLong(request.getParameter("tid"));
            logger.debug("Tid (" + tid + ") statt Id bekommen...");

            long count = (Long) hiberSession.getNamedQuery("Seitentext.count by id").setLong("id", tid).uniqueResult();
            if (count == 1) {
                Seitentext seitentext = (Seitentext) hiberSession.load(Seitentext.class, tid);
                logger.debug("Tid ist '" + tid + "', sende 'permanently moved' zu id '" + seitentext.getSeiteid().getId() + "'...");

                response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                response.setHeader("Location", "/retrobib/seite.html?id=" + seitentext.getSeiteid().getId());
                return null;
            } else {
                response.sendError(404);
                return null;
            }
        }
        // fid...
        if (request.getParameter("fid") != null) {
            long fid = Long.parseLong(request.getParameter("fid"));
            logger.debug("Fid (" + fid + ") statt Id bekommen...");

            long count = (Long) hiberSession.getNamedQuery("Faksimile.count by id").setLong("id", fid).uniqueResult();
            if (count == 1) {
                Faksimile faksimile = (Faksimile) hiberSession.load(Faksimile.class, fid);
                logger.debug("Fid ist '" + fid + "', sende 'permanently moved' zu id '" + faksimile.getSeiteid().getId() + "'...");

                response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                response.setHeader("Location", "/retrobib/seite.html?id=" + faksimile.getSeiteid().getId());
                return null;
            } else {
                response.sendError(404);
                return null;
            }
        }
        // hfid...
        if (request.getParameter("hfid") != null) {
            long hfid = Long.parseLong(request.getParameter("hfid"));
            logger.debug("Hfid (" + hfid + ") statt Id bekommen...");

            long count = (Long) hiberSession.getNamedQuery("Hiresfaksimile.count by id").setLong("id", hfid).uniqueResult();
            if (count == 1) {
                Hiresfaksimile hiresfaksimile = (Hiresfaksimile) hiberSession.load(Hiresfaksimile.class, hfid);
                logger.debug("Hfid ist '" + hfid + "', sende 'permanently moved' zu id '" + hiresfaksimile.getSeiteid().getId() + "'...");

                response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                response.setHeader("Location", "/retrobib/seite.html?id=" + hiresfaksimile.getSeiteid().getId());
                return null;
            } else {
                response.sendError(404);
                return null;
            }
        }

        //
        // Werk, Band, Teil und Seite anhängen
        //
        logger.debug("Lade Seite für id " + request.getParameter("id") + "...");
        Seite seite = null;
        try {
            String id = request.getParameter("id");
            //
            // Redirect-Bug für IE <= 6 beheben...
            // Der IE <= 6 sendet bei einem Redirect (z. B. "seite.html?id=1234#Wasser") als id "1234#Wasser" statt "1234"
            //
            if (id.indexOf("#") != -1) {
                logger.debug("Fixe #-redirect-bug...");
                id = id.substring(0, id.indexOf("#"));
                logger.debug("Id jetzt " + id);
            }
            seite = (Seite) hiberSession.load(Seite.class, Long.parseLong(id));
            seite.getId();
        } catch (Exception ex) {
            logger.debug(ex.getMessage());
            response.sendError(404);
            return null;
        }
        Teil teil = seite.getTeilid();
        Band band = teil.getBandid();
        Werk werk = band.getWerkid();

        request.setAttribute("seite", seite);
        request.setAttribute("teil", teil);
        request.setAttribute("band", band);
        request.setAttribute("werk", werk);

        //
        // Editierten Text speichern?
        //
        if (getAccountFromSession(httpSession) != null) {
            if (request.getParameter("seitentext") != null) {
                SeiteLogic.updateSeite(seite, hiberSession, getAccountFromSession(httpSession), true, true);

                Seitentext seitentext = (Seitentext) hiberSession.getNamedQuery("Seitentext by seiteid").setLong("seiteid", seite.getId()).setMaxResults(1).list().get(0);
                sendBackupMail(getAccountFromSession(httpSession), seite, seitentext.getContent(), request.getParameter("seitentext"));
                seitentext.setContent(request.getParameter("seitentext"));
                SeitentextLogic.updateSeitentext(seitentext, hiberSession, getAccountFromSession(httpSession), true, true);
            }
        }

        //
        // vorhergehende und nächste Seite finden
        //
        Seite nextSeite = SeiteLogic.next(hiberSession, seite);
        if (nextSeite != null) {
            request.setAttribute("next", nextSeite);
        }

        Seite prevSeite = SeiteLogic.prev(hiberSession, seite);
        if (prevSeite != null) {
            request.setAttribute("prev", prevSeite);
        }

        Account account = getAccountFromSession(httpSession);
        //
        // Ist es erlaubt, Keywords zu löschen?
        //
        boolean delWordsAllowed = ((account != null) && (seite.getStatus() != Seite.TAGGED));
        request.setAttribute("delallowed", delWordsAllowed);
        //
        // See-/Keywords hinzufügen?
        //
        manageKeySeeWordRequest(hiberSession, httpSession, request, seite, account);

        //
        // zusätzliche Menüs? (Online-Edit, Taggen...)
        //
        boolean tagmode = false;
        if (!RequestUtil.commandIs("edit", request) && (account != null) && (seite.getStatus() != Seite.TAGGED)) {
            // Tagmodus ein-/ausschalten?
            if ((request.getParameter(MiscConstants.tagmodus) != null) && (request.getParameter(MiscConstants.tagmodus).equals("on"))) {
                httpSession.setAttribute(MiscConstants.tagmodus, true);
            }
            if ((request.getParameter(MiscConstants.tagmodus) != null) && (request.getParameter(MiscConstants.tagmodus).equals("off"))) {
                httpSession.setAttribute(MiscConstants.tagmodus, false);
            }

            // welchen Zustand haben wir?
            if ((httpSession.getAttribute(MiscConstants.tagmodus) != null) && ((Boolean) httpSession.getAttribute(MiscConstants.tagmodus))) {
                tagmode = true;
                // taggen nur bei bestimmten Stati...
                if ((seite.getStatus() == Seite.NORMAL) || (seite.getStatus() == Seite.FEIN)) {
                    // Im Tagmodus QuickSearch abschalten...
                    request.setAttribute("disableQuickSearch", true);
                    // ... und Script einschalten...
                    request.setAttribute("showTagScript", true);
                }
            }

            ArrayList<MainMenueItem> menue = new ArrayList<MainMenueItem>();

            menue.add(new MainMenueItem("Seitenspezifisches", MainMenueItem.MAINENTRY, null));
            // Online Edit nur bei bestimmten Stati und wenn die Seite nicht ausgeklinkt ist...
            if (((seite.getStatus() == Seite.NORMAL) || (seite.getStatus() == Seite.FEIN)) && (!seite.getBlockmarker())) {
                menue.add(new MainMenueItem("Text bearbeiten", "", "/retrobib/seite.html?id=" + seite.getId() + "&amp;command=edit"));
            }
            if (!tagmode) {
                menue.add(new MainMenueItem("Tagmodus: an", MainMenueItem.NOBOTTOM, "/retrobib/seite.html?id=" + seite.getId() + "&amp;" + MiscConstants.tagmodus + "=on"));
            } else {
                menue.add(new MainMenueItem("Tagmodus: aus", MainMenueItem.NOBOTTOM, "/retrobib/seite.html?id=" + seite.getId() + "&amp;" + MiscConstants.tagmodus + "=off"));
            }
            request.setAttribute("moreMenueItems", menue);
        }

        //
        // (HIRES-)Faksimiles anhängen
        //
        Long faksimileCount = (Long) hiberSession.getNamedQuery("Faksimile.count by seiteid").setLong("seiteid", seite.getId()).uniqueResult();
        long faksimileid = -1;
        if (faksimileCount != 0) {
            Object[] faks = (Object[]) hiberSession.getNamedQuery("Faksimile.part by seiteid").setLong("seiteid", seite.getId()).setMaxResults(1).list().get(0);
            faksimileid = (Long) faks[0];
            request.setAttribute("faksimileid", faksimileid);
        }
        Long hiresfaksimileCount = (Long) hiberSession.getNamedQuery("Hiresfaksimile.count by seiteid").setLong("seiteid", seite.getId()).uniqueResult();
        if (hiresfaksimileCount != 0) {
            Object[] hffaks = (Object[]) hiberSession.getNamedQuery("Hiresfaksimile.part by seiteid").setLong("seiteid", seite.getId()).setMaxResults(1).list().get(0);
            long hfid = (Long) hffaks[0];
            request.setAttribute("hiresfaksimileid", hfid);
        }

        //
        // Seiteninhalt...
        //
        Seitentext seitentext = (Seitentext) hiberSession.getNamedQuery("Seitentext by seiteid").setLong("seiteid", seite.getId()).setMaxResults(1).list().get(0);
        @SuppressWarnings("unchecked")
        List<Schlagwort> schlagworte = (List<Schlagwort>) hiberSession.getNamedQuery("Schlagwort by seiteid order by length").setLong("seiteid", seite.getId()).list();
        @SuppressWarnings("unchecked")
        List<Siehewort> sieheworte = (List<Siehewort>) hiberSession.getNamedQuery("Siehewort by seiteid order by length").setLong("seiteid", seite.getId()).list();

        // je nach Status der Seite rendern...
        String content = null;
        if (seite.getStatus() == Seite.UNKORRIGIERT) {
            // unkorrigierte Seite
            content = prepareRawContent(seitentext);
            request.setAttribute("hinweis", "Diese Seite ist noch nicht korrigiert worden und enthält Fehler.");
        } else if (seite.getStatus() == Seite.TAGGED) {
            // Seite in XML
            content = new XhtmlTransformer().getTransformedText(seitentext.getContent(), "" + faksimileid);
        } else {
            // normal oder fein korrigierte Seite
            content = prepareContent(seitentext, schlagworte, sieheworte, tagmode);
        }
        request.setAttribute("text", content);
        if (RequestUtil.commandIs("edit", request)) {
            request.setAttribute("rawtext", seitentext.getContent());
        }
        //
        // Schlagworte und Verweise auch anhängen
        //
        @SuppressWarnings("unchecked")
        List<Schlagwort> sws = (List<Schlagwort>) hiberSession.getNamedQuery("Schlagwort by seiteid").setLong("seiteid", seite.getId()).list();

        /*
         * Die Bifab-Schnittstelle wurde am 23.3.2009 abgeschaltet...; ca. 2.4.2009
         *
        if (werk.getShortname().equals("Meyers")) {
        request.setAttribute("bifab", preCheckSchlagworteForBifab(sws, hiberSession));
        }
         */

        request.setAttribute("schlagworte", sws);
        request.setAttribute("sieheworte", hiberSession.getNamedQuery("Siehewort by seiteid").setLong("seiteid", seite.getId()).list());

        //
        // Meta-Tags...
        //
        if (schlagworte.size() != 0) {
            StringBuilder meta = new StringBuilder();
            for (int i = 0; i < schlagworte.size(); i++) {
                meta.append(schlagworte.get(i).getKeyword());
                if (i < schlagworte.size() - 1) {
                    meta.append(", ");
                }
            }
            logger.debug("Meta-Keywords: " + meta);
            request.setAttribute("metakeys", meta.toString());
        }

        // jetzt neu: Bildansicht inline?
        if (request.getParameter("imageview") != null) {
            httpSession.setAttribute("imageview", true);
            request.setAttribute("imageview", true);
        }
        if (request.getParameter("textview") != null) {
            httpSession.removeAttribute("imageview");
        }

        return mapping.findForward(ViewNames.seiteView);
    }

    /**
     * Den Seitentext für <b>unkorrigierte</b> Seiten aufbereiten.
     *
     * @param seitentext Der Seitentext.
     * @return Der aufbereitete Seitentext.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    private String prepareRawContent(Seitentext seitentext)
            throws Exception {
        StringBuilder ret = new StringBuilder();

        String text = TextUtil.transcode(seitentext.getContent(), seitentext.getEncoding(), "UTF-8");
        text = text.replaceAll("\r\n", "\n");
        StringTokenizer st = new StringTokenizer(text, "\n");
        ret.append("<p>");
        while (st.hasMoreTokens()) {
            ret.append(TextUtil.encodeXML(st.nextToken()));
            ret.append("<br />");
        }
        ret.append("</p>");
        return ret.toString();
    }

    /**
     * Den Seitentext für <b>normal-/feinkorrigierte</b> Seiten aufbereiten.
     *
     * @return Der aufbereitete Text.
     * @param mitHinweisen Sollen Hinweise auf Verweise getagged werden?
     * @param seitentext Der Seitentext.
     * @param schlagworte Die Schlagwörter auf der Seite.
     * @param sieheworte Die Verweise auf der Seite.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    private String prepareContent(Seitentext seitentext, List<Schlagwort> schlagworte, List<Siehewort> sieheworte, boolean mitHinweisen)
            throws Exception {
        StringBuilder ret = new StringBuilder();

        String text = TextUtil.transcode(seitentext.getContent(), seitentext.getEncoding(), "UTF-8");
        text = text.replaceAll("\r\n", "\n");
        String[] tokens = text.split("\n");

        if (tokens.length > 0) {
            // Seitennummer
            ret.append("<p class=\"seitenzahl\">" + tokens[0] + "</p>\n");
        }

        if (tokens.length > 2) {
            // von-bis
            ret.append("<h1 class=\"seitenkopf\">" + tokens[2] + "</h1>\n");
        }

        for (int t = 4; t < tokens.length; t++) {
            String paraText = tokens[t];
            if (paraText.trim().length() != 0) {
                boolean found = false;
                // Keyword am Anfang?
                for (Schlagwort sw : schlagworte) {
                    String schlagwort = sw.getKeyword();
                    if (paraText.startsWith(schlagwort)) {
                        if ((paraText.length() == schlagwort.length()) // ParaText == Schlagwort
                                || (!Character.isLetter(paraText.charAt(schlagwort.length()))) // Schlagwort nur, wenn kein Buchstabe am Ende+1
                                ) {
                            String rest = paraText.substring(schlagwort.length(), paraText.length());
                            rest = tagSieheWorte(rest, sieheworte, mitHinweisen);
                            paraText = "<a id=\"" + TextUtil.encodeXML(schlagwort) + "\" name=\"" + TextUtil.encodeXML(schlagwort) + "\"></a><em class=\"schlagwort\">" + TextUtil.encodeXML(schlagwort) + "</em>" + rest;
                            found = true;
                            break;
                        }
                    }
                }
                if (!found) {
                    paraText = tagSieheWorte(paraText, sieheworte, mitHinweisen);
                }
                ret.append("<p class=\"artikel\">");
                ret.append(paraText);
                ret.append("</p>\n");
            }
        }

        return ret.toString();
    }

    /**
     * Rekursive Funktion zum taggen der Verweise in einem Text.
     *
     * @return Der getaggte Text.
     * @param mitHinweisen Sollen s. d. etc. hervorgehobenen werden?
     * @param text Der zu taggende Text.
     * @param sieheworte Die Liste der Sieheworte.
     *
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    public String tagSieheWorte(String text, List<Siehewort> sieheworte, boolean mitHinweisen)
            throws Exception {
        StringBuilder ret = new StringBuilder();
        boolean found = false;

        for (Siehewort sw : sieheworte) {
            String siehewort = sw.getSeeword();
            int pos = text.indexOf(siehewort);
            if (pos != -1) {
                // Treffer, kann aber in einem Teilwort aufgetaucht sein
                if (pos == text.length() - siehewort.length()) {
                    // ganz am Ende
                    ret.append(tagSieheWorte(text.substring(0, pos), sieheworte, mitHinweisen));
                    ret.append("<a id=\"");
                    ret.append(TextUtil.encodeXML(siehewort));
                    ret.append("\" name=\"");
                    ret.append(TextUtil.encodeXML(siehewort));
                    ret.append("\"></a><a href=\"/retrobib/schlagwort.html?wort=");
                    ret.append(URLEncoder.encode(siehewort, "UTF-8"));
                    ret.append("\">");
                    ret.append(TextUtil.encodeXML(siehewort));
                    ret.append("</a>");
                    found = true;
                    break;
                } else if (pos == 0) {
                    // ganz am Anfang
                    ret.append("<a id=\"");
                    ret.append(TextUtil.encodeXML(siehewort));
                    ret.append("\" name=\"");
                    ret.append(TextUtil.encodeXML(siehewort));
                    ret.append("\"></a><a href=\"/retrobib/schlagwort.html?wort=");
                    ret.append(URLEncoder.encode(siehewort, "UTF-8"));
                    ret.append("\">");
                    ret.append(TextUtil.encodeXML(siehewort));
                    ret.append("</a>");
                    ret.append(tagSieheWorte(text.substring(siehewort.length(), text.length()), sieheworte, mitHinweisen));
                    found = true;
                    break;
                } else {
                    if (Character.isLetter(text.charAt(pos + siehewort.length()))) {
                        // in Wort gefunden, Fehltreffer
                    } else if (Character.isLetter(text.charAt(pos - 1))) {
                        // in Wort gefunden, Fehltreffer
                    } else {
                        // korrekter Treffer
                        ret.append(tagSieheWorte(text.substring(0, pos), sieheworte, mitHinweisen));
                        ret.append("<a id=\"");
                        ret.append(TextUtil.encodeXML(siehewort));
                        ret.append("\" name=\"");
                        ret.append(TextUtil.encodeXML(siehewort));
                        ret.append("\"></a><a href=\"/retrobib/schlagwort.html?wort=");
                        ret.append(URLEncoder.encode(siehewort, "UTF-8"));
                        ret.append("\">");
                        ret.append(TextUtil.encodeXML(siehewort));
                        ret.append("</a>");
                        ret.append(tagSieheWorte(text.substring(pos + siehewort.length(), text.length()), sieheworte, mitHinweisen));
                        found = true;
                        break;
                    }
                }
            }
        }
        if (!found) {
            if (mitHinweisen) {
                ret.append(tagHinweisWorte(TextUtil.encodeXML(text), MiscConstants.hinweise));
            } else {
                ret.append(TextUtil.encodeXML(text));
            }
        }

        return ret.toString();
    }

    /**
     * Für das Taggen Verweiswörter hervorheben (s. d. etc.).
     *
     * @return Der Text mit den hervorgehobenen Hinweisen.
     * @param text Der Text in dem Verweise hervorgehoben werden sollen.
     * @param hinweise Eine Liste mit den Hinweisen.
     *
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    public String tagHinweisWorte(String text, String[] hinweise)
            throws Exception {
        StringBuilder ret = new StringBuilder();
        boolean found = false;

        for (String hinweis : hinweise) {
            int pos = text.indexOf(hinweis);
            if (pos != -1) {
                // Treffer, kann aber in einem Teilwort aufgetaucht sein
                if (pos == text.length() - hinweis.length()) {
                    // ganz am Ende
                    ret.append(tagHinweisWorte(text.substring(0, pos), hinweise));
                    ret.append("<span class=\"highlight\">");
                    ret.append(hinweis);
                    ret.append("</span>");
                    found = true;
                    break;
                } else if (pos == 0) {
                    // ganz am Anfang
                    ret.append("<span class=\"highlight\">");
                    ret.append(hinweis);
                    ret.append("</span>");
                    ret.append(tagHinweisWorte(text.substring(hinweis.length(), text.length()), hinweise));
                    found = true;
                    break;
                } else {
                    if (Character.isLetter(text.charAt(pos + hinweis.length()))) {
                        // in Wort gefunden, Fehltreffer
                    } else if (Character.isLetter(text.charAt(pos - 1))) {
                        // in Wort gefunden, Fehltreffer
                    } else {
                        // korrekter Treffer
                        ret.append(tagHinweisWorte(text.substring(0, pos), hinweise));
                        ret.append("<span class=\"highlight\">");
                        ret.append(hinweis);
                        ret.append("</span>");
                        ret.append(tagHinweisWorte(text.substring(pos + hinweis.length(), text.length()), hinweise));
                        found = true;
                        break;
                    }
                }
            }
        }
        if (!found) {
            // Bug-Fix 17.11.2007: Der Text kommt schon XML-Encodet.
            // ret.append(TextUtil.encodeXML(text));
            ret.append(text);
        }

        return ret.toString();
    }

    /**
     * Behandeln eventueller Anforderungen bezüglich der See-/Keywords.<br />
     * Add/Del Keyword...<br />
     * Add/Del Seeword...<br />
     * ... aus dem online-taggen.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @param request Der HttpServletRequest.
     * @param seite Die Seite.
     * @param account der Account.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    private void manageKeySeeWordRequest(Session hiberSession, HttpSession httpSession, HttpServletRequest request, Seite seite, Account account)
            throws Exception {
        if (account != null) {
            if (RequestUtil.commandIs("addSeeword", request)) {
                String seeword = new String(request.getParameter("siehewort").getBytes(), "UTF-8");
                logger.debug("add: " + seeword);
                if ((seeword != null) && (seeword.trim().length() != 0)) {
                    Siehewort siehewort = new Siehewort();
                    siehewort.setDeleted(false);
                    siehewort.setSeeword(seeword);
                    siehewort.setSeiteid(seite);
                    SiehewortLogic.createSiehewort(siehewort, hiberSession, account, true, true);
                }
            }
            if (RequestUtil.commandIs("delSeeword", request)) {
                long id = Long.parseLong(request.getParameter("wortid"));
                Siehewort siehewort = (Siehewort) hiberSession.load(Siehewort.class, id);
                siehewort.setDeleted(true);
                SiehewortLogic.updateSiehewort(siehewort, hiberSession, account, true, true);
            }
            if (RequestUtil.commandIs("markNoKeyword", request)) {
                seite.setNokeywords(!seite.getNokeywords());
                SeiteLogic.updateSeite(seite, hiberSession, account, true, true);
            }

            if (RequestUtil.commandIs("addKeyword", request)) {
                String keyword = new String(request.getParameter("schlagwort").getBytes(), "UTF-8");
                logger.debug("add: " + keyword);
                if ((keyword != null) && (keyword.trim().length() != 0)) {
                    Schlagwort schlagwort = new Schlagwort();
                    schlagwort.setDeleted(false);
                    schlagwort.setKeyword(keyword);
                    schlagwort.setSeiteid(seite);
                    SchlagwortLogic.createSchlagwort(schlagwort, hiberSession, account, true, true);
                }
            }
            if (RequestUtil.commandIs("delKeyword", request)) {
                long id = Long.parseLong(request.getParameter("wortid"));
                Schlagwort schlagwort = (Schlagwort) hiberSession.load(Schlagwort.class, id);
                schlagwort.setDeleted(true);
                SchlagwortLogic.updateSchlagwort(schlagwort, hiberSession, account, true, true);
            }
            if (RequestUtil.commandIs("markNoSeeword", request)) {
                seite.setNoseewords(!seite.getNoseewords());
                SeiteLogic.updateSeite(seite, hiberSession, account, true, true);
            }
        }
    }
}
