/**
 * StoebernAction.java
 *
 * Created on 15. August 2007, 09:14
 *
 */
package de.retrobib.actions;

import de.retrobib.Constants.ViewNames;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Teil;
import de.retrobib.db.model.Werk;
import de.retrobib.webelements.Message;
import de.retrobib.webelements.TeilMitSeiten;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.hibernate.Session;

/**
 * Action für die Seite "stoebern.html".<br />
 * Diese Action hängt je nach Parametern die passenden
 * Werke / Bände / Teile / Seiten an den request an.<br />
 * Die View stellt dann nur nicht-leere Elemente dar.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class StoebernAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(TechnikAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {

        if ((request.getParameter("bandid") != null) || (request.getParameter("teilid") != null)) {
            Band band = null;
            //
            // Band oder Teil bekommen, Werk, Band und Teile anhängen...
            //
            if (request.getParameter("teilid") != null) {
                try {
                    logger.debug("Lade Teil...");
                    Teil teil = (Teil) hiberSession.load(Teil.class, Long.parseLong(request.getParameter("teilid")));
                    band = teil.getBandid();
                } catch (Exception ex) {
                    response.sendError(404);
                    return null;
                }
            }

            if (band == null) {
                try {
                    logger.debug("Lade Band...");
                    band = (Band) hiberSession.load(Band.class, Long.parseLong(request.getParameter("bandid")));
                    band.getId();
                } catch (Exception ex) {
                    response.sendError(404);
                    return null;
                }
            }

            request.setAttribute("band", band);
            Werk werk = band.getWerkid();
            request.setAttribute("werk", werk);
            @SuppressWarnings("unchecked")
            List<Teil> teile = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", band.getId()).list();
            ArrayList<TeilMitSeiten> teileMitSeiten = new ArrayList<TeilMitSeiten>();
            for (Teil teil : teile) {
                @SuppressWarnings("unchecked")
                List<Seite> seiten = (List<Seite>) hiberSession.getNamedQuery("Seite by teilid").setLong("teilid", teil.getId()).list();
                teileMitSeiten.add(new TeilMitSeiten(teil, seiten, hiberSession));
            }
            request.setAttribute("teileMitSeiten", teileMitSeiten);
        } else if (request.getParameter("werkid") != null) {
            //
            // Werk bekommen, Werk und Bände anhängen...
            //
            logger.debug("Lade Werk...");
            Werk werk = null;
            try {
                werk = (Werk) hiberSession.load(Werk.class, Long.parseLong(request.getParameter("werkid")));
                werk.getId();
            } catch (Exception ex) {
                response.sendError(404);
                return null;
            }
            request.setAttribute("werk", werk);
            @SuppressWarnings("unchecked")
            List<Band> baende = (List<Band>) hiberSession.getNamedQuery("Band by werkid").setLong("werkid", Long.parseLong(request.getParameter("werkid"))).list();
            if (baende.size() == 1) {
                // nur ein Band, gleich aufklappen...
                Band band = baende.get(0);
                request.setAttribute("band", band);
                @SuppressWarnings("unchecked")
                List<Teil> teile = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", band.getId()).list();
                ArrayList<TeilMitSeiten> teileMitSeiten = new ArrayList<TeilMitSeiten>();
                for (Teil teil : teile) {
                    @SuppressWarnings("unchecked")
                    List<Seite> seiten = (List<Seite>) hiberSession.getNamedQuery("Seite by teilid").setLong("teilid", teil.getId()).list();
                    teileMitSeiten.add(new TeilMitSeiten(teil, seiten, hiberSession));
                }
                request.setAttribute("teileMitSeiten", teileMitSeiten);
            } else {
                // mehrere Bände, Liste anhängen
                request.setAttribute("baende", baende);
            }
        } else {
            //
            // garnichts bekommen, Werke anhängen...
            //
            request.setAttribute("werke", hiberSession.getNamedQuery("Werk").list());
        }

        return mapping.findForward(ViewNames.stoebernView);
    }
}
