/**
 * SuchTrefferAction.java
 *
 * Created on 27. August 2007, 09:14
 *
 */
package de.retrobib.actions;

import de.retrobib.Constants.ViewNames;
import de.retrobib.lucene.Sherlock;
import de.retrobib.webelements.LuceneHits;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.BooleanQuery.TooManyClauses;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.hibernate.Session;

/**
 * Action für die Seite "suchtreffer.html".
 *
 * 27.11.2007: 1.1 - HttpSpamFilter eingebaut.
 *
 * @author caschoff
 * @version 1.1
 *
 */
public class SuchTrefferAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SuchTrefferAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {

        String query = null;
        if (request.getParameter("suchtext") != null) {
            query = new String(request.getParameter("suchtext").getBytes(), "UTF-8");
        }
        if (query == null) {
            query = "";
        }
        query = filterHttpSpam(query.trim());
        request.setAttribute("query", query);

        ArrayList<String> shortNames = null;
        // auf Werke einschränken?
        @SuppressWarnings("unchecked")
        Enumeration<String> enu = (Enumeration<String>) request.getParameterNames();
        while (enu.hasMoreElements()) {
            String parName = enu.nextElement();
            if (parName.startsWith("sn_")) {
                if (shortNames == null) {
                    shortNames = new ArrayList<String>();
                }
                shortNames.add(parName.substring(3, parName.length()));
            }
        }

        if (query.length() != 0) {
            try {
                LuceneHits hits = Sherlock.search(query, shortNames, hiberSession);

                // Treffer anhängen
                request.setAttribute("hits", hits);
            } catch (ParseException pe) {
                // Fehler in der Query-Syntax...
                request.setAttribute("hinweis", "Ihre Suchanfrage konnte nicht ausgewertet werden (Syntaxfehler). Bitte verwenden Sie die auf der Suchseite beschriebene Syntax.");
            } catch (TooManyClauses tmc) {
                // Fehler in der Query-Syntax...
                request.setAttribute("hinweis", "Ihre Suchanfrage konnte nicht ausgewertet werden (sie war zu ungenau). Bitte verwenden Sie die auf der Suchseite beschriebene Syntax.");
            }
        } else {
            request.setAttribute("hinweis", "Sie müssen mindestens ein Stichwort angeben.");
        }

        return mapping.findForward(ViewNames.suchtrefferView);
    }
}
