/**
 * FaksimileAction.java
 *
 * Created on 16. August 2007, 09:19
 *
 */
package de.retrobib.actions.downloads;

import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.model.Faksimile;
import de.retrobib.utils.ImageUtil;
import de.retrobib.webelements.Message;
import java.awt.Rectangle;
import java.util.ArrayList;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.RequestUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "faksimile_*.jpeg". (Bilder downloaden)
 *
 * 3.12.2007: 1.1 - Croppen ermöglicht.
 *
 * @author caschoff
 * @version 1.1
 *
 */
public class FaksimileAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(FaksimileAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        // Id aus dem Namen ([...]/faksimile_*.jpeg) holen...
        String url = request.getRequestURI();
        String id = null;
        String sid = null;
        Faksimile faksimile = null;

        try {
            logger.debug("URI=" + url);
            if (url.indexOf("_sid_") == -1) {
                id = url.substring(url.indexOf("_") + 1, url.indexOf("."));
                logger.debug("Id für Faksimile ist " + id + ".");
                faksimile = (Faksimile) hiberSession.load(Faksimile.class, Long.parseLong(id));
            } else {
                sid = url.substring(url.indexOf("_sid_") + 5, url.indexOf("."));
                logger.debug("Sid für Faksimile ist " + sid + ".");
                faksimile = (Faksimile) hiberSession.getNamedQuery("Faksimile by seiteid").setLong("seiteid", Long.parseLong(sid)).setMaxResults(1).list().get(0);
            }
        } catch (Exception ex) {
            // Faksimile nicht gefunden...
            response.sendError(404);
            return null;
        }

        Rectangle area = null;
        // nur einen Teil des Bildes liefern?
        if (!RequestUtil.isEmpty("x", request)) {
            logger.debug("Croppen angefordert...");
            area = new Rectangle(
                    Integer.parseInt(request.getParameter("x")),
                    Integer.parseInt(request.getParameter("y")),
                    Integer.parseInt(request.getParameter("w")),
                    Integer.parseInt(request.getParameter("h")));
            logger.debug("area=" + area);
        }

        try {
            response.setContentType("image/jpeg");

            String dispos = "inline";
            if (url.indexOf("a_") != -1) {
                dispos = "attachment";
            }

            response.setHeader("Content-disposition", dispos + "; filename=" + FileNamer.getFileName(faksimile) + ".jpg");
            if (area == null) {
                response.setContentLength(faksimile.getScan().length);

                ServletOutputStream sos = response.getOutputStream();
                sos.write(faksimile.getScan());
                sos.flush();
                logger.debug("Normales Faksimile geschrieben.");
            } else {
                ServletOutputStream sos = response.getOutputStream();
                new ImageUtil().cropImage(faksimile, area, sos);
                sos.flush();
                logger.debug("Beschnittenes Faksimile geschrieben.");
            }
        } catch (Throwable ex) {
            logger.debug("Unkritische Exception (Laden abgebrochen): " + ex.getMessage());
        }

        return null;
    }
}
