/*
 * FileNamer.java
 *
 * Created on 11. September 2007, 08:40
 *
 */
package de.retrobib.actions.downloads;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.db.model.Faksimile;
import de.retrobib.db.model.Hiresfaksimile;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import org.ca.utils.misc.TextUtil;

/**
 * Eine kleine Helper-Klasse um überall gleiche
 * Namen für Texte/Bilder etc. zu haben.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class FileNamer {

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private FileNamer() {
    }

    /**
     * Einen 'virtuellen' Pfad zu einer Seite bekommen.
     *
     * @param seite Die Seite.
     * @return Der Pfad.
     */
    public static String getPathName(Seite seite) {
        StringBuilder ret = new StringBuilder();

        ret.append(seite.getTeilid().getBandid().getWerkid().getSort());
        ret.append("_");
        ret.append(TextUtil.encodeFileName(seite.getTeilid().getBandid().getWerkid().getShortname()));
        ret.append("/");
        ret.append(seite.getTeilid().getBandid().getNumber());
        ret.append("_");
        ret.append(TextUtil.encodeFileName(seite.getTeilid().getBandid().getName()));
        ret.append("/");
        ret.append(seite.getTeilid().getSort());
        ret.append("_");
        ret.append(TextUtil.encodeFileName(seite.getTeilid().getName()));
        ret.append("/");
        ret.append(MiscConstants.statusNamen[seite.getStatus()]);

        return ret.toString();
    }

    /**
     * Einen Namen für ein Faksimile bekommen.
     *
     * @param faksimile Das Faksimile.
     * @return Der Name.
     */
    public static String getFileName(Faksimile faksimile) {
        StringBuilder ret = new StringBuilder();

        ret.append(faksimile.getSeiteid().getTeilid().getBandid().getWerkid().getShortname());
        ret.append("_");
        ret.append("b");
        ret.append(faksimile.getSeiteid().getTeilid().getBandid().getNumber());
        ret.append("_");
        ret.append("nr");
        ret.append(faksimile.getSeiteid().getPagenumber());
        ret.append("_");
        ret.append("fid_");
        ret.append(faksimile.getId());

        return ret.toString();
    }

    /**
     * Einen Namen für ein hires-Faksimile bekommen.
     *
     * @param hiresfaksimile Das Faksimile.
     * @return Der Name.
     */
    public static String getFileName(Hiresfaksimile hiresfaksimile) {
        StringBuilder ret = new StringBuilder();

        ret.append(hiresfaksimile.getSeiteid().getTeilid().getBandid().getWerkid().getShortname());
        ret.append("_");
        ret.append("b");
        ret.append(hiresfaksimile.getSeiteid().getTeilid().getBandid().getNumber());
        ret.append("_");
        ret.append("nr");
        ret.append(hiresfaksimile.getSeiteid().getPagenumber());
        ret.append("_");
        ret.append("hfid_");
        ret.append(hiresfaksimile.getId());

        return ret.toString();
    }

    /**
     * Einen Namen für einen Seitentext bekommen.
     *
     * @param seitentext Der Seitentext. 
     * @return Der Name.
     */
    public static String getFileName(Seitentext seitentext) {
        StringBuilder ret = new StringBuilder();

        ret.append(seitentext.getSeiteid().getTeilid().getBandid().getWerkid().getShortname());
        ret.append("_");
        ret.append("b");
        ret.append(seitentext.getSeiteid().getTeilid().getBandid().getNumber());
        ret.append("_");
        ret.append("nr");
        ret.append(seitentext.getSeiteid().getPagenumber());
        ret.append("_");
        ret.append("tid_");
        ret.append(seitentext.getId());

        return ret.toString();
    }
}
