/**
 * HiresFaksimileAction.java
 *
 * Created on 16. August 2007, 09:19
 *
 */
package de.retrobib.actions.downloads;

import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.model.Hiresfaksimile;
import de.retrobib.utils.ImageUtil;
import de.retrobib.webelements.Message;
import java.awt.Rectangle;
import java.util.ArrayList;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.RequestUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "hiresfaksimile_*.jpeg".
 *
 * 3.12.2007: 1.1 - Croppen ermöglicht.
 *
 * @author caschoff
 * @version 1.1
 *
 */
public class HiresFaksimileAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(HiresFaksimileAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        // Id aus dem Namen ([...]/hiresfaksimile_*.jpeg) holen...
        String url = request.getRequestURI();
        Hiresfaksimile hiresfaksimile = null;

        try {
            String id = url.substring(url.indexOf("_") + 1, url.indexOf("."));
            logger.debug("Id für Hiresfaksimile ist " + id + ".");
            hiresfaksimile = (Hiresfaksimile) hiberSession.load(Hiresfaksimile.class, Long.parseLong(id));
        } catch (Exception ex) {
            // Hiresfaksimile nicht gefunden
            response.sendError(404);
            return null;
        }

        Rectangle area = null;
        // nur einen Teil des Bildes liefern?
        if (!RequestUtil.isEmpty("x", request)) {
            logger.debug("Croppen angefordert...");
            area = new Rectangle(
                    Integer.parseInt(request.getParameter("x")),
                    Integer.parseInt(request.getParameter("y")),
                    Integer.parseInt(request.getParameter("w")),
                    Integer.parseInt(request.getParameter("h")));
            logger.debug("area=" + area);
        }

        try {
            response.setContentType("image/jpeg");
            response.setHeader("Content-disposition", "inline; filename=" + FileNamer.getFileName(hiresfaksimile) + ".jpg");
            if (area == null) {
                response.setContentLength(hiresfaksimile.getScan().length);

                ServletOutputStream sos = response.getOutputStream();
                sos.write(hiresfaksimile.getScan());
                sos.flush();
            } else {
                ServletOutputStream sos = response.getOutputStream();
                new ImageUtil().cropImage(hiresfaksimile, area, sos);
                sos.flush();
                logger.debug("Beschnittenes Faksimile geschrieben.");
            }
        } catch (Throwable ex) {
            logger.debug("Unkritische Exception (Laden abgebrochen): " + ex.getMessage());
        }

        return null;
    }
}
