/**
 * SitemapAction.java
 *
 * Created on 16. August 2007, 09:19
 *
 */
package de.retrobib.actions.downloads;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.logic.WerkLogic;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Teil;
import de.retrobib.db.model.Werk;
import de.retrobib.webelements.Message;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.hibernate.Session;

/**
 * Action für die Seite "sitemap_*.xml" (Sitemaps für Suchmaschinen).
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class SitemapAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SitemapAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        // Sitemap aus dem Namen ([...]/sitemap_*.xml) holen...
        String url = request.getRequestURI();
        String host = request.getRequestURL().toString().split("/")[2];
        logger.debug("Host ist " + host);

        String sitemap = null;

        try {
            logger.debug("URL=" + url);
            sitemap = url.substring(url.indexOf("_") + 1, url.indexOf("."));
            logger.debug("Sitemap ist " + sitemap + ".");

            response.setContentType("text/xml");
            PrintWriter out = response.getWriter();

            if (sitemap.equals("Index")) {
                // Sitemap-Index angefragt...
                out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
                out.println("<sitemapindex xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">");
                out.println("   <sitemap>");
                out.print("       <loc>");
                out.print(MiscConstants.baseURL);
                out.println("/retrobib/sitemap_Statics.xml</loc>");
                out.print("       <lastmod>");
                out.print(getGoogleDate(System.currentTimeMillis()));
                out.println("</lastmod>");
                out.println("   </sitemap>");
                @SuppressWarnings("unchecked")
                List<Werk> werke = (List<Werk>) hiberSession.getNamedQuery("Werk").list();
                for (Werk werk : werke) {
                    out.println("   <sitemap>");
                    out.print("       <loc>");
                    out.print(MiscConstants.baseURL);
                    out.print("/retrobib/sitemap_");
                    out.print(werk.getShortname());
                    out.println(".xml</loc>");
                    out.print("       <lastmod>");
                    out.print(getGoogleDate(System.currentTimeMillis()));
                    out.println("</lastmod>");
                    out.println("   </sitemap>");
                }
                out.println("</sitemapindex>");
            } else {
                // Sitemap angefragt...
                out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
                out.println("<urlset xmlns=\"http://www.google.com/schemas/sitemap/0.84\">");

                if (sitemap.equals("Statics")) {
                    String[] statics = {
                        "index.html",
                        "impressum.html",
                        "suche.html",
                        "stoebern.html",
                        "technik.html",
                        "statistik.html",
                        "korrektur_faq.html",
                        "korrektur_hilfe.html",
                        "meyers/autorennamen.html",
                        "meyers/abkuerzungen.html",
                        "meyers/meyers_fakten_und_zahlen.html",
                        "meyers/meyers_verlag.html",
                        "meyers/meyers_rundgang.html",
                        "meyers/meyers_redaktion.html",
                        "meyers/meyers_herrmann_j_meyer.html",
                        "meyers/meyers_drucken.html",
                        "meyers/meyers_lexikonkauf_1890.html",
                        "meyers/meyers_digitalisierung.html",
                        "meyers/meyers_literatur.html",
                        "kuenstler/index_kuenstler_AE.html",
                        "kuenstler/index_kuenstler_FK.html",
                        "kuenstler/index_kuenstler_LR.html",
                        "kuenstler/index_kuenstler_SZ.html"
                    };
                    for (String stat : statics) {
                        out.println("   <url>");
                        out.print("       <loc>http://");
                        out.print(host);
                        out.print("/retrobib/");
                        out.print(stat);
                        out.println("</loc>");
                        out.println("       <changefreq>weekly</changefreq>");
                        out.println("   </url>");
                    }
                } else {
                    // sitemap enthält den Kurznamen des Werkes...
                    Werk werk = WerkLogic.getWerkByShortname(sitemap, hiberSession);
                    out.println("   <url>");
                    out.print("       <loc>http://");
                    out.print(host);
                    out.print("/retrobib/stoebern.html?werkid=");
                    out.print(werk.getId());
                    out.println("</loc>");
                    out.print("       <lastmod>");
                    out.print(getGoogleDate(werk.getTimestamp()));
                    out.println("</lastmod>");
                    out.println("       <changefreq>weekly</changefreq>");
                    out.println("   </url>");
                    for (Band band : werk.getBandCollection()) {
                        out.println("   <url>");
                        out.print("       <loc>http://");
                        out.print(host);
                        out.print("/retrobib/stoebern.html?bandid=");
                        out.print(band.getId());
                        out.println("</loc>");
                        out.print("       <lastmod>");
                        out.print(getGoogleDate(band.getTimestamp()));
                        out.println("</lastmod>");
                        out.println("       <changefreq>weekly</changefreq>");
                        out.println("   </url>");
                        for (Teil teil : band.getTeilCollection()) {
                            for (Seite seite : teil.getSeiteCollection()) {
                                out.println("   <url>");
                                out.print("       <loc>http://");
                                out.print(host);
                                out.print("/retrobib/seite.html?id=");
                                out.print(seite.getId());
                                out.println("</loc>");
                                out.print("       <lastmod>");
                                out.print(getGoogleDate(seite.getTimestamp()));
                                out.println("</lastmod>");
                                out.println("       <changefreq>weekly</changefreq>");
                                out.println("   </url>");
                            }
                        }
                    }
                }

                out.println("</urlset>");
            }

            out.flush();
        } catch (Throwable ex) {
            logger.debug("Unkritisch: ", ex);
            response.sendError(404);
        }

        return null;
    }

    /**
     * Ein Datum im Google-Format yyyy-mm-dd bekommen.
     *
     * @param timestamp Der Zeitstempel.
     * @return Das Google-Datum.
     */
    private String getGoogleDate(long timestamp) {
        GregorianCalendar gc = new GregorianCalendar();
        gc.setTimeInMillis(timestamp);
        String year = "" + gc.get(GregorianCalendar.YEAR);
        String month = "" + (gc.get(GregorianCalendar.MONTH) + 1);
        if (month.length() < 2) {
            month = "0" + month;
        }
        String day = "" + gc.get(GregorianCalendar.DATE);
        if (day.length() < 2) {
            day = "0" + day;
        }
        return year + "-" + month + "-" + day;
    }
}
