/**
 * SuggestAction.java
 *
 * Created on 24. Juli 2009, 09:34
 *
 */
package de.retrobib.actions.downloads;

import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.model.Schlagwort;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.hibernate.Session;

/**
 * Action für das Suggest-Feld. Für den Anfang einer Sucheingabe werden mögliche
 * Vervollständigungen zurückgegeben.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class SuggestAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SuggestAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        String query = "";
        if (request.getParameter("suchtext") != null) {
            query = request.getParameter("suchtext").toLowerCase() + "%";
        }

        try {
            logger.debug("suchtext = " + query);

            response.setContentType("text/plain; charset=UTF-8");
            response.setHeader("Cache-Control", "no-cache");
            @SuppressWarnings("unchecked")
            ArrayList<Schlagwort> l = (ArrayList<Schlagwort>) hiberSession.getNamedQuery("Schlagwort by startword").setString("startword", query).setMaxResults(100).list();

            // Distinct nachbauen...
            ArrayList<String> l2 = new ArrayList<String>();
            for (Schlagwort sw : l) {
                if (!l2.contains(sw.getKeyword())) {
                    l2.add(sw.getKeyword());
                }
            }

            StringBuilder responseText = new StringBuilder();
            responseText.append("<ul>");
            for (int i = 0; i < 10 && i < l2.size(); i++) {
                responseText.append("<li>");
                responseText.append(l2.get(i));
                responseText.append("</li>");
            }
            responseText.append("</ul>");

            logger.debug("responseText = " + responseText);
            response.setContentLength(responseText.toString().getBytes("UTF-8").length);
            ServletOutputStream sos = response.getOutputStream();
            sos.write(responseText.toString().getBytes("UTF-8"));
            sos.flush();
        } catch (Throwable ex) {
            logger.debug("Unkritisch: ", ex);
            response.sendError(404);
        }

        return null;
    }
}
