/**
 * TextAction.java
 *
 * Created on 16. August 2007, 09:19
 *
 */
package de.retrobib.actions.downloads;

import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.webelements.Message;
import de.retrobib.xml.Converter;
import java.util.ArrayList;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.TextUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "text_*.txt" (Texte downloaden).
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class TextAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(TextAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        // Id aus dem Namen ([...]/text_*.txt) holen...
        String url = request.getRequestURI();
        String id = null;
        String sid = null;
        Seitentext text = null;

        try {
            logger.debug("URI=" + url);
            if (url.indexOf("_sid_") == -1) {
                id = url.substring(url.indexOf("_") + 1, url.indexOf("."));
                logger.debug("Id für Text ist " + id + ".");
                text = (Seitentext) hiberSession.load(Seitentext.class, Long.parseLong(id));
            } else {
                sid = url.substring(url.indexOf("_sid_") + 5, url.indexOf("."));
                logger.debug("Sid für Text ist " + sid + ".");
                text = (Seitentext) hiberSession.getNamedQuery("Seitentext by seiteid").setLong("seiteid", Long.parseLong(sid)).setMaxResults(1).list().get(0);
            }

            //
            // UTF-8 'Byte Order Mark' voranstellen?
            // --> Wenn Accountsetting o.k. UND TextEncoding UTF-8 UND Download...
            //
            boolean utf8bom = getAccountFromSession(httpSession).getUtf8bom() && (text.getSeiteid().getStatus() != Seite.TAGGED) && text.getEncoding().equals("UTF-8") && (url.indexOf("a_") != -1);

            String fileName = FileNamer.getFileName(text);
            String content = text.getContent();
            if ((url.indexOf("xml_") != -1) || (text.getSeiteid().getStatus() == Seite.TAGGED)) {
                response.setContentType("text/xml; charset=UTF-8");
                fileName += ".xml";
                if (url.indexOf("xml_") != -1) {
                    content = Converter.convert(hiberSession, text.getSeiteid());
                }
                utf8bom = false;
            } else {
                response.setContentType("text/plain; charset=UTF-8");
                fileName += ".txt";
            }

            String dispos = "inline";
            if (url.indexOf("a_") != -1) {
                dispos = "attachment";
            }

            response.setHeader("Content-disposition", dispos + "; filename=" + fileName);
            if (!utf8bom) {
                response.setContentLength(content.getBytes(text.getEncoding()).length);
            } else {
                response.setContentLength(content.getBytes(text.getEncoding()).length + TextUtil.utf8_bom.length);
            }
            ServletOutputStream sos = response.getOutputStream();
            if (utf8bom) {
                sos.write(TextUtil.utf8_bom);
            }
            sos.write(content.getBytes(text.getEncoding()));
            sos.flush();
        } catch (Throwable ex) {
            logger.debug("Unkritisch: ", ex);
            response.sendError(404);
        }

        return null;
    }
}
