/**
 * ZipAction.java
 *
 * Created on 16. August 2007, 09:19
 *
 */
package de.retrobib.actions.downloads;

import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Faksimile;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.db.model.Teil;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.TextUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "paket_*.zipped" (Downloads von Zip-Files).
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class ZipAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(ZipAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @return Der Action-Forward.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    @SuppressWarnings("unchecked")
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        // Id aus dem Namen ([...]/paket_*.zipped) holen...
        String url = request.getRequestURI();

        boolean incTexte = false;
        boolean incBilder = false;

        try {
            logger.debug("URI=" + url);

            ArrayList<Seite> seiten = new ArrayList<Seite>();

            // Paket für einen Korrektor vorbereiten...
            if (url.indexOf("_aid_") != -1) {
                String aid = url.substring(url.indexOf("_aid_") + 5, url.indexOf("."));
                logger.debug("Aid für Zip ist " + aid + ".");
                seiten.addAll(hiberSession.getNamedQuery("ausgeklinkte Seiten").setLong("accountid", Long.parseLong(aid)).list());
                if (url.indexOf("_t_") != -1) {
                    incTexte = true;
                }
                if (url.indexOf("_f_") != -1) {
                    incBilder = true;
                }
                if (url.indexOf("_tf_") != -1) {
                    incTexte = true;
                    incBilder = true;
                }
            }

            // Paket mit Texten eines Bandes vorbereiten...
            if (url.indexOf("_tid_") != -1) {
                String tid = url.substring(url.indexOf("_tid_") + 5, url.indexOf("."));
                logger.debug("Tid für Zip ist " + tid + ".");
                seiten.addAll(hiberSession.getNamedQuery("Seite by teilid").setLong("teilid", Long.parseLong(tid)).list());
                incTexte = true;
            }

            // Paket mit Texten eines Teiles vorbereiten...
            if (url.indexOf("_bid_") != -1) {
                String bid = url.substring(url.indexOf("_bid_") + 5, url.indexOf("."));
                logger.debug("Bid für Zip ist " + bid + ".");
                Band band = (Band) hiberSession.load(Band.class, Long.parseLong(bid));
                List<Teil> teile = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", band.getId()).list();
                for (Teil teil : teile) {
                    seiten.addAll(hiberSession.getNamedQuery("Seite by teilid").setLong("teilid", teil.getId()).list());
                }
                incTexte = true;
            }

            //
            // UTF-8 'Byte Order Mark' voranstellen?
            //
            boolean utf8bom = getAccountFromSession(httpSession).getUtf8bom();

            response.setContentType("application/zip");
            response.setHeader("Content-disposition", "attachment; filename=rb_abzug.zip");

            ServletOutputStream sos = response.getOutputStream();
            ZipOutputStream zos = new ZipOutputStream(sos);

            logger.debug("Packe " + seiten.size() + " Seiten/Bilder in ZIP-File...");
            for (Seite seite : seiten) {
                if (incTexte) {
                    List<Seitentext> texte = (List<Seitentext>) hiberSession.getNamedQuery("Seitentext by seiteid").setLong("seiteid", seite.getId()).list();
                    if (texte.size() > 0) {
                        Seitentext seitentext = texte.get(0);

                        logger.debug("Packe Text mit " + seitentext.getContent().length() + " Zeichen in ZIP-File...");
                        String fileName = FileNamer.getFileName(seitentext);
                        if (seitentext.getSeiteid().getStatus() == Seite.TAGGED) {
                            fileName += ".xml";
                        } else {
                            fileName += ".txt";
                        }
                        ZipEntry entry = new ZipEntry(FileNamer.getPathName(seite) + "/" + fileName);
                        entry.setTime(seitentext.getTimestamp());
                        entry.setMethod(ZipEntry.DEFLATED);

                        zos.putNextEntry(entry);
                        if (utf8bom && seitentext.getEncoding().equals("UTF-8") && (seitentext.getSeiteid().getStatus() != Seite.TAGGED)) {
                            zos.write(TextUtil.utf8_bom);
                        }
                        zos.write(seitentext.getContent().getBytes(seitentext.getEncoding()));
                        zos.closeEntry();
                    }
                }

                if (incBilder) {
                    List<Faksimile> bilder = (List<Faksimile>) hiberSession.getNamedQuery("Faksimile by seiteid").setLong("seiteid", seite.getId()).list();
                    if (bilder.size() > 0) {
                        Faksimile faksimile = bilder.get(0);

                        logger.debug("Packe Bild mit " + faksimile.getScan().length + " Bytes in ZIP-File...");
                        ZipEntry entry = new ZipEntry(FileNamer.getPathName(seite) + "/" + FileNamer.getFileName(faksimile) + ".jpg");
                        entry.setTime(faksimile.getTimestamp());
                        // Bilder müssten eigentlich nicht DEFLATED sein, ich bekomme aber
                        // eine Exception, wenn ich hier STORED angebe. (Request already submitted)
                        entry.setMethod(ZipEntry.DEFLATED);

                        zos.putNextEntry(entry);
                        zos.write(faksimile.getScan());
                        zos.closeEntry();
                    }
                }
            }

            zos.close();
            sos.flush();
        } catch (Throwable ex) {
            logger.debug("Unkritisch: ", ex);
            response.sendError(404);
        }

        return null;
    }
}
