/**
 * AdddataAction.java
 *
 * Created on 29. April 2009, 11:50
 *
 */
package de.retrobib.actions.secure;

import de.retrobib.Constants.ViewNames;
import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.logic.BandLogic;
import de.retrobib.db.logic.FaksimileLogic;
import de.retrobib.db.logic.HiresfaksimileLogic;
import de.retrobib.db.logic.SeiteLogic;
import de.retrobib.db.logic.SeitentextLogic;
import de.retrobib.db.logic.TeilLogic;
import de.retrobib.db.logic.WerkLogic;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Faksimile;
import de.retrobib.db.model.Hiresfaksimile;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.db.model.Teil;
import de.retrobib.db.model.Werk;
import de.retrobib.utils.Mailer;
import de.retrobib.webelements.Message;
import java.io.File;
import java.util.ArrayList;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.DateUtil;
import org.ca.utils.misc.FileUploadWrapper;
import org.ca.utils.misc.MD5Util;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Action für die Seite "secure/adddata.html".
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class AdddataAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(AdddataAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @return Der Action-Forward.
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        String mailAdresse = getAccountFromSession(httpSession).getEmail();

        messages.add(new Message("Startzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

        String werk = request.getParameter("werk");
        String band = request.getParameter("band");
        String teil = request.getParameter("teil");
        String directory = request.getParameter("directory");
        String encoding = request.getParameter("encoding");

        boolean error = false;
        // Nachsehen, ob zu allen Texten lores-Bilder da sind...
        File f = new File(directory);
        if (f.isDirectory()) {
            File textDir = new File(directory + "/texte");
            String[] texteNamen = textDir.list();
            for (String name : texteNamen) {
                if (name.endsWith(".txt")) {
                    File tmp = new File(directory + "/lores/" + name.substring(0, name.length() - 4) + ".jpg");
                    if (!tmp.exists()) {
                        messages.add(new Message(tmp.getName() + " fehlt.", Message.FATAL));
                        error = true;
                    }
                }
            }
        } else {
            messages.add(new Message(directory + " ist kein Verzeichnis.", Message.FATAL));
            error = true;
        }

        Teil t = null;
        // Daten holen...
        try {
            Werk w = WerkLogic.getWerkByShortname(werk, hiberSession);
            Band b = BandLogic.getBandByWerkAndNummer(w, band, hiberSession);
            t = TeilLogic.getTeilByBandAndName(b, teil, hiberSession);
        } catch (Exception ex) {
            error = true;
            messages.add(new Message(ex.getMessage(), Message.FATAL));
        }

        // Fehlerfrei!
        if (!error) {
            if (f.isDirectory()) {
                File textDir = new File(directory + "/texte");
                String[] texteNamen = textDir.list();
                for (String name : texteNamen) {
                    if (name.endsWith(".txt")) {
                        Transaction tx = null;
                        try {
                            tx = hiberSession.beginTransaction();

                            String text = directory + "/texte/" + name;
                            String lores = directory + "/lores/" + name.substring(0, name.length() - 4) + ".jpg";
                            String hires = directory + "/hires/" + name.substring(0, name.length() - 4) + ".jpg";
                            messages.add(new Message("Laden: " + text, Message.INFO));

                            // Seite aufbauen...
                            FileUploadWrapper fuw = new FileUploadWrapper(text, "text/plain");

                            Seite seiteNeu = new Seite();
                            seiteNeu.setBlockmarker(false);
                            seiteNeu.setDeleted(false);
                            seiteNeu.setStatus(0);
                            seiteNeu.setTeilid(t);
                            seiteNeu.setOldid(0l);

                            String[] zeilen = new String(fuw.getData(), encoding).split("\n");
                            if (zeilen[0].length() > 0) {
                                messages.add(new Message("Seite: " + zeilen[0], Message.INFO));
                            } else {
                                messages.add(new Message("Seite: " + zeilen[0], Message.WARN));
                            }
                            seiteNeu.setPagenumber(zeilen[0]);
                            seiteNeu.setSort(zeilen[0]);

                            if (zeilen[2].contains(" - ")) {
                                String[] vonbis = zeilen[1].split(" - ");
                                messages.add(new Message("von: " + vonbis[0] + " bis: " + vonbis[1], Message.INFO));
                                seiteNeu.setStartword(vonbis[0]);
                                seiteNeu.setEndword(vonbis[1]);
                            } else {
                                messages.add(new Message("Von: " + zeilen[2], Message.INFO));
                                seiteNeu.setStartword(zeilen[2]);
                            }

                            SeiteLogic.createSeite(seiteNeu, hiberSession, getAccountFromSession(httpSession), true, false);

                            // Seitentext aufbauen...
                            Seitentext seitentextNeu = new Seitentext();
                            seitentextNeu.setContent(new String(fuw.getData(), encoding));
                            seitentextNeu.setDeleted(false);
                            seitentextNeu.setEncoding("UTF-8");
                            seitentextNeu.setHash(MD5Util.plainStringToMD5(seitentextNeu.getContent()));
                            seitentextNeu.setSeiteid(seiteNeu);
                            seitentextNeu.setOldid(0l);
                            SeitentextLogic.createSeitentext(seitentextNeu, hiberSession, getAccountFromSession(httpSession), true, false);

                            // Lores-Bild aufbauen...
                            fuw = new FileUploadWrapper(lores, "image/jpeg");
                            Faksimile faksimileNeu = new Faksimile();
                            faksimileNeu.setDeleted(false);
                            faksimileNeu.setMimetype("image/jpeg");
                            faksimileNeu.setSeiteid(seiteNeu);
                            faksimileNeu.setScan(fuw.getData());
                            faksimileNeu.setHash(MD5Util.plainStringToMD5(new String(fuw.getData())));
                            faksimileNeu.setOldid(0l);
                            FaksimileLogic.createFaksimile(faksimileNeu, hiberSession, getAccountFromSession(httpSession), true, false);

                            if (new File(hires).exists()) {
                                // Lores-Bild aufbauen...
                                fuw = new FileUploadWrapper(hires, "image/jpeg");
                                Hiresfaksimile hiresfaksimileNeu = new Hiresfaksimile();
                                hiresfaksimileNeu.setDeleted(false);
                                hiresfaksimileNeu.setMimetype("image/jpeg");
                                hiresfaksimileNeu.setSeiteid(seiteNeu);
                                hiresfaksimileNeu.setScan(fuw.getData());
                                hiresfaksimileNeu.setHash(MD5Util.plainStringToMD5(new String(fuw.getData())));
                                hiresfaksimileNeu.setOldid(0l);
                                HiresfaksimileLogic.createHiresfaksimile(hiresfaksimileNeu, hiberSession, getAccountFromSession(httpSession), true, false);
                            } else {
                                messages.add(new Message(hires + " übersprungen, kein File.", Message.INFO));
                            }

                            tx.commit();
                        } catch (Exception ex) {
                            messages.add(new Message(ex.getMessage(), Message.FATAL));
                            tx.rollback();
                        }
                    }
                }
            }
        }

        messages.add(new Message("Endzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

        // eMail versenden
        logger.debug("Sende eMail...");
        new Mailer().send("Benachrichtigung über den Daten-Import", messages, mailAdresse, null);

        return mapping.findForward(ViewNames.adddataView);
    }
}
