/**
 * BatchAction.java
 *
 * Created on 21. August 2007, 12:29
 *
 */
package de.retrobib.actions.secure;

import com.ibm.icu.text.CharsetDetector;
import com.ibm.icu.text.CharsetMatch;
import de.retrobib.Constants.MiscConstants;
import de.retrobib.Constants.RunningTasks;
import de.retrobib.Constants.ViewNames;
import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.logic.MetaLogic;
import de.retrobib.db.logic.SchlagwortLogic;
import de.retrobib.db.logic.SeiteLogic;
import de.retrobib.db.logic.SeitentextLogic;
import de.retrobib.db.logic.SiehewortLogic;
import de.retrobib.db.model.Account;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Schlagwort;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.db.model.Siehewort;
import de.retrobib.db.model.Teil;
import de.retrobib.db.model.Werk;
import de.retrobib.db.model.comperators.SeiteSortComperator;
import de.retrobib.utils.Mailer;
import de.retrobib.webelements.Message;
import de.retrobib.xml.Extractor;
import de.retrobib.xml.Validator;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.DateUtil;
import org.ca.utils.misc.FileUploadWrapper;
import org.ca.utils.misc.RequestParameterFetcher;
import org.ca.utils.misc.RequestUtil;
import org.ca.utils.misc.TextUtil;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Action für die Seite "secure/batch.html".<br />
 * Diverse BatchAction-Verarbeitungen.
 *
 * @author caschoff
 * @version 1.0
 */
public class BatchAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(BatchAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @return Der Action-Forward.
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param mpRequest The multipart-HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    @SuppressWarnings("unchecked")
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest mpRequest, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        String mvid = null;
        // Mail-Adresse holen, der User kann bei langen Aktionen schon weg sein, wenn diese abschließt.
        String mailAdresse = getAccountFromSession(httpSession).getEmail();
        RequestParameterFetcher rpf = new RequestParameterFetcher(mpRequest, MiscConstants.repositoryPath, "UTF-8");

        if (RequestUtil.commandIs("renumber", rpf)) {
            // Alles neu durchnummerieren...
            if (!RunningTasks.renumberRunning) {
                MetaLogic.renumberAll(hiberSession);

                // Abschliessend eMail schicken...
                logger.debug("Sende eMail...");
                new Mailer().send("Benachrichtigung über die Neu-Nummerierung", "Neu-Nummerierung beendet.", mailAdresse, null);

                mpRequest.setAttribute("result", "Die Neu-Nummerierung wurde durchgeführt.");
            } else {
                mpRequest.setAttribute("result", "Die Neu-Nummerierung läuft bereits.");
            }

        } else if (RequestUtil.commandIs("metaAllWerke", rpf)) {
            // Meta-Daten aller Werke ändern...
            // Werke zum Editieren anhängen...
            List<Werk> werke = (List<Werk>) hiberSession.getNamedQuery("Werk").list();
            mpRequest.setAttribute("werke", werke);

        } else if (RequestUtil.commandIs("metaBaende", rpf)) {
            // Meta-Daten aller Bände eines Werkes ändern...
            String werkId = rpf.getParameter("werkid");
            // Bände zum Editieren anhängen...
            List<Band> baende = (List<Band>) hiberSession.getNamedQuery("Band by werkid").setLong("werkid", Long.parseLong(werkId)).list();
            mpRequest.setAttribute("baende", baende);

        } else if (RequestUtil.commandIs("metaTeile", rpf)) {
            // Meta-Daten aller Teile eines Bandes ändern...
            String bandId = rpf.getParameter("bandid");
            // Teile zum Editieren anhängen...
            List<Teil> teile = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", Long.parseLong(bandId)).list();
            mpRequest.setAttribute("teile", teile);

        } else {
            // Seiten wollen bearbeitet sein...
            // Ids von Checkboxen fischen, beginnen mit id_, Seiten holen
            Enumeration<String> enu = rpf.getParameterNames();
            ArrayList<Seite> seiten = new ArrayList<Seite>();
            while (enu.hasMoreElements()) {
                String parName = enu.nextElement();
                if ((rpf.getParameter("allmarked") == null) || (rpf.getParameter("allmarked").equals("m"))) {
                    if (parName.startsWith("id_")) {
                        Seite seite = (Seite) hiberSession.load(Seite.class, Long.parseLong(parName.substring(3, parName.length())));
                        seite.getId();
                        seiten.add(seite);
                    }
                } else {
                    if (parName.startsWith("ida_")) {
                        Seite seite = (Seite) hiberSession.load(Seite.class, Long.parseLong(parName.substring(4, parName.length())));
                        seite.getId();
                        seiten.add(seite);
                    }
                }
                if (parName.startsWith("mv_")) {
                    mvid = parName.substring(3, parName.length());
                    logger.debug("Move-id " + mvid + " bekommen.");
                }
            }
            Collections.sort(seiten, new SeiteSortComperator());

            // was tun? (Info aus Submit-Button)
            if (rpf.getParameter("uploadZip") != null) {
                // Zip-File einspielen...
                logger.debug("Zip-File einspielen...");
                messages.addAll(uploadZip(hiberSession, getAccountFromSession(httpSession), rpf));
                new Mailer().send("Nachricht über den ZIP-Import", messages, mailAdresse, null);

            } else if (rpf.getParameter("changeStatus") != null) {
                // aus der Liste der ausgeklinkten Seiten den Status ändern...
                int status = Integer.parseInt(rpf.getParameter("status"));
                logger.debug("Ändere den Status von " + seiten.size() + " auf " + status + "...");
                messages.addAll(changeStatus(hiberSession, getAccountFromSession(httpSession), seiten, status));

            } else if (rpf.getParameter("uploadText") != null) {
                // aus der Liste der ausgeklinkten Seiten Texte einspielen...
                messages.addAll(uploadText(hiberSession, getAccountFromSession(httpSession), seiten, rpf));

            } else if (rpf.getParameter("deleteFromList") != null) {
                // aus der Liste der ausgeklinkten Seiten diese wieder einklinken...
                logger.debug("Einklinken von " + seiten.size() + " Seiten...");
                messages.addAll(putIn(hiberSession, getAccountFromSession(httpSession), seiten));

            } else if (rpf.getParameter("meta") != null) {
                // aus  diversen Listen Metadaten für Seiten ändern...
                logger.debug("Metadaten für " + seiten.size() + " Seiten ändern...");
                // nur Seiten zum Editieren anhängen...
                mpRequest.setAttribute("seiten", seiten);

            } else if (rpf.getParameter("putout") != null) {
                // aus diversen Listen ausklinken...
                logger.debug("Ausklinken von " + seiten.size() + " Seiten...");
                messages.addAll(putOut(hiberSession, getAccountFromSession(httpSession), seiten));

            } else if (mvid != null) {
                // bleibt nur verschieben aus Stöbern-Liste...
                String target_id = rpf.getParameter("teilid_" + mvid);
                logger.debug("Verschieben von " + seiten.size() + " Seiten (" + mvid + "->" + target_id + ").");
                Teil teil = (Teil) hiberSession.load(Teil.class, Long.parseLong(target_id));
                messages.addAll(moveSeiten(hiberSession, getAccountFromSession(httpSession), teil, seiten));

            } else {
                mpRequest.setAttribute("result", "Es konnte keine Aktion bestimmt werden.");

            }
        }

        return mapping.findForward(ViewNames.batchView);
    }

    /**
     * Eine Zip-Datei mit Texten einspielen.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param rpf Der RequestParameterFetcher (Parser für Multipart-Posts).
     * @throws java.lang.Exception Bei einem Fehler.
     * @return Eine Liste mit den Nachrichten für die View.
     */
    private ArrayList<Message> uploadZip(Session hiberSession, Account account, RequestParameterFetcher rpf)
            throws Exception {
        ArrayList<Message> messages = new ArrayList<Message>();

        // Zip-File mit Texten...
        String tempFileName = MiscConstants.repositoryPath + "/" + System.currentTimeMillis() + ".zip";
        logger.debug("Speichere ZipFile unter '" + tempFileName + "'...");
        rpf.getFile("zipfile").saveAs(tempFileName);
        ZipFile zf = new ZipFile(tempFileName);
        Enumeration eze = zf.entries();
        int ok = 0;
        while (eze.hasMoreElements()) {
            ZipEntry ze = (ZipEntry) eze.nextElement();
            if ((ze.getName().indexOf("MACOSX") == -1) && (ze.getName().toLowerCase().endsWith(".txt") || ze.getName().toLowerCase().endsWith(".xml")) && (ze.getName().indexOf("_tid_") != -1)) {
                logger.debug("Name: " + ze.getName());
                logger.debug("Zeit: " + DateUtil.getFormattedDateString(ze.getTime(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT));
                try {
                    long tid = 0;

                    if (ze.getName().endsWith(".txt")) {
                        tid = Long.parseLong(ze.getName().substring(ze.getName().indexOf("_tid_") + 5, ze.getName().indexOf(".txt")));
                    } else {
                        tid = Long.parseLong(ze.getName().substring(ze.getName().indexOf("_tid_") + 5, ze.getName().indexOf(".xml")));
                    }
                    logger.debug("tid = " + tid);
                    Seitentext seitentext = (Seitentext) hiberSession.load(Seitentext.class, tid);
                    Seite seite = seitentext.getSeiteid();
                    if (seite.getBlockmarker()) {
                        if (seite.getAccountid().getEmail().equals(account.getEmail())) {
                            FileUploadWrapper fuw = new FileUploadWrapper(zf.getInputStream(ze), (int) ze.getSize());

                            String name = ze.getName();
                            String oldText = seitentext.getContent();
                            String newRawText = new String(fuw.getData());
                            String newText = new String(fuw.getData(), "UTF-8");

                            // welche Codierung?
                            messages.addAll(checkUTF8Encoding(name, newRawText));

                            // BOM wegnehmen?
                            if (TextUtil.startsWithBOM(newText)) {
                                logger.debug("Entferne UTF-8-bom...");
                                newText = TextUtil.stripBOM(newText);
                            }

                            // XML?
                            if (isXML(name, newText)) {
                                messages.add(new Message("'" + name + "' ist als XML deklariert, prüfe auf Validität...", Message.INFO));

                                ArrayList<Message> validationMessages = null;
                                Validator v = new Validator();
                                validationMessages = v.validate(newText);
                                messages.addAll(validationMessages);

                                // Fehler?
                                if (!Message.hasErrors(validationMessages)) {
                                    // Nein, alles o.k.
                                    seitentext.setContent(newText);
                                    seitentext.setTimestamp(ze.getTime());
                                    SeitentextLogic.updateSeitentext(seitentext, hiberSession, account, false, true);

                                    seite.setStatus(Seite.TAGGED);
                                    seite.setTimestamp(ze.getTime());
                                    SeiteLogic.updateSeite(seite, hiberSession, account, false, true);
                                    sendBackupMail(account, seite, oldText, newText);
                                    ok++;
                                    messages.addAll(postProcessXMLDocument(hiberSession, seitentext, true));
                                } else {
                                    // ja...
                                    messages.add(new Message("'" + name + "' enthält Fehler und wurde übersprungen.", Message.ERROR));
                                }
                            } else {
                                // kein XML...
                                seitentext.setContent(newText);
                                seitentext.setTimestamp(ze.getTime());
                                SeitentextLogic.updateSeitentext(seitentext, hiberSession, account, false, true);

                                seite.setTimestamp(ze.getTime());
                                if (seite.getStatus() == Seite.UNKORRIGIERT) {
                                    seite.setStatus(Seite.NORMAL);
                                }
                                SeiteLogic.updateSeite(seite, hiberSession, account, false, true);
                                sendBackupMail(account, seite, oldText, newText);
                                ok++;
                            }

                            messages.add(new Message("O.k. für " + name, Message.INFO));
                        } else {
                            messages.add(new Message("Die Seite wurde nicht von Ihnen ausgeklinkt: " + ze.getName(), Message.WARN));
                            messages.add(new Message("Wenden Sie sich an: " + seitentext.getSeiteid().getAccountid().getEmail(), Message.INFO));
                        }
                    } else {
                        messages.add(new Message("Die Seite war nicht ausgeklinkt: " + ze.getName(), Message.WARN));
                    }
                } catch (Exception ex) {
                    messages.add(new Message("Fehler (" + ex.getMessage() + ") bei dem Import von " + ze.getName(), Message.ERROR));
                }
            }
        }
        zf.close();
        messages.add(new Message(ok + " Seiten wurden eingespielt.", Message.INFO));
        File f = new File(tempFileName);
        f.delete();
        messages.add(new Message("Temp-File '" + tempFileName + "' gelöscht.", Message.INFO));

        return messages;
    }

    /**
     * Wechselt den Status von Seiten.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param seiten Die Seiten.
     * @param status Der neue Status.
     * @return Eine Liste mit den Nachrichten für die View.
     */
    private ArrayList<Message> changeStatus(Session hiberSession, Account account, ArrayList<Seite> seiten, int status) {
        ArrayList<Message> messages = new ArrayList<Message>();

        int ok = 0;
        int skipped = 0;
        Transaction tx = hiberSession.beginTransaction();
        for (Seite seite : seiten) {
            if (seite.getStatus() != status) {
                seite.setStatus(status);
                SeiteLogic.updateSeite(seite, hiberSession, account, true, false);
                ok++;
            } else {
                skipped++;
            }
        }
        tx.commit();

        if (ok == 0) {
            messages.add(new Message("Keine Seite wurde geändert.", Message.INFO));
        } else if (ok == 1) {
            messages.add(new Message("Eine Seite wurde geändert.", Message.INFO));
        } else {
            messages.add(new Message(ok + " Seiten wurden geändert.", Message.INFO));
        }

        if (skipped == 0) {
        } else if (skipped == 1) {
            messages.add(new Message("Eine Seite wurde übersprungen weil der Status gleich war.", Message.INFO));
        } else {
            messages.add(new Message(skipped + " Seiten wurden übersprungen weil der Status gleich war.", Message.INFO));
        }

        return messages;
    }

    /**
     * Lädt Texte in das System.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param seiten Die betroffenen Seiten.
     * @param rpf Der RequestParameterFetcher (Parser für Multipart-Posts).
     * @throws java.lang.Exception In einem Fehlerfall.
     * @return Eine Liste mit den Nachrichten für die View.
     */
    private ArrayList<Message> uploadText(Session hiberSession, Account account, ArrayList<Seite> seiten, RequestParameterFetcher rpf)
            throws Exception {
        ArrayList<Message> messages = new ArrayList<Message>();

        int ok = 0;
        int skipped = 0;
        int stripped = 0;

        CharsetDetector cd = new CharsetDetector();

        Transaction tx = hiberSession.beginTransaction();
        for (Seite seite : seiten) {
            FileUploadWrapper fuw = rpf.getFile("file_" + seite.getId());
            if (fuw != null) {
                Seitentext seitentext = (Seitentext) hiberSession.getNamedQuery("Seitentext by seiteid").setLong("seiteid", seite.getId()).setMaxResults(1).list().get(0);

                logger.debug(fuw);

                String fileName = fuw.getFilename();
                long tid = 0;
                try {
                    if (fileName.endsWith(".txt")) {
                        tid = Long.parseLong(fileName.substring(fileName.indexOf("_tid_") + 5, fileName.indexOf(".txt")));
                    } else {
                        tid = Long.parseLong(fileName.substring(fileName.indexOf("_tid_") + 5, fileName.indexOf(".xml")));
                    }
                } catch (Exception ex) {
                    logger.debug("Fehler beim dem Parsen der _tid_.");
                }

                logger.debug("tid = " + tid);

                if (tid == seitentext.getId()) {
                    String name = rpf.getFile("file_" + seite.getId()).getFilename();
                    String oldText = seitentext.getContent();
                    String newRawText = new String(rpf.getFile("file_" + seite.getId()).getData());
                    String newText = new String(rpf.getFile("file_" + seite.getId()).getData(), "UTF-8");

                    // welche Codierung?
                    messages.addAll(checkUTF8Encoding(name, newRawText));

                    // BOM wegnehmen?
                    if (TextUtil.startsWithBOM(newText)) {
                        logger.debug("Entferne UTF-8-bom...");
                        newText = TextUtil.stripBOM(newText);
                        stripped++;
                    }

                    // XML?
                    if (isXML(name, newText)) {
                        messages.add(new Message("'" + name + "' ist als XML deklariert, prüfe auf Validität...", Message.INFO));

                        ArrayList<Message> validationMessages = null;
                        Validator v = new Validator();
                        validationMessages = v.validate(newText);
                        messages.addAll(validationMessages);

                        // Fehler?
                        if (!Message.hasErrors(validationMessages)) {
                            // Nein, alles o.k.
                            seitentext.setContent(newText);
                            SeitentextLogic.updateSeitentext(seitentext, hiberSession, account, true, false);

                            seite.setStatus(Seite.TAGGED);
                            SeiteLogic.updateSeite(seite, hiberSession, account, true, false);
                            sendBackupMail(account, seite, oldText, newText);
                            ok++;
                            messages.addAll(postProcessXMLDocument(hiberSession, seitentext, false));
                        } else {
                            // ja...
                            messages.add(new Message("'" + name + "' enthält Fehler und wurde übersprungen.", Message.ERROR));
                            skipped++;
                        }
                    } else {
                        // kein XML...
                        seitentext.setContent(newText);
                        SeitentextLogic.updateSeitentext(seitentext, hiberSession, account, true, false);

                        if (seite.getStatus() == Seite.UNKORRIGIERT) {
                            seite.setStatus(Seite.NORMAL);
                        }
                        SeiteLogic.updateSeite(seite, hiberSession, account, true, false);
                        sendBackupMail(account, seite, oldText, newText);
                        ok++;
                    }
                } else {
                    skipped++;
                }
            } else {
                skipped++;
            }
        }
        tx.commit();

        if (ok == 0) {
            messages.add(new Message("Kein Text wurde eingespielt.", Message.INFO));
        } else if (ok == 1) {
            messages.add(new Message("Ein Text wurde eingespielt.", Message.INFO));
        } else {
            messages.add(new Message(ok + " Texte wurden eingespielt.", Message.INFO));
        }

        if (skipped == 0) {
        } else if (skipped == 1) {
            messages.add(new Message("Ein Text wurde übersprungen weil kein File gefunden wurde oder die tid nicht stimmt oder die XML-Struktur nicht valide ist.", Message.INFO));
        } else {
            messages.add(new Message(skipped + " Texte wurden übersprungen weil kein File gefunden wurde oder die tid nicht stimmt.", Message.INFO));
        }

        if (stripped == 1) {
            messages.add(new Message("Von einem Text wurde die UTF-8 'Byte Order Mark' entfernt.", Message.INFO));
        } else if (stripped > 1) {
            messages.add(new Message("Von " + stripped + " Texten wurde die UTF-8 'Byte Order Mark' entfernt.", Message.INFO));
        }

        return messages;
    }

    /**
     * Klinkt Seiten ein.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param seiten Die Seiten.
     * @return Eine Liste mit den Nachrichten für die View.
     */
    private ArrayList<Message> putIn(Session hiberSession, Account account, ArrayList<Seite> seiten) {
        ArrayList<Message> messages = new ArrayList<Message>();

        int ok = 0;
        int skipped = 0;
        Transaction tx = hiberSession.beginTransaction();
        for (Seite seite : seiten) {
            if (seite.getBlockmarker()) {
                seite.setBlockmarker(false);
                SeiteLogic.updateSeite(seite, hiberSession, account, true, false);
                ok++;
            } else {
                skipped++;
            }
        }
        tx.commit();

        if (ok == 0) {
            messages.add(new Message("Keine Seite wurde eingeklinkt.", Message.INFO));
        } else if (ok == 1) {
            messages.add(new Message("Eine Seite wurde eingeklinkt.", Message.INFO));
        } else {
            messages.add(new Message(ok + " Seiten wurden eingeklinkt.", Message.INFO));
        }

        if (skipped == 0) {
        } else if (skipped == 1) {
            messages.add(new Message("Eine Seite wurde übersprungen weil sie bereits eingeklinkt war.", Message.INFO));
        } else {
            messages.add(new Message(skipped + " Seiten wurden übersprungen weil sie bereits eingeklinkt waren.", Message.INFO));
        }

        return messages;
    }

    /**
     * Klinkt Seiten aus.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param seiten Die Seiten.
     * @return Eine Liste mit den Nachrichten für die View.
     */
    private ArrayList<Message> putOut(Session hiberSession, Account account, ArrayList<Seite> seiten) {
        ArrayList<Message> messages = new ArrayList<Message>();

        int ok = 0;
        int skipped = 0;
        Transaction tx = hiberSession.beginTransaction();
        for (Seite seite : seiten) {
            if (!seite.getBlockmarker()) {
                seite.setBlockmarker(true);
                SeiteLogic.updateSeite(seite, hiberSession, account, true, false);
                ok++;
            } else {
                skipped++;
            }
        }
        tx.commit();

        if (ok == 0) {
            messages.add(new Message("Keine Seite wurde ausgeklinkt.", Message.INFO));
        } else if (ok == 1) {
            messages.add(new Message("Eine Seite wurde ausgeklinkt.", Message.INFO));
        } else {
            messages.add(new Message(ok + " Seiten wurden ausgeklinkt.", Message.INFO));
        }

        if (skipped == 0) {
        } else if (skipped == 1) {
            messages.add(new Message("Eine Seite wurde übersprungen weil sie bereits ausgeklinkt war.", Message.INFO));
        } else {
            messages.add(new Message(skipped + " Seiten wurden übersprungen weil sie bereits ausgeklinkt waren.", Message.INFO));
        }

        return messages;
    }

    /**
     * Verschiebt Seiten in einen anderen Teil.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param targetTeil Der Teil, in den die Seiten verschoben werden sollen.
     * @param seiten Die Seiten.
     * @return Eine Liste mit den Nachrichten für die View.
     */
    private ArrayList<Message> moveSeiten(Session hiberSession, Account account, Teil targetTeil, ArrayList<Seite> seiten) {
        ArrayList<Message> messages = new ArrayList<Message>();

        Transaction tx = hiberSession.beginTransaction();
        for (Seite seite : seiten) {
            seite.setTeilid(targetTeil);
            SeiteLogic.updateSeite(seite, hiberSession, account, true, false);
        }
        tx.commit();

        if (seiten.size() == 0) {
            messages.add(new Message("Keine Seite wurde nach '" + targetTeil.getName() + "' verschoben.", Message.INFO));
        } else if (seiten.size() == 1) {
            messages.add(new Message("Eine Seite wurde nach '" + targetTeil.getName() + "' verschoben.", Message.INFO));
        } else {
            messages.add(new Message(seiten.size() + " Seiten wurden nach '" + targetTeil.getName() + "' verschoben.", Message.INFO));
        }

        return messages;
    }

    /**
     * Eine neu eingespielte XML-Datei nachbearbeiten.
     * 
     * @return Gegebenfalls aufgetretene Nachrichten.
     * @param hiberSession Die Hibernate-Session.
     * @param seitentext Der Seitentext.
     * @param withTransaction Mit SQL-Transaktionen?
     * @throws java.lang.Exception Bei einem Fehler.
     */
    private ArrayList<Message> postProcessXMLDocument(Session hiberSession, Seitentext seitentext, boolean withTransaction)
            throws Exception {
        ArrayList<Message> ret = new ArrayList<Message>();

        Extractor extractor = new Extractor(seitentext.getContent());
        // bisherige Schlagwörter und Siehewörter löschen...
        @SuppressWarnings("unchecked")
        List<Schlagwort> schlagworte = (List<Schlagwort>) hiberSession.getNamedQuery("Schlagwort by seiteid order by length").setLong("seiteid", seitentext.getSeiteid().getId()).list();
        for (Schlagwort schlagwort : schlagworte) {
            schlagwort.setDeleted(true);
            SchlagwortLogic.updateSchlagwort(schlagwort, hiberSession, seitentext.getAccountid(), true, withTransaction);
        }
        @SuppressWarnings("unchecked")
        List<Siehewort> sieheworte = (List<Siehewort>) hiberSession.getNamedQuery("Siehewort by seiteid order by length").setLong("seiteid", seitentext.getSeiteid().getId()).list();
        for (Siehewort siehewort : sieheworte) {
            siehewort.setDeleted(true);
            SiehewortLogic.updateSiehewort(siehewort, hiberSession, seitentext.getAccountid(), true, withTransaction);
        }
        // und die Neuen einfügen (Sieheworte sind überflüssig, wird über XSL gemacht)...
        for (String wort : extractor.getSchlagWorte()) {
            wort = wort.trim();

            Schlagwort schlagwort = new Schlagwort();
            schlagwort.setDeleted(false);
            schlagwort.setKeyword(wort);
            schlagwort.setSeiteid(seitentext.getSeiteid());
            SchlagwortLogic.createSchlagwort(schlagwort, hiberSession, seitentext.getAccountid(), true, withTransaction);
        }

        return extractor.getMessages();
    }

    /**
     * Prüft einen Text darauf, ob er UTF-8 codiert ist.
     *
     * @param name Der Name der Quelle des Textes für die Nachricht.
     * @param text Der zu prüfende Text.
     * @return Eine Liste mit Nachrichten. Die Nachrichten sind leer, wenn das
     *         Encoding UTF-8 ist, oder enthalten die Warnungen, falls nicht.
     */
    private ArrayList<Message> checkUTF8Encoding(String name, String text) {
        ArrayList<Message> ret = new ArrayList<Message>();

        try {
            CharsetDetector cd = new CharsetDetector();
            cd.setText(text.getBytes());
            CharsetMatch cm = cd.detect();
            logger.debug(cm.getLanguage() + "/" + cm.getName());
            if (!cm.getName().equals("UTF-8")) {
                ret.add(new Message("Achtung! '" + name + "' scheint nicht die Zeichencodierung UTF-8 zu haben (sondern " + cm.getName() + ")!", Message.ERROR));
            }
        } catch (Exception ex) {
            ret.add(new Message("Achtung! Das Encoding von '" + name + "' konnte nicht geprüft werden (" + ex.getMessage() + ")!", Message.FATAL));
        }

        return ret;
    }

    /**
     * Testen einen Text darauf, ob er als XML deklariert ist.
     *
     * @param name Der Dateiname.
     * @param text Der zu prüfende Text.
     * @return <code>true</code> wenn der Text als XML deklariert ist, <code>false</code> andernfalls.
     */
    private boolean isXML(String name, String text) {
        String xmlSignatur = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";

        if ((name != null) && (name.toLowerCase().endsWith(".xml"))) {
            return true;
        }

        if (text.toLowerCase().indexOf(xmlSignatur) != -1) {
            return true;
        }

        return false;
    }
}
