/**
 * ImportAction.java
 *
 * Created on 21. August 2007, 12:29
 *
 */
package de.retrobib.actions.secure;

import de.retrobib.Constants.ViewNames;
import de.retrobib.actions.RetrobibAction;
import de.retrobib.utils.Mailer;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.DateUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "secure/import.html".<br />
 * Ja nach dem, was aus der Verwaltung kommt, wird
 * migriert und die Nachrichten für die View
 * an den <code>request</code> angehängt.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class ImportAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(ImportAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @return Der Action-Forward.
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        String mailAdresse = getAccountFromSession(httpSession).getEmail();

        messages.add(new Message("Startzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

        if (request.getParameter("accounts") != null) {
            messages.add(new Message("Starte Import der Accounts...", Message.INFO));
            messages.add(new Message("... Import der Accounts beendet.", Message.INFO));
        }
        if (request.getParameter("werke") != null) {
            messages.add(new Message("Starte Import der Werke...", Message.INFO));
            messages.add(new Message("... Import der Werke beendet.", Message.INFO));
        }
        if (request.getParameter("baende") != null) {
            messages.add(new Message("Starte Import der Bänder...", Message.INFO));
            messages.add(new Message("... Import der Bänder beendet", Message.INFO));
        }
        if (request.getParameter("seiten") != null) {
            messages.add(new Message("Starte Import der Seiten...", Message.INFO));
            messages.add(new Message("... Import der Seiten beendet", Message.INFO));
        }
        if (request.getParameter("texte") != null) {
            messages.add(new Message("Starte Import der Texte...", Message.INFO));
            messages.add(new Message("... Import der Texte beendet", Message.INFO));
        }
        if (request.getParameter("bilder") != null) {
            messages.add(new Message("Starte Import der Bilder...", Message.INFO));
            messages.add(new Message("... Import der Bilder beendet", Message.INFO));
        }
        if (request.getParameter("hiresbilder") != null) {
            messages.add(new Message("Starte Import der Hires-Bilder...", Message.INFO));
            messages.add(new Message("... Import der Hires-Bilder beendet", Message.INFO));
        }
        if (request.getParameter("verweise") != null) {
            messages.add(new Message("Starte Import der Verweise...", Message.INFO));
            messages.add(new Message("... Import der Verweise beendet", Message.INFO));
        }
        if (request.getParameter("schlagworte") != null) {
            messages.add(new Message("Starte Import der Schlagworte...", Message.INFO));
            messages.add(new Message("... Import der Schlagworte beendet", Message.INFO));
        }

        messages.add(new Message("Endzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

        // eMail versenden
        logger.debug("Sende eMail...");
        new Mailer().send("Benachrichtigung über den Import", messages, mailAdresse, null);

        return mapping.findForward(ViewNames.importView);
    }
}
