/**
 * LuceneAction.java
 *
 * Created on 21. August 2007, 12:29
 *
 */
package de.retrobib.actions.secure;

import de.retrobib.Constants.ViewNames;
import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.logic.MetaLogic;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.hibernate.Session;

/**
 * Action für die Seite "secure/lucene.html".<br />
 * Ja nach dem, was aus der Verwaltung kommt, wird
 * agiert und die Nachrichten für die View
 * an den <code>request</code> angehängt.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class LuceneAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(LuceneAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     * @return Der Action-Forward.
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        String mailAdresse = getAccountFromSession(httpSession).getEmail();

        //
        // Lucene-Index neu aufbauen...
        //
        if (request.getParameter("new") != null) {
            messages.addAll(MetaLogic.createLuceneIndex(hiberSession, mailAdresse));
        }

        //
        // Lucene-Index aktualisieren...
        //
        if (request.getParameter("update") != null) {
            messages.addAll(MetaLogic.updateLuceneIndex(hiberSession, mailAdresse));
        }

        return mapping.findForward(ViewNames.luceneView);
    }
}
