/**
 * ChangeBatchAction.java
 *
 * Created on 15. August 2007, 11:18
 *
 */
package de.retrobib.actions.secure.change;

import de.retrobib.Constants.ViewNames;
import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.logic.BandLogic;
import de.retrobib.db.logic.SeiteLogic;
import de.retrobib.db.logic.TeilLogic;
import de.retrobib.db.logic.WerkLogic;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Teil;
import de.retrobib.db.model.Werk;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.DateUtil;
import org.ca.utils.misc.RequestUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "secure/change/changeBatch.html".
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class ChangeBatchAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(ChangeBatchAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @return Der Action-Forward.
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {

        messages.add(new Message("Startzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));
        ArrayList<Long> ids = RequestUtil.getNotNullNumericIds(request, "id_");

        if (request.getParameter("changeWerke") != null) {
            messages.add(new Message("Starte Stapel-Änderung der Werke...", Message.INFO));
            for (Long id : ids) {
                Werk werk = (Werk) hiberSession.load(Werk.class, id);

                // Werte setzen
                werk.setSort(request.getParameter("sort_" + id));
                werk.setShortname(request.getParameter("shortname_" + id));
                werk.setName(request.getParameter("name_" + id));
                werk.setAuthor(request.getParameter("author_" + id));
                werk.setPublisher(request.getParameter("publisher_" + id));
                werk.setIssued(request.getParameter("issued_" + id));

                WerkLogic.updateWerk(werk, hiberSession, getAccountFromSession(httpSession), true, true);
            }
            messages.add(new Message("Betroffen sind " + ids.size() + " Werke.", Message.INFO));
            messages.add(new Message("... Stapel-Änderung der Werke beendet.", Message.INFO));

        } else if (request.getParameter("changeBaende") != null) {
            messages.add(new Message("Starte Stapel-Änderung der Bände...", Message.INFO));
            for (Long id : ids) {
                Band band = (Band) hiberSession.load(Band.class, id);

                // Werte setzen
                band.setSort(request.getParameter("sort_" + id));
                band.setName(request.getParameter("name_" + id));
                band.setAuthor(request.getParameter("author_" + id));
                band.setPublisher(request.getParameter("publisher_" + id));
                band.setIssued(request.getParameter("issued_" + id));

                BandLogic.updateBand(band, hiberSession, getAccountFromSession(httpSession), true, true);
            }
            messages.add(new Message("Betroffen sind " + ids.size() + " Bände.", Message.INFO));
            messages.add(new Message("... Stapel-Änderung der Bände beendet.", Message.INFO));

        } else if (request.getParameter("changeTeile") != null) {
            messages.add(new Message("Starte Stapel-Änderung der Teile...", Message.INFO));
            for (Long id : ids) {
                Teil teil = (Teil) hiberSession.load(Teil.class, id);

                // Werte setzen
                teil.setSort(request.getParameter("sort_" + id));
                teil.setName(request.getParameter("name_" + id));

                TeilLogic.updateTeil(teil, hiberSession, getAccountFromSession(httpSession), true, true);
            }
            messages.add(new Message("Betroffen sind " + ids.size() + " Teile.", Message.INFO));
            messages.add(new Message("... Stapel-Änderung der Teile beendet.", Message.INFO));

        } else if (request.getParameter("changeSeiten") != null) {
            messages.add(new Message("Starte Stapel-Änderung der Seiten...", Message.INFO));
            for (Long id : ids) {
                Seite seite = (Seite) hiberSession.load(Seite.class, id);

                // Werte setzen
                seite.setSort(request.getParameter("sort_" + id));
                seite.setStartword(request.getParameter("startword_" + id));
                seite.setEndword(request.getParameter("endword_" + id));

                SeiteLogic.updateSeite(seite, hiberSession, getAccountFromSession(httpSession), true, true);
            }
            messages.add(new Message("Betroffen sind " + ids.size() + " Seiten.", Message.INFO));
            messages.add(new Message("... Stapel-Änderung der Seiten beendet.", Message.INFO));

        } else {
            messages.add(new Message("Keine sinnvolle Aktion bekommen!", Message.WARN));
        }

        messages.add(new Message("Endzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

        return mapping.findForward(ViewNames.changebatchView);
    }
}
