/**
 * CreateAccountAction.java
 *
 * Created on 24. November 2007, 11:18
 *
 */
package de.retrobib.actions.secure.create;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.Constants.ViewNames;
import de.retrobib.actions.RetrobibAction;
import de.retrobib.db.logic.AccountLogic;
import de.retrobib.db.logic.AccountsettingLogic;
import de.retrobib.db.model.Account;
import de.retrobib.utils.Mailer;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ca.utils.misc.MD5Util;
import org.ca.utils.misc.PasswordUtil;
import org.hibernate.Session;

/**
 * Action für die Seite "secure/createAccount.html".
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class CreateAccountAction extends RetrobibAction {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(CreateAccountAction.class);

    /**
     * Diese Methode muß jede Action überschreiben.
     *
     * @return Der Action-Forward.
     * @param mapping The ActionMapping used to select this instance.
     * @param form The optional ActionForm bean for this request.
     * @param request The HTTP Request we are processing.
     * @param response The HTTP Response we are processing.
     * @param messages Eventuelle Nachrichten für die View.
     * @param hiberSession Die Hibernate-Session.
     * @param httpSession Die Http-Session.
     * @throws java.lang.Exception Bei einem Fehlerfall.
     */
    protected ActionForward executeAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ArrayList<Message> messages, Session hiberSession, HttpSession httpSession)
            throws Exception {
        String email = request.getParameter("email").toLowerCase();

        if (hiberSession.getNamedQuery("Account by email").setString("email", email).list().size() == 0) {
            Account account = new Account();
            String passwort = PasswordUtil.getTempPassword();
            account.setDeleted(false);
            account.setEmail(email);
            account.setPassword(MD5Util.plainStringToMD5(passwort));
            AccountLogic.createAccount(account, hiberSession, true, true);

            AccountsettingLogic.saveAccountsetting(account, hiberSession, MiscConstants.accountSettingBackupmail, "true");
            AccountsettingLogic.saveAccountsetting(account, hiberSession, MiscConstants.accountSettingUTF8BOM, "true");

            StringBuilder mailText = new StringBuilder();
            mailText.append("Hallo,\n\n");
            mailText.append("willkommen bei der Retrobibliothek. Ihr neuer Account hat die folgenden Daten:\n\n");
            mailText.append("Username=" + email + "\n");
            mailText.append("Passwort=" + passwort + "\n\n");
            mailText.append("Sie können sich mit den angegebenen Daten an der Retrobibliothek anmelden ('Login für Korrektoren' oben rechts). ");
            mailText.append("Als Erstes sollten Sie Ihr Passwort auf einen Wert ändern, der mehr Ihren Vorstellungen entspricht.\n\n");
            mailText.append("Wie geht es jetzt weiter?\n");
            mailText.append("Lesen Sie gegebenenfalls die Anleitungen (im Hauptmenü 'Für Korrektoren'). Wir versuchen, die Anleitungen ");
            mailText.append("kontinuiertlich zu verbessern. Falls Ihr Betriebssystem nicht dabei ist, hilft meist dennoch die Anleitung ");
            mailText.append("für ein anderes Betriebssystem - viele Aspekte sind ja überall gleich.\n");
            mailText.append("Sie sollten sich auch in der 'Meyers-Mailing-Liste' eintragen (auf der Hauptseite). Über diese Liste ");
            mailText.append("kommen ca. 1-2 Mails pro Woche mit Anregungen, Fragen, Verbesserungen und Ankündigungen. Falls Sie Fragen, ");
            mailText.append("Anregungen oder Ähnliches haben, sind Sie natürlich herzlich eingeladen, an die Liste zu schreiben.\n\n");
            mailText.append("Natürlich können Sie sich bei Fragen immer auch an mich wenden.\n");
            mailText.append("Grüße aus Ulm,\n");
            mailText.append("Christian Aschoff\n\n");
            mailText.append("--------------------------------\n");
            mailText.append("caschoff@mac.com\n");
            mailText.append("(Diese eMail wurde automatisch generiert.)\n");

            new Mailer().send("Ihr Account in der Retrobibliothek", mailText.toString(), email, null);

            // den Account anhängen
            request.setAttribute("newaccount", account);
        } else {
            request.setAttribute("hint", "Die eMail-Adresse exisitert schon.");
        }

        return mapping.findForward(ViewNames.createaccountView);
    }
}
