/*
 * DailyJobs.java
 *
 * Created on 31. Oktober 2007, 08:00
 *
 */
package de.retrobib.cron;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.db.logic.MetaLogic;
import de.retrobib.utils.HibernateUtil;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

/**
 * Jobs, die jeden Tag ausgeführt werden sollen. Der Job wird um 3:00 Uhr nachts ausgeführt.
 *
 * @author caschoff
 * @version 1.0
 */
public class DailyJobs implements Job {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(DailyJobs.class);

    /** Creates a new instance of DailyJobs */
    public DailyJobs() {
    }

    /**
     * Den Job ausführen.
     *
     * @param jobExecutionContext Der Kontext.
     * @throws org.quartz.JobExecutionException Bei einem Fehler.
     */
    public void execute(JobExecutionContext jobExecutionContext)
            throws JobExecutionException {
        logger.debug("Job-Ausführung (Daily).");

        logger.debug("Öffne statische Hibernate-Session.");
        @SuppressWarnings("deprecation")
        Session session = HibernateUtil.getSessionFactory().openSession();
        try {
            //
            // Datenprüfung
            //
            logger.debug("Datenprüfung...");
            MetaLogic.checkAccounts(session, MiscConstants.adminMail);
            MetaLogic.checkData(session, MiscConstants.adminMail);

            //
            // Datenabgleich mit dem alten System
            //
            /*
            ArrayList<Message> messages = new ArrayList<Message>();
            messages.add(new Message("Startzeit: "+DateUtil.getFriendlyDateString(), Message.INFO));
            logger.info("Migration der Daten. Nur während der Beta-Phase!");
            messages.addAll(Migrator.migrateAccounts(session));
            messages.addAll(Migrator.migrateSeiten(session));
            messages.addAll(Migrator.migrateTexte(session));
            messages.addAll(Migrator.migrateVerweise(session));
            messages.addAll(Migrator.migrateSchlagworte(session));
            messages.add(new Message("Endzeit: "+DateUtil.getFriendlyDateString(), Message.INFO));
            // eMail versenden
            logger.debug("Sende eMail...");
            new Mailer().send("Benachrichtigung über den Import", messages, MiscConstants.adminMail, null);
             */

            //
            // Lucene-Index
            //
            logger.debug("Lucene-Index aktualisieren...");
            MetaLogic.updateLuceneIndex(session, MiscConstants.adminMail);
        } catch (Exception ex) {
            throw new JobExecutionException(ex);
        } finally {
            logger.debug("Schliesse statische Hibernate-Session.");
            session.close();
        }
    }
}
