/*
 * BandLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Werk;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Bände.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class BandLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(BandLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private BandLogic() {
    }

    /**
     * Einen Band aufgrund des Werkes und einer Bandnummer bekommen.
     *
     * @param werk Das Werk zu dem der Band gehört.
     * @param number Die Bandnummer.
     * @param hiberSession Die Hibernate-Session.
     *
     * @return Der Band.
     */
    public static Band getBandByWerkAndNummer(Werk werk, String number, Session hiberSession) {
        Band band = null;
        @SuppressWarnings("unchecked")
        List<Band> l = (List<Band>) hiberSession.getNamedQuery("Band by werkid and nummer").setLong("werkid", werk.getId()).setString("number", number).list();
        if (l.size() == 1) {
            band = l.get(0);
        }

        return band;
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Einen Band anlegen.
     *
     * @param band Der Band.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createBand(Band band, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            band.setTimestamp(System.currentTimeMillis());
        }
        band.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + band);
        hiberSession.persist(band);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Einen Band aktualisieren.
     *
     * @param band Der Band.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateBand(Band band, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            band.setTimestamp(System.currentTimeMillis());
        }
        if (account != null) {
            band.setAccountid(account);
        }

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + band);
        hiberSession.update(band);
        if (withTransaction) {
            tx.commit();
        }
    }
}
