/*
 * FaksimileLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Faksimile;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Faksimiles.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class FaksimileLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(FaksimileLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private FaksimileLogic() {
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Ein Faksimile anlegen.
     *
     * @param faksimile Das Faksimile.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createFaksimile(Faksimile faksimile, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            faksimile.setTimestamp(System.currentTimeMillis());
        }
        faksimile.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + faksimile);
        hiberSession.persist(faksimile);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Ein Faksimile aktualisieren.
     *
     * @param faksimile Das Faksimile.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateFaksimile(Faksimile faksimile, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            faksimile.setTimestamp(System.currentTimeMillis());
        }
        faksimile.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + faksimile);
        hiberSession.update(faksimile);
        if (withTransaction) {
            tx.commit();
        }
    }
}
