/*
 * GlobalsettingLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Globalsetting;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Globalsettings.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class GlobalsettingLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(GlobalsettingLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private GlobalsettingLogic() {
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Ein Globalsetting speichern.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param name Der Name des Settings.
     * @param value Der Wert des Settings.
     */
    public static void saveGlobalsetting(Session hiberSession, String name, String value) {
        @SuppressWarnings("unchecked")
        List<Globalsetting> l = (List<Globalsetting>) hiberSession.getNamedQuery("Globalsetting").list();

        for (Globalsetting gs : l) {
            if (gs.getName().equals(name)) {
                gs.setValue(value);
                Transaction tx = hiberSession.beginTransaction();
                logger.debug("Update: " + gs);
                hiberSession.update(gs);
                tx.commit();

                return;
            }
        }

        Globalsetting globalSetting = new Globalsetting();

        globalSetting.setDeleted(false);
        globalSetting.setTimestamp(System.currentTimeMillis());
        globalSetting.setName(name);
        globalSetting.setValue(value);

        Transaction tx = hiberSession.beginTransaction();
        logger.debug("Create " + globalSetting);
        hiberSession.persist(globalSetting);
        tx.commit();
    }

    /**
     * Ein Globalsetting bekommen.
     * 
     * @return Das Setting oder <code>null</code>, wenn das Setting nicht existiert.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param name Der Name des Settings.
     */
    public static Globalsetting getGlobalsetting(Session hiberSession, String name) {
        @SuppressWarnings("unchecked")
        List<Globalsetting> l = (List<Globalsetting>) hiberSession.getNamedQuery("Globalsetting by name").setString("name", name).list();

        if (l.size() > 0) {
            return l.get(0);
        }

        return null;
    }

    /**
     * Ein Globalsetting löschen.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param name Der Name des Settings.
     */
    public static void deleteGlobalsetting(Session hiberSession, String name) {
        Globalsetting gs = getGlobalsetting(hiberSession, name);

        if (gs != null) {
            gs.setDeleted(true);
            Transaction tx = hiberSession.beginTransaction();
            logger.debug("Update: " + gs);
            hiberSession.update(gs);
            tx.commit();
        }
    }
}
