/*
 * HiresfaksimileLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Hiresfaksimile;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für hires-Faksimiles.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class HiresfaksimileLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(HiresfaksimileLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private HiresfaksimileLogic() {
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Ein Hires-Faksimile anlegen.
     *
     * @param hiresfaksimile Das Hires-Faksimile.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createHiresfaksimile(Hiresfaksimile hiresfaksimile, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            hiresfaksimile.setTimestamp(System.currentTimeMillis());
        }
        hiresfaksimile.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + hiresfaksimile);
        hiberSession.persist(hiresfaksimile);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Ein Hires-Faksimile aktualisieren.
     *
     * @param hiresfaksimile Das Hires-Faksimile.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateHiresfaksimile(Hiresfaksimile hiresfaksimile, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            hiresfaksimile.setTimestamp(System.currentTimeMillis());
        }
        hiresfaksimile.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + hiresfaksimile);
        hiberSession.update(hiresfaksimile);
        if (withTransaction) {
            tx.commit();
        }
    }
}
