/*
 * MetaLogic.java
 *
 * Created on 11. September 2007, 17:02
 *
 */
package de.retrobib.db.logic;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.Constants.RunningTasks;
import de.retrobib.db.model.Account;
import de.retrobib.db.model.Band;
import de.retrobib.db.model.Faksimile;
import de.retrobib.db.model.Globalsetting;
import de.retrobib.db.model.Hiresfaksimile;
import de.retrobib.db.model.Schlagwort;
import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Seitentext;
import de.retrobib.db.model.Siehewort;
import de.retrobib.db.model.Teil;
import de.retrobib.db.model.Werk;
import de.retrobib.lucene.Manager;
import de.retrobib.lucene.Sherlock;
import de.retrobib.utils.Mailer;
import de.retrobib.webelements.Message;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import org.apache.log4j.Logger;
import org.ca.utils.misc.DateUtil;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logic, die verschiedene Objekte umspannt.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class MetaLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(MetaLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private MetaLogic() {
    }

    /**
     * Die Konsistenz der Accounts prüfen.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param mailAdresse Die eMail-Adresse für das Ergebnis.
     * @return Eine Liste mit Nachrichten über den Verlauf.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public static ArrayList<Message> checkAccounts(Session hiberSession, String mailAdresse)
            throws Exception {
        ArrayList<Message> messages = new ArrayList<Message>();

        if (!RunningTasks.checkAccountsRunning) {
            RunningTasks.checkAccountsRunning = true;

            messages.add(new Message("Startzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

            messages.add(new Message("Start Prüfung der Accounts...", Message.INFO));
            ArrayList<Account> accounts = (ArrayList<Account>) AccountLogic.getAllAccountsWithDeletions(hiberSession);
            messages.add(new Message("Überprüfe " + accounts.size() + " Accounts...", Message.INFO));
            for (Account account : accounts) {
                if (account.getDeleted()) {
                    messages.add(new Message("Überprüfe " + account, Message.INFO));

                    // Daten von gelöschten Accounts entfernen...
                    for (Werk werk : account.getWerkCollection()) {
                        werk.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(werk + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        WerkLogic.updateWerk(werk, hiberSession, null, false, true);
                    }

                    for (Band band : account.getBandCollection()) {
                        band.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(band + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        BandLogic.updateBand(band, hiberSession, null, false, true);
                    }

                    for (Teil teil : account.getTeilCollection()) {
                        teil.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(teil + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        TeilLogic.updateTeil(teil, hiberSession, null, false, true);
                    }

                    for (Seite seite : account.getSeiteCollection()) {
                        seite.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(seite + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        SeiteLogic.updateSeite(seite, hiberSession, null, false, true);
                    }

                    for (Faksimile faksimile : account.getFaksimileCollection()) {
                        faksimile.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(faksimile + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        FaksimileLogic.updateFaksimile(faksimile, hiberSession, null, false, true);
                    }

                    for (Hiresfaksimile hirresfaksimile : account.getHiresfaksimileCollection()) {
                        hirresfaksimile.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(hirresfaksimile + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        HiresfaksimileLogic.updateHiresfaksimile(hirresfaksimile, hiberSession, null, false, true);
                    }

                    for (Seitentext seitentext : account.getSeitentextCollection()) {
                        seitentext.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(seitentext + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        SeitentextLogic.updateSeitentext(seitentext, hiberSession, null, false, true);
                    }

                    for (Siehewort siehewort : account.getSiehewortCollection()) {
                        siehewort.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(siehewort + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        SiehewortLogic.updateSiehewort(siehewort, hiberSession, null, false, true);
                    }

                    for (Schlagwort schlagwort : account.getSchlagwortCollection()) {
                        schlagwort.setAccountid(AccountLogic.getAccountByEmail("caschoff@mac.com", hiberSession));
                        messages.add(new Message(schlagwort + " von gelöschtem Account abgekoppelt.", Message.INFO));
                        SchlagwortLogic.updateSchlagwort(schlagwort, hiberSession, null, false, true);
                    }
                }
            }
            messages.add(new Message("... Prüfung der Accounts beendet.", Message.INFO));

            messages.add(new Message("Endzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

            RunningTasks.checkAccountsRunning = false;
        } else {
            messages.add(new Message("Die Accounts werden bereits geprüft.", Message.INFO));
        }

        // eMail versenden
        logger.debug("Sende eMail...");
        new Mailer().send("Benachrichtigung über den Account-Check", messages, mailAdresse, null);

        return messages;
    }

    /**
     * Die Konsistenz der Daten prüfen.
     *
     * @param hiberSession Die Hibernate-Session.
     * @param mailAdresse Die eMail-Adresse für das Ergebnis.
     * @return Eine Liste mit Nachrichten über den Verlauf.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public static ArrayList<Message> checkData(Session hiberSession, String mailAdresse)
            throws Exception {
        ArrayList<Message> messages = new ArrayList<Message>();

        if (!RunningTasks.checkDataRunning) {
            RunningTasks.checkDataRunning = true;

            messages.add(new Message("Startzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

            messages.add(new Message("Starte Prüfung der Daten...", Message.INFO));
            // Von Oben nach Unten durch Alles durchgehen...
            @SuppressWarnings("unchecked")
            List<Werk> werkListe = (List<Werk>) hiberSession.getNamedQuery("Werk").list();
            for (int werkCount = 0; werkCount < werkListe.size(); werkCount++) {
                Werk werk = werkListe.get(werkCount);
                messages.add(new Message(werk.getName(), Message.INFO));
                logger.debug("Checker->Werk " + werkCount + "/" + werkListe.size());
                @SuppressWarnings("unchecked")
                List<Band> bandListe = (List<Band>) hiberSession.getNamedQuery("Band by werkid").setLong("werkid", werk.getId()).list();
                for (int bandCount = 0; bandCount < bandListe.size(); bandCount++) {
                    Band band = bandListe.get(bandCount);
                    messages.add(new Message(band.getName(), Message.INFO));
                    logger.debug("Checker->Band " + bandCount + "/" + bandListe.size());
                    @SuppressWarnings("unchecked")
                    List<Teil> teilListe = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", band.getId()).list();
                    for (int teilCount = 0; teilCount < teilListe.size(); teilCount++) {
                        Teil teil = teilListe.get(teilCount);
                        messages.add(new Message(teil.getName(), Message.INFO));
                        @SuppressWarnings("unchecked")
                        List<Seite> seitenListe = (List<Seite>) hiberSession.getNamedQuery("Seite by teilid").setLong("teilid", teil.getId()).list();
                        messages.add(new Message("Seiten: " + seitenListe.size(), Message.INFO));

                        for (int seitenCount = 0; seitenCount < seitenListe.size(); seitenCount++) {
                            Seite seite = seitenListe.get(seitenCount);

                            // Ist die Seite zu lange (> 30 Tage) ausgeklinkt?
                            if (seite.getBlockmarker()) {
                                if (seite.getTimestamp() < System.currentTimeMillis() - (30l * 24l * 60l * 60l * 1000l)) {
                                    seite.setBlockmarker(false);
                                    SeiteLogic.updateSeite(seite, hiberSession, null, true, true);
                                    messages.add(new Message("Automatisches einklinken von " + seite, Message.INFO));
                                    new Mailer().send("Nachricht vom automatischen System-Check", "Die Seite " + seite + " (http://www.retrobibliothek.de/retrobib/seite.html?id=" + seite.getId() + ") war länger als 30 Tage von Ihnen ausgeklinkt und wurde wieder eingeklinkt. Bei Bedarf können Sie die Seite wieder ausklinken.", seite.getAccountid().getEmail(), null);
                                }

                            }

                            Seitentext seitentext = seite.getSeitentextCollection().iterator().next();
                            // Gibt es die Schlagworte?
                            for (Schlagwort sw : seite.getSchlagwortCollection()) {
                                if (seitentext.getContent().indexOf(sw.getKeyword()) == -1) {
                                    messages.add(new Message("Das Schlagwort '<b>" + sw.getKeyword() + "</b>' ist nicht auf der <a href=\"/retrobib/seite.html?id=" + seite.getId() + "\">Seite</a> " + seite.getPagenumber() + ".", Message.WARN));
                                }
                            }
                            // Gibt es die Sieheworte?
                            for (Siehewort se : seite.getSiehewortCollection()) {
                                if (seitentext.getContent().indexOf(se.getSeeword()) == -1) {
                                    messages.add(new Message("Das Siehewort '<b>" + se.getSeeword() + "</b>' ist nicht auf der <a href=\"/retrobib/seite.html?id=" + seite.getId() + "\">Seite</a> " + seite.getPagenumber() + ".", Message.WARN));
                                }
                            }
                        }
                    }
                    hiberSession.flush();
                    hiberSession.clear();
                }
            }

            messages.add(new Message("... Prüfung der Daten beendet.", Message.INFO));
            messages.add(new Message("Endzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

            RunningTasks.checkDataRunning = false;
        } else {
            messages.add(new Message("Die Daten werden bereits geprüft.", Message.INFO));
        }

        // eMail versenden
        logger.debug("Sende eMail...");
        new Mailer().send("Benachrichtigung über den Daten-Check", messages, mailAdresse, null);

        return messages;
    }

    /**
     * Den Lucene-Index neu aufbauen.
     *
     * @return Eine Liste von Meldungen.
     * @param mailAdresse Die eMail-Adresse für die Benachrichtigung.
     * @param hiberSession Die Hibernate-Session.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public static ArrayList<Message> createLuceneIndex(Session hiberSession, String mailAdresse)
            throws Exception {
        ArrayList<Message> messages = new ArrayList<Message>();

        if (!RunningTasks.luceneRunning) {
            RunningTasks.luceneRunning = true;

            GlobalsettingLogic.saveGlobalsetting(hiberSession, MiscConstants.globalSettingLastLuceneUpdate, "" + System.currentTimeMillis());

            messages.add(new Message("Startzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));
            messages.add(new Message("Starte Neuaufbau des Lucene-Indexes...", Message.INFO));

            // Damit es keine Probleme mit dem Reader gibt, der öffnet sich dann selbst wieder
            Sherlock.close();

            Manager manager = new Manager(true);
            manager.open();
            // Von Oben nach Unten durch Alles durchgehen...
            @SuppressWarnings("unchecked")
            List<Werk> werkListe = (List<Werk>) hiberSession.getNamedQuery("Werk").list();
            for (int werkCount = 0; werkCount < werkListe.size(); werkCount++) {
                Werk werk = werkListe.get(werkCount);
                messages.add(new Message(werk.getName(), Message.INFO));
                logger.debug("Lucene->Werk " + werkCount + "/" + werkListe.size());
                @SuppressWarnings("unchecked")
                List<Band> bandListe = (List<Band>) hiberSession.getNamedQuery("Band by werkid").setLong("werkid", werk.getId()).list();
                for (int bandCount = 0; bandCount < bandListe.size(); bandCount++) {
                    Band band = bandListe.get(bandCount);
                    messages.add(new Message(band.getName(), Message.INFO));
                    logger.debug("Lucene->Band " + bandCount + "/" + bandListe.size());
                    @SuppressWarnings("unchecked")
                    List<Teil> teilListe = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", band.getId()).list();
                    for (int teilCount = 0; teilCount < teilListe.size(); teilCount++) {
                        Teil teil = teilListe.get(teilCount);
                        messages.add(new Message(teil.getName(), Message.INFO));
                        @SuppressWarnings("unchecked")
                        List<Seite> seitenListe = (List<Seite>) hiberSession.getNamedQuery("Seite by teilid").setLong("teilid", teil.getId()).list();
                        messages.add(new Message("Seiten: " + seitenListe.size(), Message.INFO));

                        for (int seitenCount = 0; seitenCount < seitenListe.size(); seitenCount++) {
                            Seite seite = seitenListe.get(seitenCount);
                            manager.addToLucene(seite, hiberSession);
                        }
                    }
                    hiberSession.flush();
                    hiberSession.clear();
                }
            }

            manager.optimize();
            messages.add(new Message("Optimierung des Lucene-Indexes...", Message.INFO));

            manager.close();
            messages.add(new Message("... Neuaufbau des Lucene-Indexes beendet.", Message.INFO));

            messages.add(new Message("Starte Aufbau des 'Did you mean...'-Index...", Message.INFO));
            manager.createDidYouMeanIndex();
            messages.add(new Message("... Aufbau des 'Did you mean...'-Index beendet.", Message.INFO));

            // Damit es keine Probleme mit dem Reader gibt, der öffnet sich dann selbst wieder
            Sherlock.close();

            messages.add(new Message("Endzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

            RunningTasks.luceneRunning = false;
        } else {
            messages.add(new Message("Der Lucene-Index wird bereits aufgebaut.", Message.INFO));
        }

        // eMail versenden
        logger.debug("Sende eMail...");
        new Mailer().send("Benachrichtigung über den Lucene-Index", messages, mailAdresse, null);

        return messages;
    }

    /**
     * Den Lucene-Index aktualisieren.
     *
     * @return Eine Liste von Meldungen.
     * @param mailAdresse Die eMail-Adresse für die Benachrichtigung.
     * @param hiberSession Die Hibernate-Session.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public static ArrayList<Message> updateLuceneIndex(Session hiberSession, String mailAdresse)
            throws Exception {
        ArrayList<Message> messages = new ArrayList<Message>();

        if (!RunningTasks.luceneRunning) {
            RunningTasks.luceneRunning = true;

            messages.add(new Message("Startzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));
            messages.add(new Message("Starte Aktualisierung des Lucene-Indexes...", Message.INFO));

            Globalsetting gs = GlobalsettingLogic.getGlobalsetting(hiberSession, MiscConstants.globalSettingLastLuceneUpdate);
            if (gs != null) {
                long lastUpdate = Long.parseLong(gs.getValue());
                @SuppressWarnings("unchecked")
                List<Seite> l = (List<Seite>) hiberSession.getNamedQuery("Seite younger than ts").setLong("ts", lastUpdate).list();

                if (l.size() > 0) {
                    messages.add(new Message(l.size() + " Seiten werden aktualisiert.", Message.INFO));
                    // Damit es keine Probleme mit dem Reader gibt, der öffnet sich dann selbst wieder
                    Sherlock.close();

                    Manager manager = new Manager(false);

                    // aus Index löschen...
                    manager.deleteFromLucene(l);

                    // in Index aufnehmen...
                    manager.open();

                    GlobalsettingLogic.saveGlobalsetting(hiberSession, MiscConstants.globalSettingLastLuceneUpdate, "" + System.currentTimeMillis());
                    for (int i = 0; i < l.size(); i++) {
                        manager.addToLucene(l.get(i), hiberSession);
                    }

                    manager.optimize();
                    messages.add(new Message("Optimierung des Lucene-Indexes...", Message.INFO));

                    manager.close();
                    messages.add(new Message("... Aktualisierung des Lucene-Indexes beendet.", Message.INFO));

                    messages.add(new Message("Starte Aufbau des 'Did you mean...'-Index...", Message.INFO));
                    manager.createDidYouMeanIndex();
                    messages.add(new Message("... Aufbau des 'Did you mean...'-Index beendet.", Message.INFO));

                    // Damit es keine Probleme mit dem Reader gibt, der öffnet sich dann selbst wieder
                    Sherlock.close();
                } else {
                    messages.add(new Message("Keine Seiten für die Aktualisierung gefunden.", Message.INFO));
                }
            } else {
                messages.add(new Message("Der Zeitpunkt der letzten Aktualisierung konnte nicht ermittelt werden.", Message.FATAL));
            }

            messages.add(new Message("... Aktualisierung des Lucene-Indexes beendet.", Message.INFO));

            messages.add(new Message("Endzeit: " + DateUtil.getFormattedDateString(System.currentTimeMillis(), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT), Message.INFO));

            RunningTasks.luceneRunning = false;
        } else {
            messages.add(new Message("Der Lucene-Index wird bereits aktualisiert.", Message.INFO));
        }

        // eMail versenden
        logger.debug("Sende eMail...");
        new Mailer().send("Benachrichtigung über den Lucene-Index", messages, mailAdresse, null);

        return messages;
    }

    /**
     * Den Sort aller Elemente neu nummerieren. Hierbei werden weder Accounts
     * noch Timestamps aktualisiert. Das ist eher eine einmalige Aktion nach
     * dem Datenimport.
     *
     * @param hiberSession Die Hibernate-Session.
     */
    public static void renumberAll(Session hiberSession) {
        RunningTasks.renumberRunning = true;

        Transaction tx = null;

        int wCount = 0;
        int bCount = 0;
        int tCount = 0;
        int sCount = 0;
        @SuppressWarnings("unchecked")
        List<Werk> werke = (List<Werk>) hiberSession.getNamedQuery("Werk").list();
        for (int werkCount = 0; werkCount < werke.size(); werkCount++) {
            Werk werk = werke.get(werkCount);
            werk.setSort(fillZeros(wCount, 3));
            tx = hiberSession.beginTransaction();
            hiberSession.update(werk);
            tx.commit();
            wCount += 10;
            @SuppressWarnings("unchecked")
            List<Band> baende = (List<Band>) hiberSession.getNamedQuery("Band by werkid").setLong("werkid", werk.getId()).list();
            for (int bandCount = 0; bandCount < baende.size(); bandCount++) {
                Band band = baende.get(bandCount);
                band.setSort(fillZeros(bCount, 4));
                band.setNumber("" + (bandCount + 1));
                tx = hiberSession.beginTransaction();
                hiberSession.update(band);
                tx.commit();
                bCount += 10;
                @SuppressWarnings("unchecked")
                List<Teil> teile = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", band.getId()).list();
                for (int teilCount = 0; teilCount < teile.size(); teilCount++) {
                    Teil teil = teile.get(teilCount);
                    teil.setSort(fillZeros(tCount, 5));
                    tx = hiberSession.beginTransaction();
                    hiberSession.update(teil);
                    tx.commit();
                    tCount += 10;
                    @SuppressWarnings("unchecked")
                    List<Seite> seiten = (List<Seite>) hiberSession.getNamedQuery("Seite by teilid").setLong("teilid", teil.getId()).list();
                    tx = hiberSession.beginTransaction();
                    for (int seitenCount = 0; seitenCount < seiten.size(); seitenCount++) {
                        Seite seite = seiten.get(seitenCount);
                        seite.setSort(fillZeros(sCount, 6));
                        hiberSession.update(seite);
                        sCount += 10;
                    }

                    tx.commit();
                }

                hiberSession.flush();
                hiberSession.clear();
            }
        }

        RunningTasks.renumberRunning = false;
    }

    /**
     * Eine Zahl mit führenden Nullen bekommen.
     *
     * @param number Die Zahl.
     * @param count Die Anzahl der Gesamtstellen.
     * @return Die Zahl als String mit führenden Nullen.
     */
    private static String fillZeros(int number, int count) {
        String ret = new String();

        ret += "" + number;
        while (ret.length() < count) {
            ret = "0" + ret;
        }

        return ret;
    }
}
