/*
 * SchlagwortLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Schlagwort;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Schlagworte.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class SchlagwortLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SchlagwortLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private SchlagwortLogic() {
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Ein Schlagwort anlegen.
     *
     * @param schlagwort Das Schlagwort.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createSchlagwort(Schlagwort schlagwort, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            schlagwort.setTimestamp(System.currentTimeMillis());
        }
        schlagwort.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + schlagwort);
        hiberSession.persist(schlagwort);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Ein Schlagwort aktualisieren.
     *
     * @param schlagwort Das Schlagwort.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateSchlagwort(Schlagwort schlagwort, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            schlagwort.setTimestamp(System.currentTimeMillis());
        }
        schlagwort.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + schlagwort);
        hiberSession.update(schlagwort);
        if (withTransaction) {
            tx.commit();
        }
    }
}
