/*
 * SeitentextLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Seitentext;
import org.apache.log4j.Logger;
import org.ca.utils.misc.MD5Util;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Werke.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class SeitentextLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SeitentextLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private SeitentextLogic() {
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Einen Seitentext anlegen.
     *
     * @param seitentext Der Seitentext.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createSeitentext(Seitentext seitentext, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            seitentext.setTimestamp(System.currentTimeMillis());
        }
        seitentext.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + seitentext);
        hiberSession.persist(seitentext);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Einen Seitentext aktualisieren.
     *
     * @param seitentext Der Seitentext.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateSeitentext(Seitentext seitentext, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction)
            throws Exception {
        if (withTimeUpdate) {
            seitentext.setTimestamp(System.currentTimeMillis());
        }
        if (account != null) {
            seitentext.setAccountid(account);
        }
        seitentext.setHash(MD5Util.plainStringToMD5(seitentext.getContent()));

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + seitentext);
        hiberSession.update(seitentext);
        if (withTransaction) {
            tx.commit();
        }
    }
}
