/*
 * SiehewortLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Siehewort;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Sieheworte.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class SiehewortLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SiehewortLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private SiehewortLogic() {
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Ein Siehewort anlegen.
     *
     * @param siehewort Das Siehewort.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createSiehewort(Siehewort siehewort, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            siehewort.setTimestamp(System.currentTimeMillis());
        }
        siehewort.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + siehewort);
        hiberSession.persist(siehewort);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Eine Siehewort aktualisieren.
     *
     * @param siehewort Das Siehewort.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateSiehewort(Siehewort siehewort, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            siehewort.setTimestamp(System.currentTimeMillis());
        }
        siehewort.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + siehewort);
        hiberSession.update(siehewort);
        if (withTransaction) {
            tx.commit();
        }
    }
}
