/*
 * WerkLogic.java
 *
 * Created on 11. September 2007, 13:28
 *
 */
package de.retrobib.db.logic;

import de.retrobib.db.model.Account;
import de.retrobib.db.model.Werk;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Logik für Werke.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class WerkLogic {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(WerkLogic.class);

    /** Alles ist statisch. Kein Konstruktor von Nöten. */
    private WerkLogic() {
    }

    /**
     * Ein Werk aufgrund seines Kurznamens bekommen.
     * 
     * @param shortname Der Kurzname des Werkes.
     * @param hiberSession Die Hibernate-Session.
     * 
     * @return Das Werk.
     */
    public static Werk getWerkByShortname(String shortname, Session hiberSession) {
        Werk werk = null;
        @SuppressWarnings("unchecked")
        List<Werk> l = (List<Werk>) hiberSession.getNamedQuery("Werk by shortname").setString("shortname", shortname).list();
        if (l.size() == 1) {
            werk = l.get(0);
        }

        return werk;
    }

    /*
     * CRUD-Methoden.
     */
    /**
     * Ein Werk anlegen.
     *
     * @param werk Das Werk.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void createWerk(Werk werk, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            werk.setTimestamp(System.currentTimeMillis());
        }
        werk.setAccountid(account);

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Create " + werk);
        hiberSession.persist(werk);
        if (withTransaction) {
            tx.commit();
        }
    }

    /**
     * Ein Werk aktualisieren.
     *
     * @param werk Das Werk.
     * @param hiberSession Die Hibernate-Session.
     * @param account Der initiierende Account.
     * @param withTimeUpdate Soll die Timestamp aktualisiert werden?
     * @param withTransaction Soll die Aktion in eine Transaktion gepackt werden?
     */
    public static void updateWerk(Werk werk, Session hiberSession, Account account, boolean withTimeUpdate, boolean withTransaction) {
        if (withTimeUpdate) {
            werk.setTimestamp(System.currentTimeMillis());
        }
        if (account != null) {
            werk.setAccountid(account);
        }

        Transaction tx = null;
        if (withTransaction) {
            tx = hiberSession.beginTransaction();
        }
        logger.debug("Update " + werk);
        hiberSession.update(werk);
        if (withTransaction) {
            tx.commit();
        }
    }
}
