/*
 * Account.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.db.logic.AccountsettingLogic;
import java.io.Serializable;
import java.util.Collection;
import java.util.Locale;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Version;
import org.ca.utils.misc.DateUtil;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Account
 * 
 * 21.11.2007: 1.1 - getLastlogin hinzugefügt.
 *
 * @author caschoff
 * @version 1.1
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "account")
@NamedQueries({
    @NamedQuery(name = "Account.count", query = "select count(id) from Account where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account.count by oldid", query = "select count(id) from Account where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account.count by email", query = "select count(id) from Account where email=:email", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account.validation", query = "select count(id) from Account where email=:email and password=:password and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account by data", query = "from Account where email=:email and password=:password and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account by email", query = "from Account where email=:email and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account by oldid", query = "from Account where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account with deletions", query = "from Account", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Account", query = "from Account where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Account implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_account_id")
    private Long id;
    /**
     * Die ID aus dem alten Meyers.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_account_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Der Zeitpunkt der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_account_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_account_deleted")
    private boolean deleted;
    /**
     * Die E-Mail-Adresse.
     */
    @Column(name = "email", nullable = false, length = 256)
    @Index(name = "i_account_email")
    private String email;
    /**
     * Das Passwort als MD5-Hash.
     */
    @Column(name = "password", nullable = false, length = 32)
    @Index(name = "i_account_password")
    private String password;
    /*
     * Die Beziehungen.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Seite> seiteCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Seitentext> seitentextCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Faksimile> faksimileCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Band> bandCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Schlagwort> schlagwortCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Accountsetting> accountsettingCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Hiresfaksimile> hiresfaksimileCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Siehewort> siehewortCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Werk> werkCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "accountid")
    private Collection<Teil> teilCollection;

    /** Creates a new instance of Account */
    public Account() {
    }

    /**
     * Creates a new instance of Account with the specified values.
     * @param id the id of the Account
     */
    public Account(Long id) {
        this.id = id;
    }

    /**
     * Ist der Account ein Master-Account?
     *
     * @return <code>true</code>, wenn der Account ein Master-Account ist,
     * <code>false</code> andernfalls.
     */
    public boolean getMaster() {
        Accountsetting as = AccountsettingLogic.getAccountsetting(this, MiscConstants.accountSettingMaster);
        if ((as != null) && as.getValue().equals("true")) {
            return true;
        }

        return false;
    }

    /**
     * Will der Benutzer BackupMails?
     *
     * @return <code>true</code>, wenn der Benutzer BackupMails möchte,
     * <code>false</code> andernfalls.
     */
    public boolean getBackupmail() {
        Accountsetting as = AccountsettingLogic.getAccountsetting(this, MiscConstants.accountSettingBackupmail);
        if ((as != null) && as.getValue().equals("true")) {
            return true;
        }

        return false;
    }

    /**
     * Will der Benutzer die UTF-8 'Byte Order Mark' am Anfang von Texten?
     *
     * @return <code>true</code>, wenn der Benutzer die BOM möchte,
     * <code>false</code> andernfalls.
     */
    public boolean getUtf8bom() {
        Accountsetting as = AccountsettingLogic.getAccountsetting(this, MiscConstants.accountSettingUTF8BOM);
        if ((as != null) && as.getValue().equals("true")) {
            return true;
        }

        return false;
    }

    /**
     * Den Zeitpunkt des letzten Logins bekommen.
     *
     * @return Der Zeitpunkt des letzten Logins als <i>Friendly String</i>.
     */
    public String getLastlogin() {
        Accountsetting as = AccountsettingLogic.getAccountsetting(this, MiscConstants.accountSettingLastLogin);

        if (as != null) {
            return DateUtil.getFormattedDateString(Long.parseLong(as.getValue()), Locale.GERMAN, DateUtil.LANG_DATUM_ZEIT);
        }

        return null;
    }

    /*
     * Getter/Setter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getEmail() {
        return this.email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getPassword() {
        return this.password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public Collection<Seite> getSeiteCollection() {
        return this.seiteCollection;
    }

    public void setSeiteCollection(Collection<Seite> seiteCollection) {
        this.seiteCollection = seiteCollection;
    }

    public Collection<Seitentext> getSeitentextCollection() {
        return this.seitentextCollection;
    }

    public void setSeitentextCollection(Collection<Seitentext> seitentextCollection) {
        this.seitentextCollection = seitentextCollection;
    }

    public Collection<Faksimile> getFaksimileCollection() {
        return this.faksimileCollection;
    }

    public void setFaksimileCollection(Collection<Faksimile> faksimileCollection) {
        this.faksimileCollection = faksimileCollection;
    }

    public Collection<Band> getBandCollection() {
        return this.bandCollection;
    }

    public void setBandCollection(Collection<Band> bandCollection) {
        this.bandCollection = bandCollection;
    }

    public Collection<Schlagwort> getSchlagwortCollection() {
        return this.schlagwortCollection;
    }

    public void setSchlagwortCollection(Collection<Schlagwort> schlagwortCollection) {
        this.schlagwortCollection = schlagwortCollection;
    }

    public Collection<Accountsetting> getAccountsettingCollection() {
        return this.accountsettingCollection;
    }

    public void setAccountsettingCollection(Collection<Accountsetting> accountsettingCollection) {
        this.accountsettingCollection = accountsettingCollection;
    }

    public Collection<Hiresfaksimile> getHiresfaksimileCollection() {
        return this.hiresfaksimileCollection;
    }

    public void setHiresfaksimileCollection(Collection<Hiresfaksimile> hiresfaksimileCollection) {
        this.hiresfaksimileCollection = hiresfaksimileCollection;
    }

    public Collection<Siehewort> getSiehewortCollection() {
        return this.siehewortCollection;
    }

    public void setSiehewortCollection(Collection<Siehewort> siehewortCollection) {
        this.siehewortCollection = siehewortCollection;
    }

    public Collection<Werk> getWerkCollection() {
        return this.werkCollection;
    }

    public void setWerkCollection(Collection<Werk> werkCollection) {
        this.werkCollection = werkCollection;
    }

    public Collection<Teil> getTeilCollection() {
        return this.teilCollection;
    }

    public void setTeilCollection(Collection<Teil> teilCollection) {
        this.teilCollection = teilCollection;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Account.  The result is
     * <code>true</code> if and only if the argument is not null and is a Account object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Account)) {
            return false;
        }
        Account other = (Account) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Account [id = ");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; email = ");
        ret.append(email);
        ret.append("; password(hash) = ");
        ret.append(password);
        ret.append("]");

        return ret.toString();
    }
}
