/*
 * Band.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import java.util.Collection;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Band
 *
 * @author caschoff
 * @version 1.0
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "band")
@NamedQueries({
    @NamedQuery(name = "Band", query = "from Band where deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Band incDel", query = "from Band order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Band.count", query = "select count(id) from Band where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Band.count by oldid", query = "select count(id) from Band where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Band by oldid", query = "from Band where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Band by werkid", query = "from Band where werkid=:werkid and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Band by werkid and nummer", query = "from Band where werkid=:werkid and number=:number and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Band implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_band_id")
    private Long id;
    /**
     * Die ID aus dem alten Meyers.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_band_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_band_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_band_deleted")
    private boolean deleted;
    /**
     * Sichtbar?
     */
    @Column(name = "visible", nullable = false)
    @Index(name = "i_band_visible")
    private boolean visible;
    /**
     * Der Sortierungsschlüssel.
     */
    @Column(name = "sort", nullable = false, length = 32)
    @Index(name = "i_band_sort")
    private String sort;
    /**
     * Die Bandnummer.
     */
    @Column(name = "number", nullable = true, length = 32)
    @Index(name = "i_band_number")
    private String number;
    /**
     * Der Bandname.
     */
    @Column(name = "name", nullable = false, length = 2048)
    @Index(name = "i_band_name")
    private String name;
    /**
     * Der Publizierer/Verlag.
     */
    @Column(name = "publisher", nullable = false, length = 2048)
    @Index(name = "i_band_publisher")
    private String publisher;
    /**
     * Der Autor.
     */
    @Column(name = "author", nullable = false, length = 2048)
    @Index(name = "i_band_author")
    private String author;
    /**
     * Erscheinungsdatum.
     */
    @Column(name = "issued", nullable = false, length = 256)
    @Index(name = "i_band_issued")
    private String issued;
    /**
     * Eine Beschreibung.
     */
    @Column(name = "description", nullable = true, length = 2048)
    @Index(name = "i_band_description")
    private String description;
    /**
     * Der Account, der die letzte Änderung durchgeführt hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_band_accountid")
    @Index(name = "i_band_accountid")
    private Account accountid;
    /**
     * Das zugehörige Werk.
     */
    @JoinColumn(name = "werkid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_band_werkid")
    @Index(name = "i_band_werkid")
    private Werk werkid;
    /**
     * Die zugehörigen Teile.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "bandid")
    private Collection<Teil> teilCollection;

    /** Creates a new instance of Band */
    public Band() {
    }

    /**
     * Creates a new instance of Band with the specified values.
     * @param id the id of the Band
     */
    public Band(Long id) {
        this.id = id;
    }

    /*
     * Getter/Setter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public boolean getVisible() {
        return this.visible;
    }

    public void setVisible(boolean visible) {
        this.visible = visible;
    }

    public String getSort() {
        return this.sort;
    }

    public void setSort(String sort) {
        this.sort = sort;
    }

    public String getNumber() {
        return this.number;
    }

    public void setNumber(String number) {
        this.number = number;
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getPublisher() {
        return this.publisher;
    }

    public void setPublisher(String publisher) {
        this.publisher = publisher;
    }

    public String getAuthor() {
        return this.author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public String getIssued() {
        return this.issued;
    }

    public void setIssued(String issued) {
        this.issued = issued;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    public Werk getWerkid() {
        return this.werkid;
    }

    public void setWerkid(Werk werkid) {
        this.werkid = werkid;
    }

    public Collection<Teil> getTeilCollection() {
        return this.teilCollection;
    }

    public void setTeilCollection(Collection<Teil> teilCollection) {
        this.teilCollection = teilCollection;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Band.  The result is
     * <code>true</code> if and only if the argument is not null and is a Band object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Band)) {
            return false;
        }
        Band other = (Band) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Band [id = ");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; visible = ");
        ret.append(visible);
        ret.append("; sort = ");
        ret.append(sort);
        ret.append("; number = ");
        ret.append(number);
        ret.append("; name = ");
        ret.append(name);
        ret.append("; publisher = ");
        ret.append(publisher);
        ret.append("; author = ");
        ret.append(author);
        ret.append("; issued = ");
        ret.append(issued);
        ret.append("; description = ");
        ret.append(description);
        ret.append("]");

        return ret.toString();
    }
}
