/*
 * Faksimile.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Faksimile
 *
 * @author caschoff
 * @version 1.0
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "faksimile")
@NamedQueries({
    @NamedQuery(name = "Faksimile.count", query = "select count(id) from Faksimile where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Faksimile.count by seiteid", query = "select count(id) from Faksimile where seiteid=:seiteid and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Faksimile.count by id", query = "select count(id) from Faksimile where id=:id and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Faksimile by seiteid", query = "from Faksimile where seiteid=:seiteid and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Faksimile.part by seiteid", query = "select f.id, f.timestamp from Faksimile f where f.seiteid=:seiteid and f.deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Faksimile implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_faksimile_id")
    private Long id;
    /**
     * Die ID aus dem alten Meyers.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_faksimile_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_faksimile_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_faksimile_deleted")
    private boolean deleted;
    /**
     * Ein MD5-Hash über die Bilddaten.
     */
    @Column(name = "hash", nullable = true, length = 32)
    @Index(name = "i_faksimile_hash")
    private String hash;
    /**
     * Die Bilddaten.
     */
    @Lob
    @Column(name = "scan", nullable = true)
    private byte[] scan;
    /**
     * Der Mimetype.
     */
    @Column(name = "mimetype", nullable = false, length = 256)
    @Index(name = "i_faksimile_mimetype")
    private String mimetype;
    /**
     * Der Account, der die letzte Änderung durchgeführt hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_faksimile_accountid")
    @Index(name = "i_faksimile_accountid")
    private Account accountid;
    /**
     * Die Seite, die der Scan repräsentiert.
     */
    @JoinColumn(name = "seiteid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_faksimile_seiteid")
    @Index(name = "i_faksimile_seiteid")
    private Seite seiteid;

    /** Creates a new instance of Faksimile */
    public Faksimile() {
    }

    /**
     * Creates a new instance of Faksimile with the specified values.
     * @param id the id of the Faksimile
     */
    public Faksimile(Long id) {
        this.id = id;
    }

    /*
     * Getter/Setter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getHash() {
        return this.hash;
    }

    public void setHash(String hash) {
        this.hash = hash;
    }

    public byte[] getScan() {
        return this.scan;
    }

    public void setScan(byte[] scan) {
        this.scan = scan;
    }

    public String getMimetype() {
        return this.mimetype;
    }

    public void setMimetype(String mimetype) {
        this.mimetype = mimetype;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    public Seite getSeiteid() {
        return this.seiteid;
    }

    public void setSeiteid(Seite seiteid) {
        this.seiteid = seiteid;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Faksimile.  The result is
     * <code>true</code> if and only if the argument is not null and is a Faksimile object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Faksimile)) {
            return false;
        }
        Faksimile other = (Faksimile) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Faksimile [id=");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; hash = ");
        ret.append(hash);
        ret.append("; scan(size) = ");
        ret.append(scan.length);
        ret.append("; mimetype = ");
        ret.append(mimetype);
        ret.append("]");

        return ret.toString();
    }
}
