/*
 * Schlagwort.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Schlagwort
 *
 * 27.11.2007: 1.1 - NamedQuery erweitert.
 *
 * @author caschoff
 * @version 1.1
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "schlagwort")
@NamedQueries({
    @NamedQuery(name = "Schlagwort.count", query = "select count(id) from Schlagwort where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Schlagwort", query = "from Schlagwort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Schlagwort by keyword", query = "from Schlagwort where keyword=:keyword and deleted=false order by seiteid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Schlagwort by startword", query = "from Schlagwort s where lower(s.keyword) like :startword and s.deleted=false order by s.keyword", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Schlagwort by seiteid order by length", query = "from Schlagwort where deleted=false and seiteid=:seiteid order by length(keyword) desc", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Schlagwort by seiteid", query = "from Schlagwort where deleted=false and seiteid=:seiteid order by keyword", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Schlagwort implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_schlagwort_id")
    private Long id;
    /**
     * Die ID aus dem alten Meyers.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_schlagwort_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Ändderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_schlagwort_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_schlagwort_deleted")
    private boolean deleted;
    /**
     * Das Schlagwort.
     */
    @Column(name = "keyword", nullable = false, length = 256)
    @Index(name = "i_schlagwort_keyword")
    private String keyword;
    /**
     * Existiert das Schlagwort im neuen Meyers?
     */
    @Column(name = "bifab", nullable = true)
    @Index(name = "i_schlagwort_bifab")
    private Boolean bifab;
    /**
     * Der Account, der die letzte Änderung durchgeführt hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_schlagwort_accountid")
    @Index(name = "i_schlagwort_accountid")
    private Account accountid;
    /**
     * Die Seite, auf der das Schlagwort ist.
     */
    @JoinColumn(name = "seiteid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_schlagwort_seiteid")
    @Index(name = "i_schlagwort_seiteid")
    private Seite seiteid;

    /** Creates a new instance of Schlagwort */
    public Schlagwort() {
    }

    /**
     * Creates a new instance of Schlagwort with the specified values.
     * @param id the id of the Schlagwort
     */
    public Schlagwort(Long id) {
        this.id = id;
    }

    /*
     * Getter/Setter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getKeyword() {
        return this.keyword;
    }

    public void setKeyword(String keyword) {
        this.keyword = keyword;
    }

    public Boolean getBifab() {
        return this.bifab;
    }

    public void setBifab(Boolean bifab) {
        this.bifab = bifab;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    public Seite getSeiteid() {
        return this.seiteid;
    }

    public void setSeiteid(Seite seiteid) {
        this.seiteid = seiteid;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Schlagwort.  The result is
     * <code>true</code> if and only if the argument is not null and is a Schlagwort object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Schlagwort)) {
            return false;
        }
        Schlagwort other = (Schlagwort) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Schlagwort [id=");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; keyword = ");
        ret.append(keyword);
        ret.append("]");

        return ret.toString();
    }
}
