/*
 * Seite.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import java.util.Collection;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Seite
 *
 * @author caschoff
 * @version 1.0
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "seite")
@NamedQueries({
    @NamedQuery(name = "Seite.count", query = "select count(id) from Seite where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite.count by oldid", query = "select count(id) from Seite where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite.count by account", query = "select count(id) from Seite where accountid=:accountid and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite.count by account and blocked", query = "select count(id) from Seite where accountid=:accountid and blockmarker=true and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite.count by teilid and status", query = "select count(id) from Seite where teilid=:teilid and status=:status and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite by teilid", query = "from Seite where teilid=:teilid and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite by oldid", query = "from Seite where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Naechste Seite", query = "from Seite where teilid=:teilid and sort > :sort and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Vorhergehende Seite", query = "from Seite where teilid=:teilid and sort < :sort and deleted = false order by sort desc", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Erste Seite", query = "from Seite where teilid=:teilid and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Letzte Seite", query = "from Seite where teilid=:teilid and deleted = false order by sort desc", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "ausgeklinkte Seiten", query = "from Seite where blockmarker=true and accountid=:accountid and deleted = false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite younger than ts", query = "from Seite where deleted=false and timestamp>:ts", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite by werk, band and nummer", query = "from Seite s where s.pagenumber like :nummer and s.teilid.bandid.number=:band and s.teilid.bandid.werkid.shortname=:werk and s.deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seite", query = "from Seite where deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Seite implements Serializable {

    static final long serialVersionUID = 1l;
    /*
     * Statis der Seiten
     */
    /** Rohe Seiten aus der OCR. */
    public static final int UNKORRIGIERT = 0;
    /** Erster Korrekturlauf. */
    public static final int NORMAL = 1;
    /** Eigennamen korrigiert. */
    public static final int FEIN = 3;
    /** XML. */
    public static final int TAGGED = 7;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_seite_id")
    private Long id;
    /**
     * Die ID aus dem alten Meyres.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_seite_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_seite_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_seite_deleted")
    private boolean deleted;
    /**
     * Der Sortierungsschlüssel.
     */
    @Column(name = "sort", nullable = false, length = 32)
    @Index(name = "i_seite_sort")
    private String sort;
    /**
     * Die Seitennummer.
     */
    @Column(name = "pagenumber", nullable = true, length = 32)
    @Index(name = "i_seite_pagenumber")
    private String pagenumber;
    /**
     * Von Schlagwort?
     */
    @Column(name = "startword", nullable = true, length = 256)
    @Index(name = "i_seite_startword")
    private String startword;
    /**
     * Bis Schlagwort?
     */
    @Column(name = "endword", nullable = true, length = 256)
    @Index(name = "i_seite_endword")
    private String endword;
    /**
     * Der Status der Korrektur.
     */
    @Column(name = "status", nullable = false)
    @Index(name = "i_seite_status")
    private int status;
    /**
     * Ist die Seite ausgeklinkt?
     */
    @Column(name = "blockmarker", nullable = false)
    @Index(name = "i_seite_blockmarker")
    private boolean blockmarker;
    /**
     * Marker: Hat die Seite keine Schlagworte?
     */
    @Column(name = "nokeywords", nullable = false)
    @Index(name = "i_seite_nokeywords")
    private boolean nokeywords;
    /**
     * Marker: Hat die Seite keine Verweise?
     */
    @Column(name = "noseewords", nullable = false)
    @Index(name = "i_seite_noseewords")
    private boolean noseewords;
    /**
     * Der Account, der die letzte Änderung durchgeführt hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_seite_accountid")
    @Index(name = "i_seite_accountid")
    private Account accountid;
    /**
     * Zu welchem Teil gehört die Seite?
     */
    @JoinColumn(name = "teilid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_seite_teilid")
    @Index(name = "i_seite_teilid")
    private Teil teilid;
    /**
     * Der Text der Seite.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "seiteid")
    private Collection<Seitentext> seitentextCollection;
    /**
     * Das Faksimile der Seite.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "seiteid")
    private Collection<Faksimile> faksimileCollection;
    /**
     * Die Schlagworte der Seite.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "seiteid")
    private Collection<Schlagwort> schlagwortCollection;
    /**
     * Die hochaufgelösten Faksimiles der Seite.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "seiteid")
    private Collection<Hiresfaksimile> hiresfaksimileCollection;
    /**
     * Die Verweise auf der Seite.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "seiteid")
    private Collection<Siehewort> siehewortCollection;

    /** Creates a new instance of Seite */
    public Seite() {
    }

    /**
     * Creates a new instance of Seite with the specified values.
     * @param id the id of the Seite
     */
    public Seite(Long id) {
        this.id = id;
    }

    /*
     * Setter/Getter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getSort() {
        return this.sort;
    }

    public void setSort(String sort) {
        this.sort = sort;
    }

    public String getPagenumber() {
        return this.pagenumber;
    }

    public void setPagenumber(String pagenumber) {
        this.pagenumber = pagenumber;
    }

    public String getStartword() {
        return this.startword;
    }

    public void setStartword(String startword) {
        this.startword = startword;
    }

    public String getEndword() {
        return this.endword;
    }

    public void setEndword(String endword) {
        this.endword = endword;
    }

    public int getStatus() {
        return this.status;
    }

    public void setStatus(int status) {
        this.status = status;
    }

    public boolean getBlockmarker() {
        return this.blockmarker;
    }

    public void setBlockmarker(boolean blockmarker) {
        this.blockmarker = blockmarker;
    }

    public boolean getNokeywords() {
        return this.nokeywords;
    }

    public void setNokeywords(boolean nokeywords) {
        this.nokeywords = nokeywords;
    }

    public boolean getNoseewords() {
        return this.noseewords;
    }

    public void setNoseewords(boolean noseewords) {
        this.noseewords = noseewords;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    public Teil getTeilid() {
        return this.teilid;
    }

    public void setTeilid(Teil teilid) {
        this.teilid = teilid;
    }

    public Collection<Seitentext> getSeitentextCollection() {
        return this.seitentextCollection;
    }

    public void setSeitentextCollection(Collection<Seitentext> seitentextCollection) {
        this.seitentextCollection = seitentextCollection;
    }

    public Collection<Faksimile> getFaksimileCollection() {
        return this.faksimileCollection;
    }

    public void setFaksimileCollection(Collection<Faksimile> faksimileCollection) {
        this.faksimileCollection = faksimileCollection;
    }

    public Collection<Schlagwort> getSchlagwortCollection() {
        return this.schlagwortCollection;
    }

    public void setSchlagwortCollection(Collection<Schlagwort> schlagwortCollection) {
        this.schlagwortCollection = schlagwortCollection;
    }

    public Collection<Hiresfaksimile> getHiresfaksimileCollection() {
        return this.hiresfaksimileCollection;
    }

    public void setHiresfaksimileCollection(Collection<Hiresfaksimile> hiresfaksimileCollection) {
        this.hiresfaksimileCollection = hiresfaksimileCollection;
    }

    public Collection<Siehewort> getSiehewortCollection() {
        return this.siehewortCollection;
    }

    public void setSiehewortCollection(Collection<Siehewort> siehewortCollection) {
        this.siehewortCollection = siehewortCollection;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Seite.  The result is
     * <code>true</code> if and only if the argument is not null and is a Seite object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Seite)) {
            return false;
        }
        Seite other = (Seite) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Seite [id = ");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; sort = ");
        ret.append(sort);
        ret.append("; account = ");
        ret.append(accountid);
        ret.append("; pagenumber = ");
        ret.append(pagenumber);
        ret.append("; startword = ");
        ret.append(startword);
        ret.append("; endword = ");
        ret.append(endword);
        ret.append("; status = ");
        ret.append(status);
        ret.append("; blockmarker = ");
        ret.append(blockmarker);
        ret.append("; noseewords = ");
        ret.append(noseewords);
        ret.append("; nokeywords = ");
        ret.append(nokeywords);
        ret.append("]");

        return ret.toString();
    }
}
