/*
 * Seitentext.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Seitentext
 *
 * @author caschoff
 * @version 1.0
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "seitentext")
@NamedQueries({
    @NamedQuery(name = "Seitentext.count by seiteid", query = "select count(id) from Seitentext where seiteid=:seiteid and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seitentext.count by id", query = "select count(id) from Seitentext where id=:id and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Seitentext by seiteid", query = "from Seitentext where seiteid=:seiteid and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Seitentext implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_seitentext_id")
    private Long id;
    /**
     * Die ID aus dem alten Meyers.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_seitentext_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_seitentext_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_seitentext_deleted")
    private boolean deleted;
    /**
     * Ein MD5-Hash über den Text.
     */
    @Column(name = "hash", nullable = true, length = 32)
    @Index(name = "i_seitentext_hash")
    private String hash;
    /**
     * Der Test selbst.
     */
    @Column(name = "content", nullable = true, length = 40960)
    private String content;
    /**
     * Das Character-Encoding des Textes.
     */
    @Column(name = "encoding", nullable = false, length = 32)
    @Index(name = "i_seitentext_encoding")
    private String encoding;
    /**
     * Der Account, der die letzte Änderung durchgeführt hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_seitentext_accountid")
    @Index(name = "i_seitentext_accountid")
    private Account accountid;
    /**
     * Die Seite, zu der der Text gehört.
     */
    @JoinColumn(name = "seiteid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_seitentext_seiteid")
    @Index(name = "i_seitentext_seiteid")
    private Seite seiteid;

    /** Creates a new instance of seitentext */
    public Seitentext() {
    }

    /**
     * Creates a new instance of seitentext with the specified values.
     * @param id the id of the seitentext
     */
    public Seitentext(Long id) {
        this.id = id;
    }

    /*
     * Setter/Getter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getHash() {
        return this.hash;
    }

    public void setHash(String hash) {
        this.hash = hash;
    }

    public String getContent() {
        return this.content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public String getEncoding() {
        return this.encoding;
    }

    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    public Seite getSeiteid() {
        return this.seiteid;
    }

    public void setSeiteid(Seite seiteid) {
        this.seiteid = seiteid;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this seitentext.  The result is
     * <code>true</code> if and only if the argument is not null and is a seitentext object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Seitentext)) {
            return false;
        }
        Seitentext other = (Seitentext) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.seitentext [id=");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; hash = ");
        ret.append(hash);
        ret.append("; content(size) = ");
        ret.append(content.length());
        ret.append("; encoding = ");
        ret.append(encoding);
        ret.append("]");

        return ret.toString();
    }
}
