/*
 * Siehewort.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Siehewort
 *
 * @author caschoff
 * @version 1.0
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "siehewort")
@NamedQueries({
    @NamedQuery(name = "Siehewort.count", query = "select count(id) from Siehewort where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Siehewort", query = "from Siehewort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Siehewort by seiteid order by length", query = "from Siehewort where deleted=false and seiteid=:seiteid order by length(seeword) desc", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Siehewort by seiteid", query = "from Siehewort where deleted=false and seiteid=:seiteid order by seeword", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Siehewort implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_siehewort_id")
    private Long id;
    /**
     * Die ID aus dem alten Meyers.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_siehewort_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_siehewort_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_siehewort_deleted")
    private boolean deleted;
    /**
     * Das Siehewort.
     */
    @Column(name = "seeword", nullable = false, length = 256)
    @Index(name = "i_siehewort_seeword")
    private String seeword;
    /**
     * Der Account, der die letzte Änderung durchgeführt hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_siehewort_accountid")
    @Index(name = "i_siehewort_accountid")
    private Account accountid;
    /**
     * Die Seite, auf der der Artikel ist, auf den das Siehewort verweist.
     */
    @JoinColumn(name = "seiteid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_siehewort_seiteid")
    @Index(name = "i_siehewort_seiteid")
    private Seite seiteid;

    /** Creates a new instance of Siehewort */
    public Siehewort() {
    }

    /**
     * Creates a new instance of Siehewort with the specified values.
     * @param id the id of the Siehewort
     */
    public Siehewort(Long id) {
        this.id = id;
    }

    /*
     * Setter/Getter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getSeeword() {
        return this.seeword;
    }

    public void setSeeword(String seeword) {
        this.seeword = seeword;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    public Seite getSeiteid() {
        return this.seiteid;
    }

    public void setSeiteid(Seite seiteid) {
        this.seiteid = seiteid;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Siehewort.  The result is
     * <code>true</code> if and only if the argument is not null and is a Siehewort object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Siehewort)) {
            return false;
        }
        Siehewort other = (Siehewort) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Siehewort [id=");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; seeword = ");
        ret.append(seeword);
        ret.append("]");

        return ret.toString();
    }
}
