/*
 * Teil.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import java.util.Collection;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Teil
 *
 * @author caschoff
 * @version 1.0
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "teil")
@NamedQueries({
    @NamedQuery(name = "Teil.count by bandid", query = "select count(id) from Teil where bandid=:bandid and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Teil by bandid", query = "from Teil where bandid=:bandid and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Teil by bandid and name", query = "from Teil where bandid=:bandid and name=:name and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Naechster Teil", query = "from Teil where bandid=:bandid and sort > :sort and deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Vorhergehender Teil", query = "from Teil where bandid=:bandid and sort < :sort and deleted = false order by sort desc", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Teil", query = "from Teil where deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Teil implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_teil_id")
    private Long id;
    /**
     * Die Version fpr Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_teil_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_teil_deleted")
    private boolean deleted;
    /**
     * Der Sortierungsschlüssel.
     */
    @Column(name = "sort", nullable = false, length = 32)
    @Index(name = "i_teil_sort")
    private String sort;
    /**
     * Der Name des Teils.
     */
    @Column(name = "name", nullable = false, length = 256)
    @Index(name = "i_teil_name")
    private String name;
    /**
     * Eine Beschreibung des Teils.
     */
    @Column(name = "description", nullable = true, length = 2048)
    @Index(name = "i_teil_description")
    private String description;
    /**
     * Ist der Teil sichtbar?
     */
    @Column(name = "visible", nullable = false)
    @Index(name = "i_teil_visible")
    private boolean visible;
    /**
     * Die Seiten des Teils.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "teilid")
    private Collection<Seite> seiteCollection;
    /**
     * Der Account der die letzte Änderung durchgeführt hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_teil_accountid")
    @Index(name = "i_teil_accountid")
    private Account accountid;
    /**
     * Der Band, zu dem der Teil gehört.
     */
    @JoinColumn(name = "bandid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_teil_bandid")
    @Index(name = "i_teil_bandid")
    private Band bandid;

    /** Creates a new instance of Teil */
    public Teil() {
    }

    /**
     * Creates a new instance of Teil with the specified values.
     * @param id the id of the Teil
     */
    public Teil(Long id) {
        this.id = id;
    }

    /*
     * Setter/Getter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getSort() {
        return this.sort;
    }

    public void setSort(String sort) {
        this.sort = sort;
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public boolean getVisible() {
        return this.visible;
    }

    public void setVisible(boolean visible) {
        this.visible = visible;
    }

    public Collection<Seite> getSeiteCollection() {
        return this.seiteCollection;
    }

    public void setSeiteCollection(Collection<Seite> seiteCollection) {
        this.seiteCollection = seiteCollection;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    public Band getBandid() {
        return this.bandid;
    }

    public void setBandid(Band bandid) {
        this.bandid = bandid;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Teil.  The result is
     * <code>true</code> if and only if the argument is not null and is a Teil object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Teil)) {
            return false;
        }
        Teil other = (Teil) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     * 
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Teil [id = ");
        ret.append(id);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; sort = ");
        ret.append(sort);
        ret.append("; name = ");
        ret.append(name);
        ret.append("; description = ");
        ret.append(description);
        ret.append("; visible = ");
        ret.append(visible);
        ret.append("]");

        return ret.toString();
    }
}
