/*
 * Werk.java
 *
 * Created on 13. August 2007, 09:19<br />
 * 24.4.2009: Versionierung und Caching hizugefügt.
 *
 */
package de.retrobib.db.model;

import java.io.Serializable;
import java.util.Collection;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Version;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.CacheModeType;
import org.hibernate.annotations.ForeignKey;
import org.hibernate.annotations.Index;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;
import org.hibernate.annotations.OptimisticLockType;

/**
 * Entity class Werk
 *
 * @author caschoff
 * @version 1.0
 *
 */
@Entity
@org.hibernate.annotations.Entity(optimisticLock = OptimisticLockType.VERSION)
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@Table(name = "werk")
@NamedQueries({
    @NamedQuery(name = "Werk.count", query = "select count(id) from Werk where deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Werk.count by oldid", query = "select count(id) from Werk where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Werk by oldid", query = "from Werk where oldid=:oldid", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Werk by shortname", query = "from Werk where shortname=:shortname and deleted=false", cacheable = true, cacheMode = CacheModeType.NORMAL),
    @NamedQuery(name = "Werk", query = "from Werk where deleted=false order by sort", cacheable = true, cacheMode = CacheModeType.NORMAL)
})
public class Werk implements Serializable {

    static final long serialVersionUID = 1l;
    /**
     * Die ID.
     */
    @Id
    @GeneratedValue
    @Column(name = "id", nullable = false)
    @Index(name = "i_werk_id")
    private Long id;
    /**
     * Die ID vom alten System.
     */
    @Column(name = "oldid", nullable = true)
    @Index(name = "i_werk_oldid")
    private Long oldid;
    /**
     * Die Version für Hibernate.
     */
    @Version
    private int version;
    /**
     * Das Datum der letzten Änderung.
     */
    @Column(name = "timestamp", nullable = false)
    @Index(name = "i_werk_timestamp")
    private Long timestamp;
    /**
     * Gelöscht?
     */
    @Column(name = "deleted", nullable = false)
    @Index(name = "i_werk_deleted")
    private boolean deleted;
    /**
     * Der Sortierungsschlüssel.
     */
    @Column(name = "sort", nullable = false, length = 32)
    @Index(name = "i_werk_sort")
    private String sort;
    /**
     * Der Name des Werkes.
     */
    @Column(name = "name", nullable = false, length = 2048)
    @Index(name = "i_werk_name")
    private String name;
    /**
     * Der Kurzname für Listen etc.
     */
    @Column(name = "shortname", nullable = false, length = 256)
    @Index(name = "i_werk_shortname")
    private String shortname;
    /**
     * Der Herausgeber (Verlag).
     */
    @Column(name = "publisher", nullable = false, length = 2048)
    @Index(name = "i_werk_publisher")
    private String publisher;
    /**
     * Der Autor.
     */
    @Column(name = "author", nullable = false, length = 2048)
    @Index(name = "i_werk_autor")
    private String author;
    /**
     * Erscheinungsdatum.
     */
    @Column(name = "issued", nullable = false, length = 256)
    @Index(name = "i_werk_issued")
    private String issued;
    /**
     * Eine Beschreibung.
     */
    @Column(name = "description", nullable = true, length = 2048)
    @Index(name = "i_werk_description")
    private String description;
    /**
     * Die Bände des Werkes.
     */
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "werkid")
    private Collection<Band> bandCollection;
    /**
     * Der Account, der die letzte Änderung vorgenommen hat.
     */
    @JoinColumn(name = "accountid", referencedColumnName = "id")
    @ManyToOne
    @ForeignKey(name = "fk_werk_accountid")
    @Index(name = "i_werk_accountid")
    private Account accountid;

    /** Creates a new instance of Werk */
    public Werk() {
    }

    /**
     * Creates a new instance of Werk with the specified values.
     *
     * @param id the id of the Werk
     */
    public Werk(Long id) {
        this.id = id;
    }

    /*
     * Setter/Getter
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOldid() {
        return this.oldid;
    }

    public void setOldid(Long oldid) {
        this.oldid = oldid;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public Long getTimestamp() {
        return this.timestamp;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public boolean getDeleted() {
        return this.deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public String getSort() {
        return this.sort;
    }

    public void setSort(String sort) {
        this.sort = sort;
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getShortname() {
        return this.shortname;
    }

    public void setShortname(String shortname) {
        this.shortname = shortname;
    }

    public String getPublisher() {
        return this.publisher;
    }

    public void setPublisher(String publisher) {
        this.publisher = publisher;
    }

    public String getAuthor() {
        return this.author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public String getIssued() {
        return this.issued;
    }

    public void setIssued(String issued) {
        this.issued = issued;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Collection<Band> getBandCollection() {
        return this.bandCollection;
    }

    public void setBandCollection(Collection<Band> bandCollection) {
        this.bandCollection = bandCollection;
    }

    public Account getAccountid() {
        return this.accountid;
    }

    public void setAccountid(Account accountid) {
        this.accountid = accountid;
    }

    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.getId() != null ? this.getId().hashCode() : 0);
        return hash;
    }

    /**
     * Determines whether another object is equal to this Werk.  The result is
     * <code>true</code> if and only if the argument is not null and is a Werk object that
     * has the same id field values as this object.
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Werk)) {
            return false;
        }
        Werk other = (Werk) object;
        if (this.getId() != other.getId() && (this.getId() == null || !this.getId().equals(other.getId()))) {
            return false;
        }
        return true;
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();

        ret.append("de.retrobib.dbmodel.Werk [id = ");
        ret.append(id);
        ret.append("; oldid = ");
        ret.append(oldid);
        ret.append("; version = ");
        ret.append(version);
        ret.append("; timestamp = ");
        ret.append(timestamp);
        ret.append("; deleted = ");
        ret.append(deleted);
        ret.append("; sort = ");
        ret.append(sort);
        ret.append("; name = ");
        ret.append(name);
        ret.append("; shortname = ");
        ret.append(shortname);
        ret.append("; publisher = ");
        ret.append(publisher);
        ret.append("; author = ");
        ret.append(author);
        ret.append("; issued = ");
        ret.append(issued);
        ret.append("; description = ");
        ret.append(description);
        ret.append("]");

        return ret.toString();
    }
}
