/*
 * HeaderControlFilter.java
 *
 * Created on 14. May 2009, 08:06
 */
package de.retrobib.filter;

import java.io.IOException;
import java.util.GregorianCalendar;
import java.util.Locale;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;
import org.ca.utils.misc.DateUtil;

/**
 * Dieser Filter setzt HTTP-Headervariablen.
 *
 * @author caschoff
 * @version 1.0
 */
public class HeaderControlFilter implements Filter {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(HeaderControlFilter.class);
    /**
     * The filter configuration object we are associated with. If
     * this value is null, this filter instance is not currently
     * configured.
     */
    private FilterConfig filterConfig = null;

    /**
     * Konstruktor.
     */
    public HeaderControlFilter() {
    }

    /**
     * Do the filtering.
     *
     * @param response
     * @param request The servlet request we are processing
     * @param chain The filter chain we are processing
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        chain.doFilter(request, response);
        if ((response instanceof HttpServletResponse) && (response.getContentType() != null) && (!response.getContentType().startsWith("text/html"))) {
            GregorianCalendar gc = new GregorianCalendar();
            gc.add(GregorianCalendar.YEAR, 1);
            ((HttpServletResponse) response).setHeader("Expires", DateUtil.getFormattedDateString(gc, Locale.US, DateUtil.RFC822_DATUM_ZEIT));
        } 
    }

    /**
     * Return the filter configuration object for this filter.
     * @return the filter config
     */
    public FilterConfig getFilterConfig() {
        return (this.filterConfig);
    }

    /**
     * Set the filter configuration object for this filter.
     * @param filterConfig The filter configuration object
     */
    public void setFilterConfig(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }

    /**
     * Destroy method for this filter.
     */
    public void destroy() {
    }

    /**
     * Init method for this filter.
     * @param filterConfig
     */
    public void init(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }
}
