/**
 * SecurityFilter.java
 *
 * Created on 7. Februar 2007, 15:51
 *
 */
package de.retrobib.filter;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.db.model.Account;
import de.retrobib.exceptions.SecurityViolationException;
import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

/**
 * Dieser Filter prüft alle Anfragen auf /secure/*, ob ein Benutzer angemeldet ist.<br />
 * Falls kein Benutzer in der Session ist, wird ein HTTP-Fehler 403 (not authorized) gesendet.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class SecurityFilter implements Filter {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SecurityFilter.class);
    /**
     * The filter configuration object we are associated with. If
     * this value is null, this filter instance is not currently
     * configured.
     */
    private FilterConfig filterConfig = null;

    /**
     * Konstruktor.
     */
    public SecurityFilter() {
    }

    /**
     * Do the filtering.
     *
     * @param response
     * @param request The servlet request we are processing
     * @param chain The filter chain we are processing
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        try {
            HttpSession session = ((HttpServletRequest) request).getSession(false);
            Account account = (Account) session.getAttribute(MiscConstants.account);
            if (account != null) {
                logger.debug("SecurityFilter sagt o.k., weiter [" + account + "; url=" + ((HttpServletRequest) request).getRequestURL() + "]");
                chain.doFilter(request, response);
            } else {
                throw new SecurityViolationException("Kein Account in der Session!");
            }
        } catch (Throwable ex) {
            logger.info("SecurityFilter sagt Nein! (" + ex.getMessage() + ", url=" + ((HttpServletRequest) request).getRequestURL() + "])");
            logger.info("Grund: ", ex);
            ((HttpServletResponse) response).sendError(403);
        }
    }

    /**
     * Return the filter configuration object for this filter.
     * @return the filter config
     */
    public FilterConfig getFilterConfig() {
        return (this.filterConfig);
    }

    /**
     * Set the filter configuration object for this filter.
     * @param filterConfig The filter configuration object
     */
    public void setFilterConfig(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }

    /**
     * Destroy method for this filter.
     */
    public void destroy() {
    }

    /**
     * Init method for this filter.
     * @param filterConfig
     */
    public void init(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }
}
