/**
 * URLFilter.java
 *
 * Created on 7. Februar 2007, 15:51
 *
 */
package de.retrobib.filter;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;

/**
 * Dieser Filter prüft alle Anfragen darauf, ob 'illegale Dinge' in der URL enthalten sind.<br />
 * z. B. .jsp, uni-ulm etc.
 * Falls dies der Fall ist, wird ein HTTP-Fehler 404 (not found) gesendet.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class URLFilter implements Filter {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(URLFilter.class);
    /**
     * The filter configuration object we are associated with. If
     * this value is null, this filter instance is not currently
     * configured.
     */
    private FilterConfig filterConfig = null;

    /**
     * Konstruktor.
     */
    public URLFilter() {
    }

    /**
     * Do the filtering.
     *
     * @param response
     * @param request The servlet request we are processing
     * @param chain The filter chain we are processing
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        HttpServletRequest req = (HttpServletRequest) request;

        try {
            if (req.getRequestURL().toString().toLowerCase().indexOf(".jsp") != -1) {
                logger.warn("Direkter Aufruf von *.jsp, sende 404 [ip=" + req.getRemoteAddr() + "; url=" + req.getRequestURL() + ", query=" + req.getQueryString() + "]");
                ((HttpServletResponse) response).sendError(404);
            } else if (req.getRequestURL().toString().toLowerCase().indexOf("uni-ulm") != -1) {
                logger.info("uni-ulm in der URL, sende 404 [ip=" + req.getRemoteAddr() + "; url=" + req.getRequestURL() + ", query=" + req.getQueryString() + "]");
                ((HttpServletResponse) response).sendError(404);
            } else {
                chain.doFilter(request, response);
            }
        } catch (Throwable ex) {
            logger.error("[ip=" + req.getRemoteAddr() + "; url=" + req.getRequestURL() + ", query=" + req.getQueryString() + "]");
            logger.fatal("Exception im URLFilter!", ex);
        }
    }

    /**
     * Return the filter configuration object for this filter.
     * @return the filter config
     */
    public FilterConfig getFilterConfig() {
        return (this.filterConfig);
    }

    /**
     * Set the filter configuration object for this filter.
     * @param filterConfig The filter configuration object
     */
    public void setFilterConfig(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }

    /**
     * Destroy method for this filter.
     */
    public void destroy() {
    }

    /**
     * Init method for this filter.
     * @param filterConfig
     */
    public void init(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }
}
