/*
 * UTF8Filter.java
 *
 * Created on 7. Februar 2007, 15:51
 */
package de.retrobib.filter;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import org.apache.log4j.Logger;

/**
 * Dieser Filter setzt das richtige Encoding für alle Requests.
 *
 * @author caschoff
 * @version 1.0
 */
public class UTF8Filter implements Filter {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(UTF8Filter.class);
    /**
     * The filter configuration object we are associated with. If
     * this value is null, this filter instance is not currently
     * configured.
     */
    private FilterConfig filterConfig = null;

    /**
     * Konstruktor.
     */
    public UTF8Filter() {
    }

    /**
     * Do the filtering.
     *
     * @param response
     * @param request The servlet request we are processing
     * @param chain The filter chain we are processing
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        if (request != null) {
            request.setCharacterEncoding("UTF-8");
        }
        chain.doFilter(request, response);
    }

    /**
     * Return the filter configuration object for this filter.
     * @return the filter config
     */
    public FilterConfig getFilterConfig() {
        return (this.filterConfig);
    }

    /**
     * Set the filter configuration object for this filter.
     * @param filterConfig The filter configuration object
     */
    public void setFilterConfig(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }

    /**
     * Destroy method for this filter.
     */
    public void destroy() {
    }

    /**
     * Init method for this filter.
     * @param filterConfig
     */
    public void init(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }
}
