/*
 * SessionManager.java
 * Erstellt am 19.1.2008
 */
package de.retrobib.listeners;

import de.retrobib.Constants.MiscConstants;
import java.util.ArrayList;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;
import org.apache.log4j.Logger;

/**
 * Tracking der Sessions.
 * 
 * @author caschoff
 * @version 1.0
 */
public class SessionManager implements HttpSessionListener {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(SessionManager.class);
    /** 
     * Die Sessions. 
     */
    private static final ArrayList<HttpSession> sessions = new ArrayList<HttpSession>();

    /**
     * Wird bei der Erzeugung einer Session aufgerufen.
     * 
     * @param event Das Ereignis.
     */
    public void sessionCreated(HttpSessionEvent event) {
        synchronized (sessions) {
            sessions.add(event.getSession());
        }
        logger.debug("Eine Session [" + event.getSession().getId() + "] wurde erstellt, count=" + sessions.size());
    }

    /**
     * Wird bei dem Verwerfen einer Session aufgerufen.
     * 
     * @param event Das Ereignis.
     */
    public void sessionDestroyed(HttpSessionEvent event) {
        synchronized (sessions) {
            sessions.remove(event.getSession());
        }
        logger.debug("Eine Session [" + event.getSession().getId() + "] wurde verworfen, count=" + sessions.size());
    }

    /**
     * Die Anzahl der Sessions bekommen.
     * 
     * @return Die Anzahl der Sessions.
     */
    public static int getSessioncount() {
        return sessions.size();
    }

    /**
     * Die Anzahl der angemeldeten Benutzer bekommen.
     *
     * @return Die Anzahl.
     */
    public static int getRegistereduserscount() {
        int ret = 0;

        for (HttpSession session : sessions) {
            if (session.getAttribute(MiscConstants.account) != null) {
                ret++;
            }
        }

        return ret;
    }
}
