/**
 * HibernatePlugin.java
 *
 * Created on 7. Maerz 2007, 10:32
 *
 */
package de.retrobib.struts.plugins;

import de.retrobib.Constants.MiscConstants;
import javax.servlet.ServletException;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.action.PlugIn;
import org.apache.struts.config.ModuleConfig;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;

/**
 * Das Hibernate-Plugin stellt allen Actions die Factorys für den Datenzugriff zur Verfügung.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class HibernatePlugin implements PlugIn {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(HibernatePlugin.class);
    /**
     * Die Verbindung zur Retrobib-Datenbank.
     */
    private SessionFactory sessionFactory;

    /**
     * Creates a new instance of HibernatePlugin.
     */
    public HibernatePlugin() {
    }

    /**
     * Initialisierung.
     *
     * @param actionServlet Das ActionServlet.
     * @param moduleConfig Die Modulkonfiguration.
     * @throws ServletException Wird bei einem Initialisierungsfeheler geworfen.
     */
    public void init(ActionServlet actionServlet, ModuleConfig moduleConfig)
            throws ServletException {
        try {
            logger.debug("Erzeuge SessionFactory aus hibernate.cfg.xml");
            sessionFactory = new AnnotationConfiguration().configure("hibernate.cfg.xml").buildSessionFactory();
            actionServlet.getServletContext().setAttribute(MiscConstants.retrobibFactory, sessionFactory);
        } catch (Throwable ex) {
            logger.error("Exception in init", ex);
            throw new ServletException(ex);
        }
    }

    /**
     * Wird vom Container bei einem Undeploy/Herunterfahren aufgerufen.
     */
    public void destroy() {
        try {
            logger.debug("Schliesse SessionFactory.");
            sessionFactory.close();
        } catch (Throwable ex) {
            logger.error("Exception in destroy", ex);
        }
    }
}
