/**
 * StartStopPlugin.java
 *
 * Created on 7. Maerz 2007, 10:32
 *
 */
package de.retrobib.struts.plugins;

import de.retrobib.cron.CronManager;
import javax.servlet.ServletException;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.action.PlugIn;
import org.apache.struts.config.ModuleConfig;

/**
 * Das StartStop-Plugin dient zur Ausführung von Startup/Shutdown-Tasks.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class StartStopPlugin implements PlugIn {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(StartStopPlugin.class);

    /**
     * Creates a new instance of StartStopPlugin.
     */
    public StartStopPlugin() {
    }

    /**
     * Initialisierung.
     *
     * @param actionServlet Das ActionServlet.
     * @param moduleConfig Die Modulkonfiguration.
     * @throws ServletException Wird bei einem Initialisierungsfeheler geworfen.
     */
    public void init(ActionServlet actionServlet, ModuleConfig moduleConfig)
            throws ServletException {
        try {
            logger.debug("Ausführen von Startup-Aufgaben...");

            logger.debug("Starte Quarz-Scheduler...");
            // Cron-Manager am laufen?
            if (CronManager.cronManager == null) {
                logger.warn("Cron-Manager läuft (noch) nicht.");
            }
            logger.debug("... gestartet.");
        } catch (Throwable ex) {
            logger.error("Exception in init", ex);
            throw new ServletException(ex);
        }
    }

    /**
     * Wird vom Container bei einem Undeploy/Herunterfahren aufgerufen.
     */
    public void destroy() {
        try {
            logger.debug("Ausführen von Shutdown-Aufgaben...");

            logger.debug("Stoppe Quarz-Scheduler...");
            CronManager.cronManager.getScheduler().shutdown(true);
            logger.debug("... gestoppt.");
        } catch (Throwable ex) {
            logger.error("Exception in destroy", ex);
        }
    }
}
