/*
 * ImageUtil.java
 *
 * Created on 3. Dezember 2007, 08:43
 *
 */
package de.retrobib.utils;

import de.retrobib.Constants.MiscConstants;
import de.retrobib.db.model.Faksimile;
import de.retrobib.db.model.Hiresfaksimile;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Iterator;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.ImageOutputStream;
import org.apache.log4j.Logger;

/**
 * Klasse für die Bearbeitung von Bildern (croppen etc.).
 *
 * 8.12.2007: 1.1 - ios.close() hinzugefügt.
 *
 * @author caschoff
 * @version 1.1
 */
public class ImageUtil {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(ImageUtil.class);

    /** Creates a new instance of ImageUtil */
    public ImageUtil() {
    }

    /**
     * Ein Bild zuschneiden.
     *
     * @param faksimile Das Quellbild.
     * @param area Der Bereich, auf den Zugeschnitten werden soll.
     * @param destination Der Ziel-OutputStream.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void cropImage(Faksimile faksimile, Rectangle area, OutputStream destination)
            throws Exception {
        cropImage(new URL(MiscConstants.baseURL + "/retrobib/faksimile_" + faksimile.getId() + ".jpeg"), area, destination);
    }

    /**
     * Ein Bild zuschneiden.
     *
     * @param hiresfaksimile Das Quellbild.
     * @param area Der Bereich, auf den Zugeschnitten werden soll.
     * @param destination Der Ziel-OutputStream.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void cropImage(Hiresfaksimile hiresfaksimile, Rectangle area, OutputStream destination)
            throws Exception {
        cropImage(new URL(MiscConstants.baseURL + "/retrobib/hiresfaksimile_" + hiresfaksimile.getId() + ".jpeg"), area, destination);
    }

    /**
     * Ein Bild zuschneiden.
     *
     * @param source Die URL des Quellbildes.
     * @param area Der Bereich, auf den Zugeschnitten werden soll.
     * @param destination Der Ziel-OutputStream.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public void cropImage(URL source, Rectangle area, OutputStream destination)
            throws Exception {
        // JPEG-Reader bekommen...
        logger.debug("JPEG-Reader holen...");
        Iterator<ImageReader> readers = ImageIO.getImageReadersByFormatName("jpeg");
        ImageReader reader = readers.next();

        // Bild lesen...
        logger.debug("Bild lesen...");
        source.openConnection();
        InputStream stream = source.openStream();
        ImageInputStream iis = ImageIO.createImageInputStream(stream);
        reader.setInput(iis, false);
        BufferedImage bi = reader.read(0);

        // Bild croppen...
        logger.debug("Bild croppen...");
        BufferedImage biCrop = bi.getSubimage((int) area.getX(), (int) area.getY(), (int) area.getWidth(), (int) area.getHeight());

        // JPEG-Writer bekommen...
        logger.debug("JPEG-Writer bekommen...");
        Iterator<ImageWriter> writers = ImageIO.getImageWritersByFormatName("jpeg");
        ImageWriter writer = writers.next();

        // Bild schreiben...
        logger.debug("Bild schreiben....");
        ImageOutputStream ios = ImageIO.createImageOutputStream(destination);
        writer.setOutput(ios);
        writer.write(biCrop);
        // 8.12.2007, ca. Kann ein Grund für die IlligalStateExceptions sein.
        ios.close();
    }
}
