/**
 * Mailer.java
 *
 * Created on 29. Juli 2005, 09:28
 *
 */
package de.retrobib.utils;

import de.retrobib.Constants.MiscConstants;
import java.util.ArrayList;
import java.util.Properties;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import org.apache.log4j.Logger;

/**
 * Verschicken von eMails als Thread, da das Auflösen von 
 * Adressen und das Verschicken von Mails dauern kann.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class Mailer extends Thread {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Mailer.class);
    /** Die Betreff-Zeile */
    private String subject = null;
    /** Der Inhalt. */
    private String text = null;
    /** Der Empfänger. */
    private String rcpt = null;
    /** Weitere Empfänger. */
    private ArrayList<String> moreRcpts = null;

    /**
     * Creates a new instance of Mailer
     */
    public Mailer() {
    }

    /**
     * Starten des Threads.
     */
    @Override
    public void run() {
        try {
            Properties props = new Properties();
            props.put("mail.smtp.host", "mail.uni-ulm.de");
            Session s = Session.getInstance(props, null);

            InternetAddress from = new InternetAddress(MiscConstants.adminMail);
            InternetAddress to = new InternetAddress(rcpt);

            MimeMessage message = new MimeMessage(s);
            message.setFrom(from);
            message.addRecipient(Message.RecipientType.TO, to);
            if (moreRcpts != null) {
                for (int i = 0; i < moreRcpts.size(); i++) {
                    to = new InternetAddress(moreRcpts.get(i));
                    message.addRecipient(Message.RecipientType.TO, to);
                }
            }

            message.setSubject(subject);
            message.setText(text, "UTF-8");

            Transport.send(message);
            logger.debug("Mail an " + rcpt + " wurde gesendet.");
        } catch (Exception ex) {
            logger.error("Exception im Mailer.", ex);
        }
    }

    /**
     * Konstruktor.
     *
     * @param subject Der Betreff.
     * @param text Der Inhalt.
     * @param rcpt Der Ampfänger.
     * @param moreRcpts Weitere Empfänger als ArrayListe.
     * @throws java.lang.Exception Falls ein Fehler auftritt.
     */
    public void send(String subject, String text, String rcpt, ArrayList<String> moreRcpts)
            throws Exception {
        this.subject = subject;
        this.text = text;
        this.rcpt = rcpt;
        this.moreRcpts = moreRcpts;
        start();
    }

    /**
     * 
     *
     * @param subject Der Betreff.
     * @param messages Nachrichten, die in Text umgewandelt werden.
     * @param rcpt Der Ampfänger.
     * @param moreRcpts Weitere Empfänger als ArrayListe.
     * @throws java.lang.Exception Falls ein Fehler auftritt.
     */
    public void send(String subject, ArrayList<de.retrobib.webelements.Message> messages, String rcpt, ArrayList<String> moreRcpts)
            throws Exception {
        StringBuilder content = new StringBuilder();
        for (int i = 0; i < messages.size(); i++) {
            content.append(messages.get(i).getMessage() + " (" + messages.get(i).getTypstring() + ")\n");
        }
        send(subject, content.toString(), rcpt, moreRcpts);
    }
}
