/*
 * LuceneHit.java
 *
 * Created on 4. September 2007, 12:46
 *
 */
package de.retrobib.webelements;

import de.retrobib.Constants.LuceneFieldNames;
import de.retrobib.db.model.Seite;
import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.hibernate.Session;

/**
 * PoJo um einen Lucene-Hit zur Darstellung in JSPs.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class LuceneHit {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(LuceneHit.class);
    /** Das Lucene-Dokument. */
    private Document document = null;
    /** Der Lucene-Rank. */
    private float rank = 0.0f;
    /** Der Text-Schnipsel als Vorschau. */
    private String highlight = null;
    /** Die Seite aus der Datenbank. */
    private Seite seite;

    /**
     * Creates a new instance of LuceneHit.
     * 
     * @param document Das zu wrappende Lucene-Document.
     * @param rank Der Rank des Treffers.
     * @param highlight Der Text-Schnipsel.
     * @param hiberSession
     */
    public LuceneHit(Document document, float rank, String highlight, Session hiberSession) {
        this.document = document;
        this.rank = rank;
        this.highlight = highlight;
        seite = (Seite) hiberSession.load(Seite.class, Long.parseLong(document.get(LuceneFieldNames.seite_id)));
        seite.getStatus();
    }

    /**
     * Den Rank des Treffers bekommen.
     *
     * @return Der Rank.
     */
    public String getRank() {
        return ((int) (100.0f * rank)) + "%";
    }

    /**
     * Die Vorschau für den Treffer bekommen.
     *
     * @return Die Vorschau in HTML!
     */
    public String getHighlight() {
        return highlight;
    }

    /**
     * Den Kurznamen des Werkes bekommen.
     *
     * @return Der Kurzname.
     */
    public String getWerk() {
        return document.get(LuceneFieldNames.werk_shortname);
    }

    /**
     * Den Namen des Bandes bekommen.
     *
     * @return Der Name.
     */
    public String getBand() {
        return document.get(LuceneFieldNames.band_name);
    }

    /**
     * Den Namen des Teils bekommen.
     *
     * @return Der Name.
     */
    public String getTeil() {
        return document.get(LuceneFieldNames.teil_name);
    }

    /**
     * Die Seitennummer bekommen.
     *
     * @return Die Seitennummer.
     */
    public String getSeitennummer() {
        return document.get(LuceneFieldNames.seite_nummer);
    }

    /**
     * Das Startwort bekommen.
     *
     * @return Das Startwort.
     */
    public String getVon() {
        return document.get(LuceneFieldNames.seite_von);
    }

    /**
     * Das Endwort bekommen.
     *
     * @return Das Endwort.
     */
    public String getBis() {
        return document.get(LuceneFieldNames.seite_bis);
    }

    /**
     * Die Id der Seite bekommen.
     *
     * @return Die Id.
     */
    public String getId() {
        return document.get(LuceneFieldNames.seite_id);
    }

    /**
     * Den Status der Seite bekommen.
     *
     * @return Der Status.
     */
    public int getStatus() {
        return seite.getStatus(); // document.get(LuceneFieldNames.seite_status);
    }

    /**
     * @return
     */
     public boolean getBlockmarker() {
        return seite.getBlockmarker();
     }

    /**
     * Die eMail des letzten Bearbeiters bekommen.
     *
     * @return Die eMail.
     */
    public String getEmail() {
        return document.get(LuceneFieldNames.seite_email);
    }

    /**
     * Den Zeitpunkt der letzten Änderung bekommen.
     *
     * @return Der Zeitpunkt.
     */
    public String getTimestamp() {
        return document.get(LuceneFieldNames.seite_timestamp);
    }
}
