/*
 * LuceneHits.java
 *
 * Created on 12. Oktober 2007, 10:23
 *
 */
package de.retrobib.webelements;

import java.util.ArrayList;

/**
 * Wrapper um die Ergebnisse einer Lucene-Suche.
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class LuceneHits {

    /** Die Laufzeit der Abfrage in Millisekunden. */
    private long duration;
    /** Die Anzahl der Ergebnisse. */
    private int resultCount;
    /** Der Did-You-Mean-Vorschlag. */
    private String didYouMean = null;
    /** Die Hit-Liste. */
    private ArrayList<LuceneHit> hits = null;

    /**
     * Creates a new instance of LuceneHits.
     *
     * @param resultCount Wieviele Treffer gab es?
     * @param duration Wie lange hat die Query gedauert (in Millisekunden)?
     */
    public LuceneHits(int resultCount, long duration) {
        this.resultCount = resultCount;
        this.duration = duration;
    }

    /**
     * Einen Treffer hinzufügen.
     *
     * @param hit Der Treffer.
     */
    public void addHit(LuceneHit hit) {
        if (hits == null) {
            hits = new ArrayList<LuceneHit>();
        }
        hits.add(hit);
    }

    /**
     * Die Hit-Liste bekommen.
     * 
     * @return Die Hit-Liste.
     */
    public ArrayList<LuceneHit> getHitlist() {
        return hits;
    }

    /*
     * Getter/Setter.
     */
    /**
     * Die Laufzeit der Anfrage bekommen.
     *
     * @return Die Laufzeit der Anfrage.
     */
    public long getDuration() {
        return duration;
    }

    /**
     * Die Laufzeit der Anfrage setzen.
     *
     * @param duration Die Laufzeit der Anfrage.
     */
    public void setDuration(long duration) {
        this.duration = duration;
    }

    /**
     * Die Anzahl der Treffer der Suche bekommen.<br />
     * Die Anzahl der Treffer in der Liste kann geringer sein.
     *
     * @return Die Anzahl der Treffer der Suche.
     */
    public int getCount() {
        return resultCount;
    }

    /**
     * Die Anzahl der Treffer der Suche setzen.
     *
     * @param resultCount Die Anzahl der Treffer.
     */
    public void setCount(int resultCount) {
        this.resultCount = resultCount;
    }

    /**
     * Ein alternatives Suchwort bekommen.
     *
     * @return Das alternative Suchwort.
     */
    public String getDidyoumean() {
        return didYouMean;
    }

    /**
     * Das alternative Suchwort setzen.
     *
     * @param didYouMean Das alternative Suchwort.
     */
    public void setDidyoumean(String didYouMean) {
        this.didYouMean = didYouMean;
    }
}
