/**
 * Message.java
 *
 * Created on 27. Februar 2007, 12:19
 *
 */
package de.retrobib.webelements;

import java.util.ArrayList;
import org.apache.log4j.Logger;

/**
 * Kapselt eine Nachricht zur Anzeige in JSP's (pojo).
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class Message {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Message.class);
    /*
     * Mögliche Typen der Nachricht.
     */
    /** Nur zur Information. */
    public static final int INFO = 0;
    /** Warnung, aber kein Fehler. */
    public static final int WARN = 1;
    /** Erkannter Fehler. */
    public static final int ERROR = 2;
    /** Nicht behebbarer Fehler. */
    public static final int FATAL = 3;
    /** Die Nachricht. */
    private String message;
    /** Der Typ der Nachricht. */
    private int type;

    /**
     * Creates a new instance of Message.
     */
    public Message() {
    }

    /**
     * Creates a new instance of Message.
     *
     * @param message Der Nachrichtentext.
     * @param type Der Nachrichtentyp.
     */
    public Message(String message, int type) {
        this.message = message;
        this.type = type;
    }

    /**
     * Diese Funktion prüft eine ArrayList<Message> ob eine Message
     * vom Typ Mesage.ERROR oder Message.FATAL dabei ist.
     *
     * @param messages Die Nachrichten.
     * @return <code>true</code> wenn eine Nachricht vom Typ <code>Message.ERROR</code>
     *         oder <code>Message.FATAL</code> dabei ist, <code>false</code> andernfalls.
     */
    public static boolean hasErrors(ArrayList<Message> messages) {
        if (messages == null) {
            return false;
        }

        for (Message message : messages) {
            if ((message.getType() == Message.ERROR) || (message.getType() == Message.FATAL)) {
                return true;
            }
        }

        return false;
    }

    /*
     * Getter/Setter
     */
    /**
     * Die Nachricht bekommen.
     *
     * @return Die Nachricht.
     */
    public String getMessage() {
        return message;
    }

    /**
     * Die Nachricht setzten.
     *
     * @param message Die Nachricht.
     */
    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * Den Typ der Nachricht bekommen.
     *
     * @return Der Typ.
     */
    public int getType() {
        return type;
    }

    /**
     * Den Typ der Nachricht setzten.
     *
     * @param type Der Typ.
     */
    public void setType(int type) {
        this.type = type;
    }

    /**
     * Den Typ als <code>String</code> bekommen, z. B. zum Anzeigen in HTML-Seiten.
     *
     * @return Eine <code>String</code>-Representation des Typs.
     */
    public String getTypstring() {
        switch (type) {
            case INFO:
                return "Info";
            case WARN:
                return "Warn";
            case ERROR:
                return "Error";
            case FATAL:
                return "Fatal";
            default:
                return "Unknown";
        }
    }
}
