/**
 * TeilMitSeiten.java
 *
 * Created on 24. August 2007, 13:46
 *
 */
package de.retrobib.webelements;

import de.retrobib.db.model.Seite;
import de.retrobib.db.model.Teil;
import java.util.List;
import org.apache.log4j.Logger;
import org.hibernate.Session;

/**
 * Stellt einen Teil mit Seiten für JSPs zur Verfügung (PoJo).
 *
 * @author caschoff
 * @version 1.0
 *
 */
public class TeilMitSeiten {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(TeilMitSeiten.class);
    /** Die Seiten. */
    private List<Seite> seiten = null;
    /** Der Teil. */
    private Teil teil = null;
    /** Die Hibernate-Session. */
    private Session hiberSession = null;
    /** Das Select-Element. */
    private String selectElement = null;

    /**
     * Konstruktor.
     * 
     * @param selectedTeil Der Teil.
     * @param seiten Die Seiten.
     * @param hiberSession Die Hibernate-Session.
     */
    public TeilMitSeiten(Teil selectedTeil, List<Seite> seiten, Session hiberSession) {
        this.teil = selectedTeil;
        this.seiten = seiten;
        this.hiberSession = hiberSession;

        // Select-Element aufbauen, falls Seiten verschoben werden sollen...
        // Mehrere dieser Select-Elemente können in einer Form stehen, daher 
        // ist der Name des Elements teilid_xxx wobei xxx die Id des Teils ist.
        StringBuilder elem = new StringBuilder();
        @SuppressWarnings("unchecked")
        List<Teil> l = (List<Teil>) hiberSession.getNamedQuery("Teil by bandid").setLong("bandid", selectedTeil.getBandid().getId()).list();

        elem.append("<select name=\"teilid_" + selectedTeil.getId() + "\">");
        for (Teil teil : l) {
            elem.append("<option value=\"" + teil.getId() + "\"");
            if (teil.getId().equals(selectedTeil.getId())) {
                elem.append(" selected=\"selected\"");
            }
            elem.append(">" + teil.getName() + "</option>");
        }
        elem.append("</select");

        selectElement = elem.toString();
    }

    /*
     * Getter/Setter.
     */
    /**
     * Den Teil bekommen.
     *
     * @return Der Teil.
     */
    public Teil getTeil() {
        return teil;
    }

    /**
     * Die Seiten des Teils bekommen.
     *
     * @return Die Seiten des Teils.
     */
    public List<Seite> getSeiten() {
        return seiten;
    }

    /**
     * Für das Verschieben von Seiten zwischen Teilen
     * das Select-Element bekommen. Der Name des Elements ist 
     * teilid_xxx wobei xxx die Id des Teils ist.
     *
     * @return Das Select-Element.
     */
    public String getSelect() {
        return selectElement;
    }
}
