/*
 * Extractor.java
 *
 * Created on 9. November 2007, 11:03
 *
 */
package de.retrobib.xml;

import de.retrobib.webelements.Message;
import java.io.StringReader;
import java.util.ArrayList;
import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Klasse zum Extrahieren von Informationen aus einem XML-Dokument.
 *
 * @author caschoff
 * @version 1.0
 */
public class Extractor {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Extractor.class);
    /** Das XML-Dokument. */
    private String XMLDocument = null;
    /** Der ContentHandler. */
    ExtractorContentHandler ech = new ExtractorContentHandler();
    /** Der ErrorHandler. */
    ExtractorErrorHandler eeh = new ExtractorErrorHandler();

    /**
     * Creates a new instance of Extractor.
     *
     * @param XMLDocument Das XML-Dokument.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public Extractor(String XMLDocument)
            throws Exception {
        this.XMLDocument = XMLDocument;
        parse();
    }

    /**
     * Parsen des XML-Dokuments.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    private void parse()
            throws Exception {
        // Reader erstellen...
        XMLReader reader = XMLReaderFactory.createXMLReader();

        // Content-Handler registrieren...
        reader.setContentHandler(ech);

        // Error-Handler registrieren...
        reader.setErrorHandler(eeh);

        // Parsen...
        reader.parse(new InputSource(new StringReader(XMLDocument)));
    }

    /**
     * Den Inhalt eines XML-Dokuments für Lucene bekommen.
     *
     * @return Der Inhalt für Lucene.
     */
    public String getLuceneContent() {
        return ech.getLuceneContent();
    }

    /**
     * Die Boost-Worte für Lucene als String bekommen.
     *
     * @return Die Boost-Worte.
     */
    public String getBoostWorte() {
        return ech.getBoostWorte();
    }

    /**
     * Die Schlagwörter als ArrayListe bekommen.
     *
     * @return Die Schlagwörter.
     */
    public ArrayList<String> getSchlagWorte() {
        return ech.getSchlagworte();
    }

    /**
     * Die Nachrichten bekommen, die während des Parsens aufgetreten sind.
     *
     * @return Die Nachrichten.
     */
    public ArrayList<Message> getMessages() {
        return eeh.getMessages();
    }
}
