/*
 * ExtractorErrorHandler.java
 *
 * Created on 9. November 2007, 11:16
 *
 */
package de.retrobib.xml;

import de.retrobib.webelements.Message;
import java.util.ArrayList;
import org.apache.log4j.Logger;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Handhabung von Fehlern, die bei der Validierung von
 * XML-Dokumenten auftreten können.
 *
 * @author caschoff
 * @version 1.0
 */
public class ExtractorErrorHandler implements ErrorHandler {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(ExtractorErrorHandler.class);
    /** Die Rückmeldungen. */
    private ArrayList<Message> messages = new ArrayList<Message>();

    /**
     * Creates a new instance of ExtractorErrorHandler
     */
    public ExtractorErrorHandler() {
    }

    /**
     * Die Nachrichten bekommen, die während des Parsens auftreten.
     *
     * @return Die Nachrichten. 
     */
    public ArrayList<Message> getMessages() {
        return messages;
    }

    /**
     * Eine Warnung von Parser bekommen.
     *
     * @param exception Die Warnung. 
     * @throws org.xml.sax.SAXException Bei einem Fehler. 
     */
    public void warning(SAXParseException exception)
            throws SAXException {
        logger.debug("Parser Warnung: Zeile: " + exception.getLineNumber() + " Nachricht: " + exception.getMessage());
        messages.add(new Message("Parser Warnung: Zeile: " + exception.getLineNumber() + " Nachricht: " + exception.getMessage(), Message.WARN));
    }

    /**
     * Einen Fehler vom Parser bekommen.
     *
     * @param exception Der Fehler.
     * @throws org.xml.sax.SAXException Bei einem Fehler. 
     */
    public void error(SAXParseException exception)
            throws SAXException {
        logger.debug("Parser Fehler: Zeile: " + exception.getLineNumber() + " Nachricht: " + exception.getMessage());
        messages.add(new Message("Parser Fehler: Zeile: " + exception.getLineNumber() + " Nachricht: " + exception.getMessage(), Message.ERROR));
    }

    /**
     * Einen fatalen Fehler vom Parser bekommen.
     *
     * @param exception Der fatale Fehler.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void fatalError(SAXParseException exception)
            throws SAXException {
        logger.debug("Fataler Parser Fehler: Zeile: " + exception.getLineNumber() + " Nachricht: " + exception.getMessage());
        messages.add(new Message("Fataler Parser Fehler: Zeile: " + exception.getLineNumber() + " Nachricht: " + exception.getMessage(), Message.FATAL));
    }
}
