/*
 * MeyersBifabExtractor.java
 *
 * Created on 9. November 2007, 11:03
 *
 */
package de.retrobib.xml;

import de.retrobib.webelements.Message;
import java.io.StringReader;
import java.util.ArrayList;
import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Klasse zum Extrahieren von Informationen aus einem XML-Dokument, 
 * welches von der Schnitstelle vom Bifab kommt.
 *
 * @author caschoff
 * @version 1.0
 */
public class MeyersBifabExtractor {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(MeyersBifabExtractor.class);
    /** Das XML-Dokument. */
    private String XMLDocument = null;
    /** Der ContentHandler. */
    MeyersBifabExtractorContentHandler mbech = new MeyersBifabExtractorContentHandler();
    /** Der ErrorHandler. */
    MeyersBifabExtractorErrorHandler mbeeh = new MeyersBifabExtractorErrorHandler();

    /**
     * Creates a new instance of Extractor.
     *
     * @param XMLDocument Das XML-Dokument.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public MeyersBifabExtractor(String XMLDocument)
            throws Exception {
        this.XMLDocument = XMLDocument;
        parse();
    }

    /**
     * Hat das Schlagwort Inhalte im Bifab?
     * 
     * @return <code>true</code>, wenn es Inhalte gibt, <code>false</code> andernfalls.
     */
    public boolean hasContent() {
        return mbech.hasContent();
    }

    /**
     * Parsen des XML-Dokuments.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    private void parse()
            throws Exception {
        // Reader erstellen...
        XMLReader reader = XMLReaderFactory.createXMLReader();

        // Content-Handler registrieren...
        reader.setContentHandler(mbech);

        // Error-Handler registrieren...
        reader.setErrorHandler(mbeeh);

        // Parsen...
        reader.parse(new InputSource(new StringReader(XMLDocument)));
    }

    /**
     * Die Nachrichten bekommen, die während des Parsens aufgetreten sind.
     *
     * @return Die Nachrichten.
     */
    public ArrayList<Message> getMessages() {
        return mbeeh.getMessages();
    }
}
