/*
 * MeyersBifabExtractorContentHandler.java
 *
 * Created on 12. November 2007, 11:31
 *
 */
package de.retrobib.xml;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Der MeyersBifabExtractorContentHandler bekommt die Rückmeldungen vom Parser
 * über das Auftreten von Elementen.
 *
 * @author caschoff
 * @version 1.0
 */
public class MeyersBifabExtractorContentHandler implements ContentHandler {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(MeyersBifabExtractorContentHandler.class);
    /** Gibt es das Schlagwort beim Bifab? */
    private boolean exists = false;

    /**
     * Creates a new instance of MeyersBifabExtractorContentHandler
     */
    public MeyersBifabExtractorContentHandler() {
    }

    /**
     * Hat das Schlagwort Content im Bifab?
     *
     * @return <code>true</code>, wenn es Inhalte gibt, <code>false</code> andernfalls.
     */
    public boolean hasContent() {
        return exists;
    }

    /**
     * Callback um den Document-Locator zu setzten.
     *
     * @param locator Der Locator.
     */
    public void setDocumentLocator(Locator locator) {
    }

    /**
     * Callback beim Start eines Dokuments.
     *
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startDocument()
            throws SAXException {
    }

    /**
     * Callback beim Ende des Dokuments.
     *
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endDocument()
            throws SAXException {
    }

    /**
     * Callback beim Start eines Prefix-Mappings.
     *
     * @param prefix Der Prefix.
     * @param uri Die URI.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startPrefixMapping(String prefix, String uri)
            throws SAXException {
    }

    /**
     * Callback beim Ende eines Prefix-Mappings.
     *
     * @param prefix Der Prefix.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endPrefixMapping(String prefix)
            throws SAXException {
    }

    /**
     * Callback beim Start eines Elements.
     *
     * @param namespaceURI Die Namespace-URI.
     * @param localName Der lokale Name.
     * @param qName Der qualifizierte Name.
     * @param atts Die Attribute.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
            throws SAXException {
        if (localName.equals("revisions")) {
            exists = true;
        }
    }

    /**
     * Callback beim Ende eines Elements.
     *
     * @param namespaceURI Die Namespace-URI.
     * @param localName Der lokale Name.
     * @param qName Der qualifizierte Name.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endElement(String namespaceURI, String localName, String qName)
            throws SAXException {
    }

    /**
     * Callback wenn Character-Daten auftreten.
     *
     * @param ch Die Characters als Array.
     * @param start Der Startpunkt der Zeichenfolge <i>innerhalb</i> des Arrays.
     * @param length Die Länge der Zeichenfolge <i>ab dem Startpunkt</i>.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void characters(char[] ch, int start, int length)
            throws SAXException {
        String content = new String(ch, start, length).trim();
    }

    /**
     * Callback wenn ignorierbare Whitespaces auftreten.
     *
     * @param ch Die Characters als Array.
     * @param start Der Startpunkt der Zeichenfolge <i>innerhalb</i> des Arrays.
     * @param length Die Länge der Zeichenfolge <i>ab dem Startpunkt</i>.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void ignorableWhitespace(char[] ch, int start, int length)
            throws SAXException {
    }

    /**
     * Callback bei einer Processing-Instruction.
     *
     * @param target Das Ziel.
     * @param data Die Daten.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void processingInstruction(String target, String data)
            throws SAXException {
    }

    /**
     * Callback wenn eine Entität übersprungen wird.
     *
     * @param name Der Name der Entität.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void skippedEntity(String name)
            throws SAXException {
    }
}
