/*
 * Validator.java
 *
 * Created on 9. November 2007, 11:03
 *
 */
package de.retrobib.xml;

import de.retrobib.webelements.Message;
import java.io.StringReader;
import java.util.ArrayList;
import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Klasse für die Validierung von XML-Dokumenten.
 *
 * @author caschoff
 * @version 1.0
 */
public class Validator {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(Validator.class);

    /** Creates a new instance of Validator */
    public Validator() {
    }

    /**
     * Validieren eines XML-Dokumentes.
     *
     * @param xml Das XML-Dokuent. 
     * @return Eine Liste mit Meldungen über die Validierung. 
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public ArrayList<Message> validate(String xml)
            throws Exception {
        ValidationErrorHandler veh = new ValidationErrorHandler();

        try {
            XMLReader reader = XMLReaderFactory.createXMLReader();
            reader.setFeature("http://xml.org/sax/features/validation", true);
            reader.setErrorHandler(veh);
            reader.parse(new InputSource(new StringReader(xml)));
        } catch (SAXParseException ex) {
            logger.debug(ex);
        }

        return veh.getMessages();
    }
}
