/*
 * XhtmlExtractor.java
 *
 * Created on 9. November 2007, 11:03
 *
 */
package de.retrobib.xml;

import de.retrobib.webelements.Message;
import java.io.StringReader;
import java.util.ArrayList;
import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Klasse zum Extrahieren von Xhtml aus einer Seite im XML-Format.
 *
 * @author caschoff
 * @version 1.0
 */
public class XhtmlExtractor {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(XhtmlExtractor.class);
    /** Das XML-Dokument. */
    private String XMLDocument = null;
    /** Der ContentHandler. */
    XhtmlExtractorContentHandler xhtmlech = null;
    /** Der ErrorHandler. */
    XhtmlExtractorErrorHandler xhtmleeh = null;

    /**
     * Creates a new instance of XhtmlExtractor.
     * 
     * @param faksimileid Die ID des Faksimiles.
     * @param XMLDocument Das XML-Dokument.
     * @throws java.lang.Exception Bei einem Fehler.
     */
    public XhtmlExtractor(String XMLDocument, long faksimileid)
            throws Exception {
        this.XMLDocument = XMLDocument;
        xhtmlech = new XhtmlExtractorContentHandler(faksimileid);
        xhtmleeh = new XhtmlExtractorErrorHandler();
        parse();
    }

    /**
     * Die Xhtml-Seite bekommen.
     *
     * @return Die Xhtml-Seite. 
     */
    public StringBuffer getXhtmlSeite() {
        return xhtmlech.getXhtmlSeite();
    }

    /**
     * Parsen des XML-Dokuments.
     *
     * @throws java.lang.Exception Bei einem Fehler.
     */
    private void parse()
            throws Exception {
        // Reader erstellen...
        XMLReader reader = XMLReaderFactory.createXMLReader();

        // Content-Handler registrieren...
        reader.setContentHandler(xhtmlech);

        // Error-Handler registrieren...
        reader.setErrorHandler(xhtmleeh);

        // Parsen...
        reader.parse(new InputSource(new StringReader(XMLDocument)));
    }

    /**
     * Die Nachrichten bekommen, die während des Parsens aufgetreten sind.
     *
     * @return Die Nachrichten.
     */
    public ArrayList<Message> getMessages() {
        return xhtmleeh.getMessages();
    }
}
