/*
 * XhtmlExtractorContentHandler.java
 *
 * Created on 12. November 2007, 11:31
 *
 */
package de.retrobib.xml;

import java.util.HashMap;
import org.apache.log4j.Logger;
import org.ca.utils.misc.TextUtil;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Der XhtmlExtractorContentHandler bekommt die Rückmeldungen vom Parser
 * über das Auftreten von Elementen.
 *
 * @author caschoff
 * @version 1.0
 */
public class XhtmlExtractorContentHandler implements ContentHandler {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(XhtmlExtractorContentHandler.class);
    /**
     * Die XHTML-Seite.
     */
    private StringBuffer xhtmlSeite = new StringBuffer();
    /** Die Faksimile-ID dieser Seite. */
    private long faksimileid;
    /*
     * Lokale Vars...
     */
    private boolean artikelStart = false;
    private boolean artikelfortsetzungStart = false;
    private boolean artikelPWritten = false;
    private boolean artikelfortsetzungPWritten = false;
    private StringBuffer a_fortgesetztauf = null;
    private StringBuffer af_fortgesetztauf = null;
    private StringBuffer at_fortgesetztauf = null;
    private StringBuffer atf_fortgesetztauf = null;
    private StringBuffer az_fortgesetztauf = null;
    private StringBuffer azf_fortgesetztauf = null;
    private StringBuffer imageTitleContent = null;
    private StringBuffer imageTitleCdata = null;
    private StringBuffer imageTag = null;
    private String x = null;
    private String y = null;
    private String w = null;
    private String h = null;
    private StringBuffer schlagwortAttr = null;
    private StringBuffer schlagwortContent = null;
    private StringBuffer schlagwortCdata = null;
    private StringBuffer referenzAttr = null;
    private StringBuffer referenzCdata = null;
    private StringBuffer referenzContent = null;
    private String werk = null;
    private String bandnr = null;
    private String seitenr = null;
    private String aznr = null;
    private String lastPClass = "fortsetzung";
    private int ulOpen = 0;
    private HashMap<String, String> usedAnchors = new HashMap<String, String>();

    /**
     * Creates a new instance of MeyersBifabExtractorContentHandler.
     *
     * @param faksimileid Die ID des Faksimiles.
     */
    public XhtmlExtractorContentHandler(long faksimileid) {
        this.faksimileid = faksimileid;
    }

    /**
     * Die Xhtml-Seite bekommen.
     *
     * @return Die Xhtml-Seite.
     */
    public StringBuffer getXhtmlSeite() {
        return xhtmlSeite;
    }

    /**
     * Callback um den Document-Locator zu setzten.
     *
     * @param locator Der Locator.
     */
    public void setDocumentLocator(Locator locator) {
    }

    /**
     * Callback beim Start eines Dokuments.
     *
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startDocument()
            throws SAXException {
    }

    /**
     * Callback beim Ende des Dokuments.
     *
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endDocument()
            throws SAXException {
    }

    /**
     * Callback beim Start eines Prefix-Mappings.
     *
     * @param prefix Der Prefix.
     * @param uri Die URI.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startPrefixMapping(String prefix, String uri)
            throws SAXException {
    }

    /**
     * Callback beim Ende eines Prefix-Mappings.
     *
     * @param prefix Der Prefix.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endPrefixMapping(String prefix)
            throws SAXException {
    }

    /**
     * Callback beim Start eines Elements.
     *
     * @param namespaceURI Die Namespace-URI.
     * @param localName Der lokale Name.
     * @param qName Der qualifizierte Name.
     * @param atts Die Attribute.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
            throws SAXException {
        switch (XmlNames.element(localName)) {
            //
            // Werk, Band, Teil und Seite sind hier nicht von Interesse.
            //
            case XmlNames.en_werk:
                break;
            case XmlNames.en_band:
                break;
            case XmlNames.en_teil:
                break;
            case XmlNames.en_seite:
                break;
            //
            // Der Start von Artikel(-fortsetzunges/-teilen) kann teilweise erst bei der
            // characters()-Funktion ein <p> schreiben...
            // 'Fortgesetztvon' kann man gleich schreiben, 'Fortgesetztauf' wird aufgehoben
            // für das End-Tag.
            //
            case XmlNames.en_artikel:
                // falls Bilder drin sind...
                xhtmlSeite.append("<div class=\"floatbox\">");

                lastPClass = "artikel";
                artikelStart = true;
                if (atts.getValue("", "fortgesetztauf") != null) {
                    a_fortgesetztauf = new StringBuffer();
                    a_fortgesetztauf.append("<p class=\"anmerkung\">Anmerkung: Fortgesetzt auf Seite " + atts.getValue("", "fortgesetztauf") + ".</p>");
                }
                break;
            case XmlNames.en_artikelfortsetzung:
                // falls Bilder drin sind...
                xhtmlSeite.append("<div class=\"floatbox\">");

                lastPClass = "fortsetzung";
                artikelfortsetzungStart = true;
                if (atts.getValue("", "fortgesetztvon") != null) {
                    xhtmlSeite.append("<p class=\"anmerkung\">Anmerkung: Fortsetzung des Artikels '" + atts.getValue("", "fortgesetztvon") + "'</p>");
                }
                if (atts.getValue("", "fortgesetztauf") != null) {
                    af_fortgesetztauf = new StringBuffer();
                    af_fortgesetztauf.append("<p class=\"anmerkung\">Anmerkung: Fortgesetzt auf Seite " + atts.getValue("", "fortgesetztauf") + ".</p>");
                }
                break;
            case XmlNames.en_artikelteil:
                lastPClass = "artikel";
                if (atts.getValue("", "fortgesetztauf") != null) {
                    at_fortgesetztauf = new StringBuffer();
                    at_fortgesetztauf.append("<p class=\"anmerkung\">Anmerkung: Fortgesetzt auf Seite " + atts.getValue("", "fortgesetztauf") + ".</p>");
                }
                xhtmlSeite.append("<p class=\"artikel\">");
                artikelStart = false;
                artikelfortsetzungStart = false;
                break;
            case XmlNames.en_artikelteilfortsetzung:
                lastPClass = "fortsetzung";
                if (atts.getValue("", "fortgesetztvon") != null) {
                    xhtmlSeite.append("<p class=\"anmerkung\">Anmerkung: Fortsetzung von [" + atts.getValue("", "fortgesetztvon") + ".]</p>");
                }
                if (atts.getValue("", "fortgesetztauf") != null) {
                    atf_fortgesetztauf = new StringBuffer();
                    atf_fortgesetztauf.append("<p class=\"anmerkung\">Anmerkung: Fortgesetzt auf Seite " + atts.getValue("", "fortgesetztauf") + ".</p>");
                }
                xhtmlSeite.append("<p class=\"fortsetzung\">");
                artikelStart = false;
                artikelfortsetzungStart = false;
                break;
            case XmlNames.en_artikelzaehlung:
                lastPClass = "artikel";
                if (atts.getValue("", "fortgesetztauf") != null) {
                    az_fortgesetztauf = new StringBuffer();
                    az_fortgesetztauf.append("<p class=\"anmerkung\">Anmerkung: Fortgesetzt auf Seite " + atts.getValue("", "fortgesetztauf") + ".</p>");
                }
                xhtmlSeite.append("<p class=\"artikel\">");
                artikelStart = false;
                artikelfortsetzungStart = false;
                break;
            case XmlNames.en_artikelzaehlungsfortsetzung:
                lastPClass = "fortsetzung";
                if (atts.getValue("", "fortgesetztvon") != null) {
                    xhtmlSeite.append("<p class=\"anmerkung\">Anmerkung: Fortsetzung von Nummer " + atts.getValue("", "fortgesetztvon") + ")</p>");
                }
                if (atts.getValue("", "fortgesetztauf") != null) {
                    azf_fortgesetztauf = new StringBuffer();
                    azf_fortgesetztauf.append("<p class=\"anmerkung\">Anmerkung: Fortgesetzt auf Seite " + atts.getValue("", "fortgesetztauf") + ".</p>");
                }
                xhtmlSeite.append("<p class=\"fortsetzung\">");
                artikelStart = false;
                artikelfortsetzungStart = false;
                break;
            //
            // Nichts Spezielles hier...
            //
            case XmlNames.en_seitenzahl:
                xhtmlSeite.append("<p class=\"seitenzahl\">");
                break;
            case XmlNames.en_seitenkopf:
                xhtmlSeite.append("<h1 class=\"seitenkopf\">");
                break;
            case XmlNames.en_artikelteilueberschrift:
                xhtmlSeite.append("<h4 class=\"artikelteilueberschrift\">");
                break;
            case XmlNames.en_anmerkung:
                xhtmlSeite.append("<p class=\"mittigeanmerkung\">");
                break;
            case XmlNames.en_soloanmerkung:
                xhtmlSeite.append("<p class=\"mittigeanmerkung\">");
                break;
            case XmlNames.en_aufzaehlung:
                if (ulOpen == 0) {
                    xhtmlSeite.append("</p>"); // ul darf nicht in p's stehen!
                }
                xhtmlSeite.append("<ul>");
                ulOpen++;
                break;
            case XmlNames.en_aufzaehlungspunkt:
                xhtmlSeite.append("<li>");
                break;
            case XmlNames.en_tabelle:
                xhtmlSeite.append("</p>"); // Tabellen dürfen nicht nicht p's stehen!
                xhtmlSeite.append("<table>");
                break;
            case XmlNames.en_beschriftung:
                xhtmlSeite.append("<caption>");
                break;
            case XmlNames.en_zeile:
                xhtmlSeite.append("<tr>");
                break;
            case XmlNames.en_zelle:
                xhtmlSeite.append("<td");
                if (atts.getValue("", "hspan") != null) {
                    xhtmlSeite.append(" colspan=\"" + atts.getValue("", "hspan") + "\"");
                }
                if (atts.getValue("", "vspan") != null) {
                    xhtmlSeite.append(" rowspan=\"" + atts.getValue("", "vspan") + "\"");
                }
                if (atts.getValue("", "class") != null) {
                    xhtmlSeite.append(" class=\"" + atts.getValue("", "class") + "\"");
                }
                xhtmlSeite.append(">");
                break;
            //
            // Schlagwörter können das Wort als Attribute haben, oder in dem Tag...
            //
            case XmlNames.en_schlagwort:
                if (atts.getValue("", "wort") != null) {
                    schlagwortAttr = new StringBuffer();
                    schlagwortAttr.append(atts.getValue("", "wort"));
                }
                schlagwortCdata = new StringBuffer();
                schlagwortContent = new StringBuffer();
                break;
            //
            // Nichts Spezielles hier...
            //
            case XmlNames.en_unterschlagwort:
                xhtmlSeite.append("<em class=\"unterschlagwort\">");
                break;
            case XmlNames.en_tief:
                xhtmlSeite.append("<sub class=\"tief\">");
                break;
            case XmlNames.en_hoch:
                xhtmlSeite.append("<sup class=\"hoch\">");
                break;
            case XmlNames.en_chemischeformel:
                xhtmlSeite.append("<span class=\"chemischeformel\">");
                break;
            case XmlNames.en_mathematischeformel:
                xhtmlSeite.append("<span class=\"mathematischeformel\">");
                break;
            //
            // Abbildungen sind komplex, da sie, falls sie Koordinaten haben,
            // das Bild aus dem Faksimile anzeigen, andernfalls ein Link
            // auf das Faksimile bilden.
            //
            case XmlNames.en_abbildung:
                // Titel zusammensetzen...
                imageTitleContent = new StringBuffer();
                imageTitleCdata = new StringBuffer();
                if (atts.getValue("", "artikel") != null) {
                    imageTitleContent.append("[Zum Artikel <q>" + atts.getValue("", "artikel") + "</q>] ");
                }
                imageTitleContent.append(atts.getValue("", "typ"));
                if (atts.getValue("", "nummer") != null) {
                    imageTitleContent.append(" " + atts.getValue("", "nummer"));
                }
                imageTitleContent.append(": ");
                // Mit Koordinaten?
                if (atts.getValue("", "x") != null) {
                    // Ja, Bild anzeigen...
                    imageTag = new StringBuffer();
                    imageTag.append("</p>"); // Bilder dürfen nicht in p's stehen!
                    imageTag.append("<div class=\"float_right\">");
                    imageTag.append("<p class=\"imageborder center\" style=\"width: " + atts.getValue("", "width") + "px;\">");
                    x = atts.getValue("", "x");
                    y = atts.getValue("", "y");
                    w = atts.getValue("", "width");
                    h = atts.getValue("", "height");
                } else {
                // Nein, Link auf Faxsimile...
                }
                break;
            //
            // Nichts Spezielles hier...
            //
            case XmlNames.en_spaltenwechsel:
                xhtmlSeite.append(" ↔ ");
                break;
            case XmlNames.en_editoranmerkung:
                xhtmlSeite.append("<span class=\"editorhint\">(Anmerkung des Editors: ");
                break;
            case XmlNames.en_lautschrift:
                xhtmlSeite.append("<span class=\"lautschrift\">");
                break;
            case XmlNames.en_kleingedruckt:
                xhtmlSeite.append("<span class=\"kleingedruckt\">");
                break;
            case XmlNames.en_fussnote:
                xhtmlSeite.append("<span class=\"fussnote\">");
                break;
            case XmlNames.en_fussnotenzeichen:
                xhtmlSeite.append("<sup class=\"fussnotenzeichen\">");
                break;
            //
            // Eine Referenz kann das Verweiswort im Attribut oder im Tag
            // haben.
            //
            case XmlNames.en_verweis:
                if (atts.getValue("", "referenz") != null) {
                    referenzAttr = new StringBuffer();
                    referenzAttr.append(atts.getValue("", "referenz"));
                }
                referenzCdata = new StringBuffer();
                referenzContent = new StringBuffer();
                werk = atts.getValue("", "werk");
                bandnr = atts.getValue("", "band");
                seitenr = atts.getValue("", "seite");
                aznr = atts.getValue("", "nummer");
                break;
            //
            // Nichts Spzielles hier...
            //
            case XmlNames.en_seitenfuss:
                xhtmlSeite.append("<p class=\"seitenfuss\">");
                break;
            //
            // Hinweise können auch in Bildtiteln oder Verweisen oder
            // Schlagwörtern vorkommen.
            //
            case XmlNames.en_hinweis:
                if (schlagwortContent != null) {
                    schlagwortContent.append("<em class=\"" + atts.getValue("", "typ") + "\">");
                } else if (imageTitleContent != null) {
                    imageTitleContent.append("<em class=\"" + atts.getValue("", "typ") + "\">");
                } else if (referenzContent != null) {
                    referenzContent.append("<em class=\"" + atts.getValue("", "typ") + "\">");
                } else {
                    xhtmlSeite.append("<em class=\"" + atts.getValue("", "typ") + "\">");
                }
                break;
            default:
                throw new SAXException("Unbekannter Element-Name: " + localName);
        }
    }

    /**
     * Callback beim Ende eines Elements.
     *
     * @param namespaceURI Die Namespace-URI.
     * @param localName Der lokale Name.
     * @param qName Der qualifizierte Name.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void endElement(String namespaceURI, String localName, String qName)
            throws SAXException {
        switch (XmlNames.element(localName)) {
            case XmlNames.en_werk:
                break;
            case XmlNames.en_band:
                break;
            case XmlNames.en_teil:
                break;
            case XmlNames.en_seite:
                break;
            case XmlNames.en_artikel:
                artikelStart = false;
                if (artikelPWritten) {
                    xhtmlSeite.append("</p>");
                    artikelPWritten = false;
                }
                if (a_fortgesetztauf != null) {
                    xhtmlSeite.append(a_fortgesetztauf);
                    a_fortgesetztauf = null;
                }

                // falls Bilder drin sind (Floatbox-Div)
                xhtmlSeite.append("</div>");
                break;
            case XmlNames.en_artikelfortsetzung:
                artikelfortsetzungStart = false;
                if (artikelfortsetzungPWritten) {
                    xhtmlSeite.append("</p>");
                    artikelfortsetzungPWritten = false;
                }
                if (af_fortgesetztauf != null) {
                    xhtmlSeite.append(af_fortgesetztauf);
                    af_fortgesetztauf = null;
                }

                // falls Bilder drin sind (Floatbox-Div)
                xhtmlSeite.append("</div>");
                break;
            case XmlNames.en_artikelteil:
                xhtmlSeite.append("</p>");
                if (at_fortgesetztauf != null) {
                    xhtmlSeite.append(at_fortgesetztauf);
                    at_fortgesetztauf = null;
                }
                break;
            case XmlNames.en_artikelteilfortsetzung:
                xhtmlSeite.append("</p>");
                if (atf_fortgesetztauf != null) {
                    xhtmlSeite.append(atf_fortgesetztauf);
                    atf_fortgesetztauf = null;
                }
                break;
            case XmlNames.en_artikelzaehlung:
                xhtmlSeite.append("</p>");
                if (az_fortgesetztauf != null) {
                    xhtmlSeite.append(az_fortgesetztauf);
                    az_fortgesetztauf = null;
                }
                break;
            case XmlNames.en_artikelzaehlungsfortsetzung:
                xhtmlSeite.append("</p>");
                if (azf_fortgesetztauf != null) {
                    xhtmlSeite.append(azf_fortgesetztauf);
                    azf_fortgesetztauf = null;
                }
                break;
            case XmlNames.en_seitenzahl:
                xhtmlSeite.append("</p>");
                break;
            case XmlNames.en_seitenkopf:
                xhtmlSeite.append("</h1>");
                break;
            case XmlNames.en_artikelteilueberschrift:
                xhtmlSeite.append("</h4>");
                break;
            case XmlNames.en_anmerkung:
                xhtmlSeite.append("</p>");
                break;
            case XmlNames.en_soloanmerkung:
                xhtmlSeite.append("</p>");
                break;
            case XmlNames.en_aufzaehlung:
                xhtmlSeite.append("</ul>");
                ulOpen--;
                if (ulOpen == 0) {
                    xhtmlSeite.append("<p class=\"" + lastPClass + "\">"); // ul darf nicht nicht p's stehen!
                }
                break;
            case XmlNames.en_aufzaehlungspunkt:
                xhtmlSeite.append("</li>");
                break;
            case XmlNames.en_tabelle:
                xhtmlSeite.append("</table>");
                xhtmlSeite.append("<p class=\"" + lastPClass + "\">"); // Tabellen dürfen nicht nicht p's stehen!
                break;
            case XmlNames.en_beschriftung:
                xhtmlSeite.append("</caption>");
                break;
            case XmlNames.en_zeile:
                xhtmlSeite.append("</tr>");
                break;
            case XmlNames.en_zelle:
                xhtmlSeite.append("</td>");
                break;
            case XmlNames.en_schlagwort:
                String schlagwort;
                if (schlagwortAttr != null) {
                    schlagwort = TextUtil.encodeSchlagwort(schlagwortAttr);
                } else {
                    schlagwort = TextUtil.encodeSchlagwort(schlagwortCdata);
                }
                if (usedAnchors.get(schlagwort) == null) {
                    xhtmlSeite.append("<a id=\"" + schlagwort + "\" name=\"" + schlagwort + "\"></a>");
                    usedAnchors.put(schlagwort, schlagwort);
                }
                xhtmlSeite.append("<em class=\"schlagwort\">" + schlagwortContent + "</em>");
                schlagwortAttr = null;
                schlagwortCdata = null;
                schlagwortContent = null;
                break;
            case XmlNames.en_unterschlagwort:
                xhtmlSeite.append("</em>");
                break;
            case XmlNames.en_tief:
                xhtmlSeite.append("</sub>");
                break;
            case XmlNames.en_hoch:
                xhtmlSeite.append("</sup>");
                break;
            case XmlNames.en_chemischeformel:
                xhtmlSeite.append("</span>");
                break;
            case XmlNames.en_mathematischeformel:
                xhtmlSeite.append("</span>");
                break;
            case XmlNames.en_abbildung:
                if (imageTag != null) {
                    // Bild anzeigen...
                    xhtmlSeite.append(imageTag);
                    xhtmlSeite.append("<img src=\"/retrobib/faksimile_" + faksimileid + ".jpeg?x=" + x + "&amp;y=" + y + "&amp;w=" + w + "&amp;h=" + h + "\" width=\"" + w + "\" height=\"" + h + "\" title=\"" + imageTitleCdata + "\" alt=\"" + imageTitleCdata + "\" />");
                    xhtmlSeite.append("<br />" + imageTitleContent + "</p>");
                    xhtmlSeite.append("</div>");
                    xhtmlSeite.append("<p class=\"" + lastPClass + "\">"); // Bilder dürfen nicht in p's stehen!
                } else {
                    // nur Verweis...
                    xhtmlSeite.append("<br /><em class=\"abbildung\">");
                    xhtmlSeite.append("<a href=\"/retrobib/faksimile_" + faksimileid + ".jpeg\">");
                    xhtmlSeite.append(imageTitleContent);
                    xhtmlSeite.append("</a>");
                    xhtmlSeite.append("</em>");
                }

                imageTitleContent = null;
                imageTitleCdata = null;
                imageTag = null;
                x = null;
                y = null;
                w = null;
                h = null;
                break;
            case XmlNames.en_spaltenwechsel:
                break;
            case XmlNames.en_editoranmerkung:
                xhtmlSeite.append(")</span>");
                break;
            case XmlNames.en_lautschrift:
                xhtmlSeite.append("</span>");
                break;
            case XmlNames.en_kleingedruckt:
                xhtmlSeite.append("</span>");
                break;
            case XmlNames.en_fussnote:
                xhtmlSeite.append("</span><br />");
                break;
            case XmlNames.en_fussnotenzeichen:
                xhtmlSeite.append("</sup>");
                break;
            case XmlNames.en_verweis:
                xhtmlSeite.append("<a class=\"verweis\" href=\"/retrobib/schlagwort.html?wort=");
                if (referenzAttr != null) {
                    xhtmlSeite.append(referenzAttr);
                } else {
                    xhtmlSeite.append(referenzCdata);
                }
                if (werk != null) {
                    xhtmlSeite.append("&amp;werk=" + werk);
                }
                if (bandnr != null) {
                    xhtmlSeite.append("&amp;bandnr=" + bandnr);
                }
                if (seitenr != null) {
                    xhtmlSeite.append("&amp;seitenr=" + seitenr);
                }
                if (aznr != null) {
                    xhtmlSeite.append("&amp;aznr=" + aznr);
                }
                xhtmlSeite.append("\">");
                xhtmlSeite.append(referenzContent);
                xhtmlSeite.append("</a>");

                referenzAttr = null;
                referenzCdata = null;
                referenzContent = null;
                werk = null;
                bandnr = null;
                seitenr = null;
                aznr = null;
                break;
            case XmlNames.en_seitenfuss:
                xhtmlSeite.append("</p>");
                break;
            case XmlNames.en_hinweis:
                if (schlagwortContent != null) {
                    schlagwortContent.append("</em>");
                } else if (imageTitleContent != null) {
                    imageTitleContent.append("</em>");
                } else if (referenzContent != null) {
                    referenzContent.append("</em>");
                } else {
                    xhtmlSeite.append("</em>");
                }
                break;
            default:
                throw new SAXException("Unbekannter Element-Name: " + localName);
        }
    }

    /**
     * Callback wenn Character-Daten auftreten.
     *
     * @param ch Die Characters als Array.
     * @param start Der Startpunkt der Zeichenfolge <i>innerhalb</i> des Arrays.
     * @param length Die Länge der Zeichenfolge <i>ab dem Startpunkt</i>.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void characters(char[] ch, int start, int length)
            throws SAXException {
        String content = new String(ch, start, length);

        if (content.length() != 0) {
            if (content.trim().length() != 0) {
                if (artikelStart && !artikelfortsetzungStart) {
                    // CDATA aber noch kein p!
                    xhtmlSeite.append("<p class=\"artikel\">");
                    artikelPWritten = true;
                    artikelStart = false;
                }
                if (artikelfortsetzungStart) {
                    // CDATA aber noch kein p!
                    xhtmlSeite.append("<p class=\"fortsetzung\">");
                    artikelfortsetzungPWritten = true;
                    artikelfortsetzungStart = false;
                }
            }

            if (schlagwortContent != null) {
                schlagwortContent.append(content);
                schlagwortCdata.append(content);
            } else if (imageTitleContent != null) {
                imageTitleContent.append(content);
                imageTitleCdata.append(content);
            } else if (referenzContent != null) {
                referenzContent.append(content);
                referenzCdata.append(content);
            } else {
                xhtmlSeite.append(TextUtil.encodeXML(content));
                if (content.trim().length() != 0) {
                    lastPClass = "fortsetzung";
                }
            }
        }
    }

    /**
     * Callback wenn ignorierbare Whitespaces auftreten.
     *
     * @param ch Die Characters als Array.
     * @param start Der Startpunkt der Zeichenfolge <i>innerhalb</i> des Arrays.
     * @param length Die Länge der Zeichenfolge <i>ab dem Startpunkt</i>.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void ignorableWhitespace(char[] ch, int start, int length)
            throws SAXException {
    }

    /**
     * Callback bei einer Processing-Instruction.
     *
     * @param target Das Ziel.
     * @param data Die Daten.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void processingInstruction(String target, String data)
            throws SAXException {
    }

    /**
     * Callback wenn eine Entität übersprungen wird.
     *
     * @param name Der Name der Entität.
     * @throws org.xml.sax.SAXException Bei einem Fehler.
     */
    public void skippedEntity(String name)
            throws SAXException {
    }
}
