/*
 * XmlNames.java
 *
 * Created on 5. Dezember 2007, 08:05
 *
 */
package de.retrobib.xml;

import java.util.HashMap;
import org.apache.log4j.Logger;

/**
 * XmlNames hält die Namen der Elemente/Attribute für 
 * Überprüfungen etc.
 *
 * @author caschoff
 * @version 1.0
 */
public class XmlNames {

    /**
     * <b>Jede</b> Klasse hat ihren Logger.
     */
    private static final Logger logger = Logger.getLogger(XmlNames.class);
    /*
     * Elementnamen-Konstanten.
     */
    public static final int en_werk = 0;
    public static final int en_band = 1;
    public static final int en_teil = 2;
    public static final int en_seite = 3;
    public static final int en_seitenzahl = 4;
    public static final int en_seitenkopf = 5;
    public static final int en_artikel = 6;
    public static final int en_artikelfortsetzung = 7;
    public static final int en_artikelteil = 8;
    public static final int en_artikelteilfortsetzung = 9;
    public static final int en_artikelteilueberschrift = 10;
    public static final int en_anmerkung = 11;
    public static final int en_artikelzaehlung = 12;
    public static final int en_artikelzaehlungsfortsetzung = 13;
    public static final int en_aufzaehlung = 14;
    public static final int en_aufzaehlungspunkt = 15;
    public static final int en_tabelle = 16;
    public static final int en_beschriftung = 17;
    public static final int en_zeile = 18;
    public static final int en_zelle = 19;
    public static final int en_schlagwort = 20;
    public static final int en_unterschlagwort = 21;
    public static final int en_tief = 22;
    public static final int en_hoch = 23;
    public static final int en_chemischeformel = 24;
    public static final int en_mathematischeformel = 25;
    public static final int en_abbildung = 26;
    public static final int en_spaltenwechsel = 27;
    public static final int en_editoranmerkung = 28;
    public static final int en_lautschrift = 29;
    public static final int en_kleingedruckt = 30;
    public static final int en_fussnote = 31;
    public static final int en_fussnotenzeichen = 32;
    public static final int en_verweis = 33;
    public static final int en_seitenfuss = 34;
    public static final int en_hinweis = 35;
    public static final int en_soloanmerkung = 36;
    /*
     * Attributnamen-Konstanten.
     */
    public static final int an_kurzname = 0;
    public static final int an_nummer = 1;
    public static final int an_name = 2;
    public static final int an_von = 3;
    public static final int an_bis = 4;
    public static final int an_fortgesetztauf = 5;
    public static final int an_fortgesetztvon = 6;
    public static final int an_hspan = 7;
    public static final int an_vspan = 8;
    public static final int an_class = 9;
    public static final int an_kategorie = 10;
    public static final int an_unterkategorie = 11;
    public static final int an_artikel = 12;
    public static final int an_x = 13;
    public static final int an_y = 14;
    public static final int an_width = 15;
    public static final int an_height = 16;
    public static final int an_referenz = 17;
    public static final int an_seite = 18;
    public static final int an_typ = 19;
    public static final int an_wort = 20;
    /** HashMap für das schnelle Auffinden von Elemetnamen. */
    private static HashMap<String, Integer> elementNamen = null;
    /** HashMap für das schnelle Auffinden von Attributnamen. */
    private static HashMap<String, Integer> attributNamen = null;

    static {
        elementNamen = new HashMap<String, Integer>();
        attributNamen = new HashMap<String, Integer>();

        //
        // Aufbauen des Element-Hashs.
        //
        elementNamen.put("werk", en_werk);
        elementNamen.put("band", en_band);
        elementNamen.put("teil", en_teil);
        elementNamen.put("seite", en_seite);
        elementNamen.put("seitenzahl", en_seitenzahl);
        elementNamen.put("seitenkopf", en_seitenkopf);
        elementNamen.put("artikel", en_artikel);
        elementNamen.put("artikelfortsetzung", en_artikelfortsetzung);
        elementNamen.put("artikelteil", en_artikelteil);
        elementNamen.put("artikelteilfortsetzung", en_artikelteilfortsetzung);
        elementNamen.put("artikelteilueberschrift", en_artikelteilueberschrift);
        elementNamen.put("anmerkung", en_anmerkung);
        elementNamen.put("artikelzaehlung", en_artikelzaehlung);
        elementNamen.put("artikelzaehlungsfortsetzung", en_artikelzaehlungsfortsetzung);
        elementNamen.put("aufzaehlung", en_aufzaehlung);
        elementNamen.put("aufzaehlungspunkt", en_aufzaehlungspunkt);
        elementNamen.put("tabelle", en_tabelle);
        elementNamen.put("beschriftung", en_beschriftung);
        elementNamen.put("zeile", en_zeile);
        elementNamen.put("zelle", en_zelle);
        elementNamen.put("schlagwort", en_schlagwort);
        elementNamen.put("unterschlagwort", en_unterschlagwort);
        elementNamen.put("tief", en_tief);
        elementNamen.put("hoch", en_hoch);
        elementNamen.put("chemischeformel", en_chemischeformel);
        elementNamen.put("mathematischeformel", en_mathematischeformel);
        elementNamen.put("abbildung", en_abbildung);
        elementNamen.put("spaltenwechsel", en_spaltenwechsel);
        elementNamen.put("editoranmerkung", en_editoranmerkung);
        elementNamen.put("lautschrift", en_lautschrift);
        elementNamen.put("kleingedruckt", en_kleingedruckt);
        elementNamen.put("fussnote", en_fussnote);
        elementNamen.put("fussnotenzeichen", en_fussnotenzeichen);
        elementNamen.put("verweis", en_verweis);
        elementNamen.put("seitenfuss", en_seitenfuss);
        elementNamen.put("hinweis", en_hinweis);
        elementNamen.put("soloanmerkung", en_soloanmerkung);

        //
        // Aufbauen des Attribute-Hashes.
        //
        attributNamen.put("kurzname", an_kurzname);
        attributNamen.put("nummer", an_nummer);
        attributNamen.put("name", an_name);
        attributNamen.put("von", an_von);
        attributNamen.put("bis", an_bis);
        attributNamen.put("fortgesetztauf", an_fortgesetztauf);
        attributNamen.put("fortgesetztvon", an_fortgesetztvon);
        attributNamen.put("hspan", an_hspan);
        attributNamen.put("vspan", an_vspan);
        attributNamen.put("class", an_class);
        attributNamen.put("kategorie", an_kategorie);
        attributNamen.put("unterkategorie", an_unterkategorie);
        attributNamen.put("artikel", an_artikel);
        attributNamen.put("x", an_x);
        attributNamen.put("y", an_y);
        attributNamen.put("width", an_width);
        attributNamen.put("height", an_height);
        attributNamen.put("referenz", an_referenz);
        attributNamen.put("seite", an_seite);
        attributNamen.put("typ", an_typ);
        attributNamen.put("wort", an_wort);
    }

    /** Creates a new instance of XmlNames */
    private XmlNames() {
    }

    /**
     * Eine Konstante für einen Namen bekommen.
     *
     * @param elementName Der Name des Elements.
     * @return Die Konstante für das Element.
     */
    public static int element(String elementName) {
        return elementNamen.get(elementName);
    }

    /**
     * Eine Konstante für einen Namen bekommen.
     *
     * @param attributName Der Name des Attributs.
     * @return Die Konstante für das Attribut.
     */
    public static int attribut(String attributName) {
        return attributNamen.get(attributName);
    }
}
